/** 
 * @file microdia-dev.c
 * @author Nicolas VIVIEN
 * @date 2008-02-01
 * @version v0.0.0
 *
 * @brief Driver for Microdia USB video camera
 *
 * @note Copyright (C) Nicolas VIVIEN
 *
 * @par Licences
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include <linux/errno.h>
#include <linux/string.h>
#include <stdarg.h>

#include "microdia.h"
#include "sn9c20x.h"
#include "ov7670.h"
#include "ov965x.h"
#include "mt9vx11.h"


int dev_microdia_init_camera(struct usb_microdia *dev)
{
	return 0;
}

int dev_microdia_camera_on(struct usb_microdia *dev)
{
	int ret = -1;
	struct usb_device *udev = dev->udev;

	ret = usb_set_interface(udev, 0, 8);

	if (ret < 0)
		UDIA_ERROR("usb_set_interface failed !\n");

	return ret;
}

int dev_microdia_camera_off(struct usb_microdia *dev)
{
	int ret = -1;
	struct usb_device *udev = dev->udev;

	ret = usb_set_interface(udev, 0, 0);

	if (ret < 0)
		UDIA_ERROR("usb_set_interface failed !\n");

	return 0;
}

int dev_microdia_camera_settings(struct usb_microdia *dev)
{
	dev_microdia_camera_set_contrast(dev);
	dev_microdia_camera_set_brightness(dev);
	dev_microdia_camera_set_gamma(dev);
	dev_microdia_camera_set_exposure(dev);
	dev_microdia_camera_set_hvflip(dev);
	dev_microdia_camera_set_sharpness(dev);
	dev_microdia_camera_set_auto_exposure(dev);
	dev_microdia_camera_set_auto_whitebalance(dev);
	return 0;
}

int dev_microdia_camera_set_contrast(struct usb_microdia *dev)
{
	int ret = -ENODEV;
	if(dev && dev->set_contrast)
		ret = dev->set_contrast(dev);
	return ret;
}

int dev_microdia_camera_set_brightness(struct usb_microdia *dev)
{
	int ret = -ENODEV;
	if(dev && dev->set_brightness)
		ret = dev->set_brightness(dev);
	return ret;
}

int dev_microdia_camera_set_gamma(struct usb_microdia *dev)
{
	int ret = -ENODEV;
	if(dev && dev->set_gamma)
		ret = dev->set_gamma(dev);
	return ret;
}

int dev_microdia_camera_set_exposure(struct usb_microdia *dev)
{
	if (dev->set_exposure != NULL) {
		return dev->set_exposure(dev);
	}
	return 0;
}

int dev_microdia_camera_set_hvflip(struct usb_microdia *dev)
{
	int ret = -ENODEV;
	if(dev && dev->set_hvflip)
		ret = dev->set_hvflip(dev);

	return ret;
}

int dev_microdia_camera_set_sharpness(struct usb_microdia *dev)
{
	int ret = -ENODEV;
	if(dev && dev->set_sharpness)
		ret = dev->set_sharpness(dev);

	return ret;
}

int dev_microdia_camera_set_auto_exposure(struct usb_microdia *dev)
{
	int ret = -ENODEV;
	if(dev && dev->set_auto_exposure)
		ret = dev->set_auto_exposure(dev);

	return ret;
}

int dev_microdia_camera_set_auto_whitebalance(struct usb_microdia *dev)
{
	int ret = -ENODEV;
	if(dev && dev->set_auto_whitebalance)
		ret = dev->set_auto_whitebalance(dev);

	return ret;
}


/** 
 * @brief From UsbSnoop-plugin-parsed.log
 * 
 * @param dev 
 * 
 * @returns 
 */
int dev_microdia_initialize_device(struct usb_microdia *dev)
{
	int ret = -ENODEV;

	if(dev && dev->initialize)
		ret = dev->initialize(dev);

	return ret;
}

/**
 * @brief From init.sequence.log
 *
 * @param dev
 *
 * @returns 0
 *
 * @author GWater
 *
 * For SN9C201 with SOI968.
 * Tests showed this function is actually unnecessary.
 * Nevertheless the reads may give us valuable values we don't know about yet.
 */
int microdia_624e_initialize(struct usb_microdia *dev)
{
	__u8 buf;

	usb_microdia_control_read(dev, 0x130d, &buf, 1);
	usb_microdia_control_read(dev, 0x1040, &buf, 1);

	return 0;
}

/** 
 * @brief From UsbSnoop-plugin-parsed.log
 * 
 * @param dev 
 * 
 * @returns 
 */
int microdia_624f_initialize(struct usb_microdia *dev)
{
	/* Do nothing here */
	return 0;
}

/**
 * @brief From UsbSnoop-plugin-parsed.log
 *
 * @param dev
 *
 * @returns
 */
int microdia_6260_initialize(struct usb_microdia *dev)
{
	__u8 buf;

	usb_microdia_control_read(dev, 0x130d, &buf, 1);
        usb_microdia_control_read(dev, 0x1040, &buf, 1);
        buf = 0x00;
        usb_microdia_control_write(dev, 0x10c8, &buf, 1);
        usb_microdia_control_read(dev, 0x100a, &buf, 1);
        buf = 0x04;
        usb_microdia_control_write(dev, 0x100a, &buf, 1);
        usb_microdia_control_read(dev, 0x100b, &buf, 1);
        buf = 0x04;
        usb_microdia_control_write(dev, 0x100b, &buf, 1);
        usb_microdia_control_read(dev, 0x1001, &buf, 1);
        buf = 0xc7;
        usb_microdia_control_write(dev, 0x1001, &buf, 1);
        usb_microdia_control_read(dev, 0x1040, &buf, 1);
        buf = 0x00;
        usb_microdia_control_write(dev, 0x1040, &buf, 1);
        usb_microdia_control_read(dev, 0x1045, &buf, 1);
        buf = 0x04;
        usb_microdia_control_write(dev, 0x1045, &buf, 1);
        usb_microdia_control_read(dev, 0x1046, &buf, 1);
        buf = 0x10;
        usb_microdia_control_write(dev, 0x1046, &buf, 1);
        buf = 0x14;
        usb_microdia_control_write(dev, 0x1045, &buf, 1);
        buf = 0x01;
        usb_microdia_control_write(dev, 0x1040, &buf, 1);
        buf = 0x80;
        usb_microdia_control_write(dev, 0x1020, &buf, 1);

	return 0;
}

/**
 * @brief From 6 init-logs
 *
 * @param dev
 *
 * @returns 0 (ok) or -1 (error)
 *
 * @author Comer352l
 *
 * Windows driver versions: 5.5.8.12 and 5.7.23.000
 * Windows versions: 2000 and XP
 * USB-Logger: SniffUSB 1.8 and 2.0, USBTrace 2.0, SnoopyPro
 *
 * Comments behind read oprations contain HEX-values that appeared in the logs
 */
int microdia_6270_initialize(struct usb_microdia *dev)
{
	//int actual;
	int ret = 0;
	int retSP = 0;
	int retASS = 0;
	__u8 buf[8];
	__u8 buf_0395[1];
	__u8 buf_1000[5];
	__u8 buf_1060[2];
	__u8 buf_11b8[1];

	ret += usb_microdia_control_read(dev, 0x130d, buf, 1);		// URB 5	0x81,0x85,0x04,0x8b,0x02,...

	ret += usb_microdia_control_read(dev, 0x1040, buf, 1);		// URB 6	0xe4,0xc4,0xa4,0x44,...

	/*
	 * THE OLD DRIVER 5.5.8.12 DOES AN ADDITIONAL WRITE HERE:
	 * buf[0]=0x00;
	 * usb_microdia_control_write(dev, 0x10c8, buf, 1);
	*/

	ret += usb_microdia_control_read(dev, 0x118a, buf, 1);		// URB 7	0x05,0x7b,0x01,0x04,...

	ret += usb_microdia_control_read(dev, 0x0395, buf_0395, 1);	// URB 8	0xc2,0x82,0x38,0x98,0x04,...
	// => this value will be rewritten to 0x118a and 0x0395 later (URBs 81+82)

	buf[0] = 0x05;									
	ret += usb_microdia_control_write(dev, 0x118a, buf, 1);		// URB 9

	ret += usb_microdia_control_write(dev, 0x0395, buf, 1);		// URB 10

	ret += usb_microdia_control_read(dev, 0x11b8, buf_11b8, 1);	// URB 11	0xa0,0x00,0x24,...
	// => this value will be rewritten to 0x11b8 later (URB 83)

	buf[0] = 0x10;
	ret += usb_microdia_control_write(dev, 0x11b8, buf, 1);		// URB 12

	ret += usb_microdia_control_read(dev, 0x1000, buf_1000, 5);	// URB 13	0x30 0xCF 0x00 0x03 (0x02 or 0x00)
	// => these values will be rewritten to 0x1000 later (URB 84)

	buf[0] = 0x70; buf[1] = 0x00; buf[2] = 0x18; buf[3] = 0x00; buf[4] = 0x00;
	ret += usb_microdia_control_write(dev, 0x1000, buf, 5);		// URB 14

	ret += usb_microdia_control_read(dev, 0x1060, buf_1060, 2);	// URB 15	0x08 0x00 (always  ???)
	// => this value will be rewritten to 0x1060 later (URB 85)

	buf[0] = 0x00; buf[1] = 0x03;
	ret += usb_microdia_control_write(dev, 0x1060, buf, 2);		// URB 16

	// initialize I2C registers to avoid getting no ACK at first I2C operation:
	ret += sn9c20x_initialize_i2c(dev, SN9C20X_I2C_2WIRE, MT9V011_I2C_SLAVE_ADDRESS);

	// enable I2C communication (bit 0)
	buf[0] = 0x21;	// THE OLD DRIVER 5.5.8.12 WRITES 0x5d
	ret += usb_microdia_control_write(dev, 0x10c8, buf, 1);		// URB 17

	// probe sensor model:
	retSP = mt9vx11_sensor_probe(dev);	// substitutes URBs 18-48
	if ((retSP == 0) && (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS))
		// switch (back) to IFP address space:
		retASS = mt9v111_select_address_space(dev, MT9V111_ADDRESSSPACE_IFP);

	ret += usb_microdia_control_write(dev, 0x118a, buf_0395, 1);	// URB 81 => write value read with URB 8

	ret += usb_microdia_control_write(dev, 0x0395, buf_0395, 1);	// URB 82 => write value read with URB 8

	ret += usb_microdia_control_write(dev, 0x11b8, buf_11b8, 1);	// URB 83 => write value read with URB 11

	ret += usb_microdia_control_write(dev, 0x1000, buf_1000, 5);	// URB 84 => write value read with URB 13

	ret += usb_microdia_control_write(dev, 0x1060, buf_1060, 2);	// URB 85 => write value read with URB 15; (always 08 00 ???)

	// USB-interrupt-message:	doesn't work yet (returns error -110)
	//usb_interrupt_msg(dev->udev, usb_rcvintpipe(dev->udev, dev->int_in_endpointAddr), dev->int_in_buffer, dev->int_in_size, &actual, 1000);	// URB 86

	/*
	 * THE OLD DRIVER 5.5.8.12 DOES AN ADDITIONAL WRITE HERE:
	 * wbuf[0]=0x80;
	 * usb_microdia_control_write(dev, 0x1020, wbuf, 1);
	*/

	if ((retSP != 0) || (retASS != 0) || (ret < 33))
	{
		if (ret < 33)
			UDIA_INFO("One ore more errors occured during initialization !\n");
		return -1;
	}
	else
		return 0;
}

/** 
 * @brief From UsbSnoop-plugin-parsed.log
 * 
 * @param dev 
 * 
 * @returns 
 */
int microdia_627b_initialize(struct usb_microdia *dev)
{
	//int actual;
	int ret;
	int retI2C;
	__u16 reg;
	__u8 buf[32];
	__u8 reg0395[1];
	__u8 reg11b8[1];
	__u8 reg1000[5];
	__u8 reg1060[2];	

	reg = 0x130d;
	ret = usb_microdia_control_read(dev, reg, buf, 1);				// URB 5
	if(ret < 0) goto err;
	reg = 0x1040;
	ret = usb_microdia_control_read(dev, reg, buf, 1);				// URB 6
	if(ret < 0) goto err;
	
	reg = 0x118a;
	ret = usb_microdia_control_read(dev, reg, buf, 1);				// URB 7
	if(ret < 0) goto err;
	reg = 0x0395;
	ret = usb_microdia_control_read(dev, reg, reg0395, 1);				// URB 8
	if(ret < 0) goto err;

	reg = 0x118a;
	buf[0] = 0x05;
	ret = usb_microdia_control_write(dev, reg, buf, 1);				// URB 9
	if(ret < 0) goto err;
	reg = 0x0395;
	ret = usb_microdia_control_write(dev, reg, buf, 1);				// URB 10
	if(ret < 0) goto err;
	
	reg = 0x11b8;
	ret = usb_microdia_control_read(dev, reg, reg11b8, 1);				// URB 11
	if(ret < 0) goto err;
	
	reg = 0x11b8;
	buf[0] = 0x10;
	ret = usb_microdia_control_write(dev, reg, buf, 1);				// URB 12
	if(ret < 0) goto err;
	reg = 0x1000;
	ret = usb_microdia_control_read(dev, reg, reg1000, 5);				// URB 13
	if(ret < 0) goto err;

	reg = 0x1000;
	buf[0] = 0x70; buf[1] = 0x00; buf[2] = 0x18; buf[3] = 0x00; buf[4] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 5);				// URB 14
	if(ret < 0) goto err;

	reg = 0x1060;
	ret = usb_microdia_control_read(dev, reg, reg1060, 2);				// URB 15
	if(ret < 0) goto err;

	reg = 0x1060;
	buf[0] = 0x00; buf[1] = 0x03;
	ret = usb_microdia_control_write(dev, reg, buf, 2);				// URB 16
	if(ret < 0) goto err;

	reg = 0x10c8;
	buf[0] = 0x21;
	ret = usb_microdia_control_write(dev, reg, buf, 1);				// URB 17
	if(ret < 0) goto err;

	reg = 0x0a;
	retI2C = sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);			// URBs 18-29
	if (retI2C < 0)
	{
		// 2nd try...
		reg = 0x10c8;
		buf[0] = 0x21;
		ret = usb_microdia_control_write(dev, reg, buf, 1);			// URB 30
		if(ret < 0) goto err;

		reg = 0x0a;
		retI2C = sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);			// URBs 31-35
	}
	if(retI2C < 0)
		UDIA_ERROR("Read of sensor register 0x0a failed.");
	
	reg = 0x118a;
	buf[0] = 0xa6;
	ret = usb_microdia_control_write(dev, reg, reg0395, 1);				// URB 36
	if(ret < 0) goto err;
	reg = 0x0395;
	ret = usb_microdia_control_write(dev, reg, reg0395, 1);				// URB 37
	if(ret < 0) goto err;
	
	reg = 0x11b8;
	buf[0] = 0x06;
	ret = usb_microdia_control_write(dev, reg, reg11b8, 1); 			// URB 38
	if(ret < 0) goto err;
	
	reg = 0x1000;
	buf[0] = 0x30; buf[1] = 0xcf; buf[2] = 0x00; buf[3] = 0x03; buf[4] = 0x02;
	ret = usb_microdia_control_write(dev, reg, reg1000, 5);				// URB 39
	if(ret < 0) goto err;
	
	reg = 0x1060;
	buf[0] = 0x08; buf[1] = 0x00;
	ret = usb_microdia_control_write(dev, reg, reg1060, 2);				// URB 40
	if(ret < 0) goto err;
	
	//ret = usb_interrupt_msg(dev->udev, 0x00000083, buf, 0x0000001, &actual, 1000);	// URB 41
	return ret;
err:
	UDIA_ERROR("r/w in bridge register %x failed (%d)!\n", reg, ret);
	return ret;
}

/**
 * @brief From init-start-stop-log (made by Víctor M. Hernández Rocamora)
 *
 * @param dev
 *
 * @returns 0 (ok) or -1 (error)
 *
 * @author Comer352l
 *
 * USB-Logger: SniffUSB 2.0
 *
 * Comments behind read oprations contain HEX-values that appeared in the log
 */
int microdia_627f_initialize(struct usb_microdia *dev)
{
	__u8 buf[1];
	int ret = 0;

	ret += usb_microdia_control_read(dev, 0x130d, buf, 1);		// URB 5	0x81

	ret += usb_microdia_control_read(dev, 0x1040, buf, 1);		// URB 6	0xe4

	buf[0] = 0x00;
	ret += usb_microdia_control_write(dev, 0x10c8, buf, 1);		// URB 7

	buf[0] = 0x80;
	ret += usb_microdia_control_write(dev, 0x1020, buf, 1);		// URB 8

	ret += usb_microdia_control_read(dev, 0x1045, buf, 1);		// URB 9	0x0f

	buf[0] = 0x1f;
	ret += usb_microdia_control_write(dev, 0x1045, buf, 1);		// URB 10

	ret += usb_microdia_control_read(dev, 0x1045, buf, 1);		// URB 11	0x1f

	buf[0] = 0x0f;
	ret += usb_microdia_control_write(dev, 0x1045, buf, 1);		// URB 12

	ret += usb_microdia_control_read(dev, 0x1045, buf, 1);		// URB 13	0x0f

	buf[0] = 0x1f;
	ret += usb_microdia_control_write(dev, 0x1045, buf, 1);		// URB 14

	if (ret < 10)
	{
		UDIA_INFO("One ore more errors occured during initialization !\n");
		return -1;
	}
	return 0;
}

/** 
 * @brief From UsbSnoop-plugin-parsed.log
 * 
 * @param dev 
 * 
 * @returns 0 (OK) or <0 (Error)
 *
 * @author Vincent, Kuzja
 */
int microdia_6288_initialize(struct usb_microdia *dev)
{

	int ret;
	__u16 reg;
	__u8 buf[2];

	reg = 0x130d ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1040 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	if (ret < 0)
		return ret;
	else
		return 0;
err:
	UDIA_ERROR("r/w in Register %x failed (%d)!\n", reg, ret);
	return ret;
}

/** 
 * @brief From UsbSnoop-plugin-parsed.log
 * 
 * @param dev 
 * 
 * @returns 0 if all OK
 *
 * Function to initialize Microdia 6128 webcam
 * Bridge SN9C325 + OM6802 CMOS sensor
 * note: comments are my observations so they could be wrong
 */
int microdia_6128_initialize(struct usb_microdia *dev)
{
	int ret = 0;
	__u8 buf[2];

	UDIA_INFO("Initializing camera...\n");
	
	buf[0] = 0x01;
	ret += usb_microdia_control_write(dev, 0xf1, buf, 1);
	ret += usb_microdia_control_read(dev, 0x00, buf, 1);		// read reg 0x00 must write 0x00 in buf[0]
	ret += usb_microdia_control_write(dev, 0xf1, buf, 1);
	ret += usb_microdia_control_read(dev, 0x00, buf, 1);		// get cam ID, must read 0x12
	
	if(buf[0] != 0x12){
		UDIA_ERROR("Microdia 6128 is not connected or working !!!\n");
		return -ENODEV;
	}

	buf[0] = 0x29; buf[1] = 0x70;		// Configure GPIO 
	ret += usb_microdia_control_write(dev, 0x01, buf, 2);

	buf[0] = 0x01;
	ret += usb_microdia_control_write(dev, 0xf1, buf, 1);		// Maybe wakeup

	buf[0] = 0x0e;
	ret += usb_microdia_control_write(dev, 0x01, buf, 1);		// Command to initialize cam

	if(ret != 8)
		UDIA_DEBUG("Some read or writes  were not successful in initialization !\n");

	/* Device is initialized and ready */
	UDIA_INFO("Microdia USB2.0 Camera is ready\n");

	return 0;
}


int dev_microdia_start_stream(struct usb_microdia *dev)
{
	int ret = -ENODEV;
	if(dev && dev->start_stream)
		ret = dev->start_stream(dev);

	return ret;
}

/**
 * @brief From start.htm
 *
 * @param dev
 *
 * @returns 0
 *
 * @author GWater
 *
 * For SN9C201 with MI1310.
 * This function has not been tested yet.
 */
int microdia_6242_start_stream(struct usb_microdia *dev)
{
	int i = -1;
	__u8 buf[48];

	__u8 urb10[24] = {
		0x01, 0x18, 0x00, 0x30, 0x00, 0x09, 0x00, 0xed,
		0x0f, 0xda, 0x0f, 0x3a, 0x00, 0x3a, 0x00, 0xd0,
		0x0f, 0xf7, 0x0f, 0x00, 0x00, 0x00, 0x60, 0x0a};

	__u8 urb13[11] = {
		0x02, 0x00, 0x02, 0x00, 0x50, 0x78, 0x00, 0x00,
		0x07, 0xc0, 0x0a};

	__u8 urb14[22] = {
		0x00, 0x20, 0x20, 0x20, 0x20, 0x00, 0x24, 0x3b,
		0x4f, 0x61, 0x71, 0x80, 0x8f, 0x9d, 0xab, 0xb8,
		0xc4, 0xd1, 0xdd, 0xe9, 0xf4, 0xff};

	__u8 urb119[17] = {
		0x08, 0x38, 0x58, 0x6f, 0x7f, 0x8d, 0x9a, 0xa6,
		0xb1, 0xbb, 0xc5, 0xce, 0xd7, 0xdf, 0xe8, 0xef,
		0xf7};

	__u8 urb120[21] = {
		0x16, 0x00, 0x2b, 0x00, 0x08, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
		0x00, 0x00, 0x00, 0x00, 0x00};

	__u8 qtable1[64] = {
		0x0d, 0x08, 0x08, 0x0d, 0x08, 0x08, 0x0d, 0x0d,
		0x0d, 0x0d, 0x11, 0x0d, 0x0d, 0x11, 0x15, 0x21,
		0x15, 0x15, 0x11, 0x11, 0x15, 0x2a, 0x1d, 0x1d,
		0x19, 0x21, 0x32, 0x2a, 0x32, 0x32, 0x2e, 0x2a,
		0x2e, 0x2e, 0x36, 0x3a, 0x4b, 0x43, 0x36, 0x3a,
		0x47, 0x3a, 0x2e, 0x2e, 0x43, 0x5c, 0x43, 0x47,
		0x4f, 0x54, 0x58, 0x58, 0x58, 0x32, 0x3f, 0x60,
		0x64, 0x5c, 0x54, 0x64, 0x4b, 0x54, 0x58, 0x54};

	__u8 qtable2[64] = {
		0x0d, 0x11, 0x11, 0x15, 0x11, 0x15, 0x26, 0x15,
		0x15, 0x26, 0x54, 0x36, 0x2e, 0x36, 0x54, 0x54,
		0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54,
		0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54,
		0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54,
		0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54,
		0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54,
		0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54, 0x54};

	__u8 first10c0[7][3] = {
		{0x0d, 0x00, 0x08},
		{0x0d, 0x00, 0x09},
		{0x0d, 0x00, 0x08},
		{0xf0, 0x00, 0x01},
		{0x3a, 0x73, 0x00},
		{0x06, 0x30, 0x8c},
		{0xf0, 0x00, 0x00}};

	__u8 second10c0a[2][5] = {
		{0x01, 0x00, 0x0e, 0x00, 0x14},
		{0x03, 0x03, 0xc4, 0x05, 0x14}};
	__u8 second10c0b[3] =
		{0xc8, 0x00, 0x03};

	__u8 third10c0[4][3] = {
		{0x0a, 0x00, 0x01},
		{0x06, 0x00, 0x29},
		{0x05, 0x00, 0x72},
		{0x20, 0x00, 0x00}};

	__u8 fourth10c0a[3][3] = {
		{0x20, 0x00, 0x00},
		{0x09, 0x01, 0x90},
		{0x0d, 0x80, 0x08}};
	__u8 fourth10c0b[2][5] = {
		{0x2b, 0x01, 0x88, 0x01, 0x88},
		{0x2d, 0x01, 0x88, 0x01, 0x88}};

	__u8 fifth10c0[4][3] = {
		{0x0a, 0x00, 0x01},
		{0x06, 0x00, 0x29},
		{0x05, 0x00, 0x72}};

	// interrupt down

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x1066, buf, 1);	// URB 1

	// interrupt up

	// interrupt down

	buf[0] = 0x60;
	usb_microdia_control_write(dev, 0x1007, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x1006, buf, 1);

	buf[0] = 0x78;
	usb_microdia_control_write(dev, 0x1000, buf, 1);

	buf[0] = 0xc7;
	usb_microdia_control_write(dev, 0x1001, buf, 1);

	buf[0] = 0x1c;
	usb_microdia_control_write(dev, 0x1002, buf, 1);

	buf[0] = 0x01;
	usb_microdia_control_write(dev, 0x1061, buf, 1);

	buf[0] = 0x80;
	usb_microdia_control_write(dev, 0x1020, buf, 1);

	buf[0] = 0x40; buf[1] = 0x30; buf[2] = 0x20;
	buf[3] = 0x10; buf[4] = 0x08;
	usb_microdia_control_write(dev, 0x1067, buf, 5);

	buf[0] = 0xb0; buf[1] = dev->sensor_slave_address; buf[2] = 0x0d;
	buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00;
	buf[6] = 0x00; buf[7] = 0x10; buf[8] = 0x03;
	usb_microdia_control_write(dev, 0x10c0, buf, 9);

	usb_microdia_control_write(dev, 0x10e0, urb10, 24);	// URB 10

	buf[0] = 0x00; buf[1] = 0x00; buf[3] = 0x00;
	usb_microdia_control_write(dev, 0x10f8, buf, 3);

	buf[0] = 0x00; buf[1] = 0x40; buf[2] = 0x00;
	buf[3] = 0xe0; buf[4] = 0x03;
	usb_microdia_control_write(dev, 0x10fb, buf, 5);

	usb_microdia_control_write(dev, 0x1180, urb13, 11);

	usb_microdia_control_write(dev, 0x118b, urb14, 22);
	
	buf[0] = 0x00; buf[1] = 0x00;
	usb_microdia_control_write(dev, 0x11a1, buf, 2);

	buf[0] = 0x60; buf[1] = 0x20; buf[2] = 0x00;
	buf[3] = 0x08; buf[4] = 0x00; buf[5] = 0x00;
	buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00;
	usb_microdia_control_write(dev, 0x11b7, buf, 9);

	buf[0] = 0x30;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);

	buf[0] = 0x78;
	usb_microdia_control_write(dev, 0x1000, buf, 1);

	for (i = 0; i < 7; i++)				// URB 9-48
		sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2,
			first10c0[i][0], dev->sensor_flags, &first10c0[i][1]);

	buf[0] = 0x60;
	usb_microdia_control_write(dev, 0x1007, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x1006, buf, 1);

	buf[0] = 0x03;
	usb_microdia_control_write(dev, 0x10e0, buf, 1);	// URB 50

	buf[0] = 0x03;
	usb_microdia_control_write(dev, 0x10e0, buf, 1);

	buf[0] = 0xc6;
	usb_microdia_control_write(dev, 0x1001, buf, 1);

	buf[0] = 0xc4;
	usb_microdia_control_write(dev, 0x1001, buf, 1);

	buf[0] = 0x44;
	usb_microdia_control_write(dev, 0x1001, buf, 1);

	buf[0] = 0x80;
	usb_microdia_control_write(dev, 0x1189, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11bc, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11bd, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11be, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11bf, buf, 1);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4,
		second10c0a[0][0], dev->sensor_flags, &second10c0a[0][1]);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4,
		second10c0a[1][0], dev->sensor_flags, &second10c0a[1][1]);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0,
		0x09, dev->sensor_flags, NULL);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2,
		second10c0b[0], dev->sensor_flags, &second10c0b[1]);

	buf[0] = 0x06; buf[1] = 0x00; buf[2] = 0x02;
	buf[3] = 0x00; buf[4] = 0x28; buf[5] = 0x3c;
	usb_microdia_control_write(dev, 0x1180, buf, 6);

	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00;
	buf[3] = 0xf0; buf[4] = 0x00;
	usb_microdia_control_write(dev, 0x10fb, buf, 5);	// URB 70

	buf[0] = 0x8c;
	usb_microdia_control_write(dev, 0x1189, buf, 1);

	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x6a; buf[3] = 0x50;
	usb_microdia_control_write(dev, 0x11a1, buf, 4);

	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x50; buf[3] = 0x3c;
	usb_microdia_control_write(dev, 0x11ab, buf, 4);

	buf[0] = 0x78;
	usb_microdia_control_write(dev, 0x1000, buf, 1);

	buf[0] = 0x18;
	usb_microdia_control_write(dev, 0x1002, buf, 1);

	buf[0] = 0x18;
	usb_microdia_control_write(dev, 0x1002, buf, 1);

	buf[0] = 0x38;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);

	buf[0] = 0x02;
	usb_microdia_control_write(dev, 0x118a, buf, 1);

	buf[0] = 0x02;
	usb_microdia_control_write(dev, 0x0395, buf, 1);

	buf[0] = 0x78;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 80
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0xf9;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0xfa;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0x7b;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0x7c;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0x7b;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 90

	for (i = 0; i < 4; i++)					// URB 97-106
		sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2,
			third10c0[i][0], dev->sensor_flags, &third10c0[i][1]);

	buf[0] = 0x06;
	usb_microdia_control_write(dev, 0x1180, buf, 1);

	for (i = 0; i < 3; i++)
		sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2,
			fourth10c0a[i][0], dev->sensor_flags, &fourth10c0a[i][1]);

	for (i = 0; i < 2; i++)
		sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4,
			fourth10c0b[i][0], dev->sensor_flags, &fourth10c0b[i][1]);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0,
		0x33, dev->sensor_flags, NULL);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2,
		first10c0[0][0], dev->sensor_flags, &first10c0[0][1]);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118c, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118d, buf, 1);	// URB 110

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118e, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118f, buf, 1);

	buf[0] = 0x0a;
	usb_microdia_control_write(dev, 0x11ba, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118c, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118d, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118e, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118f, buf, 1);

	buf[0] = 0x10;
	usb_microdia_control_write(dev, 0x118b, buf, 1);

	usb_microdia_control_write(dev, 0x1190, urb119, 17);

	for (i = 0; i < 9; i++)					// URB 120-128
		usb_microdia_control_write(dev, 0x10e1, urb120, 21);

	buf[0] = 0x05;
	usb_microdia_control_write(dev, 0x10f7, buf, 1);

	buf[0] = 0x60;
	usb_microdia_control_write(dev, 0x10f6, buf, 1);	// URB 130

	usb_microdia_control_write(dev, 0x10e1, urb120, 21);

	buf[0] = 0x14;
	usb_microdia_control_write(dev, 0x10f8, buf, 1);

	buf[0] = 0xff;
	usb_microdia_control_write(dev, 0x10fa, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x10f9, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x10f9, buf, 1);

	buf[0] = 0x0a;
	usb_microdia_control_write(dev, 0x11ba, buf, 1);

	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00;
	usb_microdia_control_write(dev, 0x11bc, buf, 4);

	/* Set whole array buf to 0x00 */
	memset(buf, 0x00, 48);
	usb_microdia_control_write(dev, 0x11c0, buf, 48);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118c, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118d, buf, 1);	// URB 140

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118e, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118f, buf, 1);

	buf[0] = 0x2d; buf[1] = 0x2d; buf[2] = 0x3a;
	buf[3] = 0x05; buf[4] = 0x04; buf[5] = 0x3f;
	usb_microdia_control_write(dev, 0x11a5, buf, 6);

	buf[0] = 0x14; buf[1] = 0xec; buf[2] = 0x0a; buf[3] = 0xf6;
	usb_microdia_control_write(dev, 0x11af, buf, 4);

	buf[0] = 0x32; buf[1] = 0xdd; buf[2] = 0x2d; buf[3] = 0xdd;
	usb_microdia_control_write(dev, 0x11b3, buf, 4);

	buf[0] = 0x07;
	usb_microdia_control_write(dev, 0x10e0, buf, 1);

	buf[0] = 0x01;
	usb_microdia_control_write(dev, 0x1061, buf, 1);

	buf[0] = 0x27;
	usb_microdia_control_write(dev, 0x10e0, buf, 1);

	usb_microdia_control_write(dev, 0x1100, qtable1, 64);

	usb_microdia_control_write(dev, 0x1140, qtable2, 64);	// URB 150

	buf[0] = 0x07;
	usb_microdia_control_write(dev, 0x10e0, buf, 1);

	buf[0] = 0x03;
	usb_microdia_control_write(dev, 0x1061, buf, 1);

	buf[0] = 0x0b;
	usb_microdia_control_write(dev, 0x10e0, buf, 1);

	buf[0] = 0x8c;
	usb_microdia_control_write(dev, 0x1189, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11bc, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11bd, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11be, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11bf, buf, 1);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4,
		second10c0a[0][0], dev->sensor_flags, &second10c0a[0][1]);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4,
		second10c0a[1][0], dev->sensor_flags, &second10c0a[1][1]);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0,
		0x09, dev->sensor_flags, NULL);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2,
		second10c0b[0], dev->sensor_flags, &second10c0b[1]);

	buf[0] = 0x06; buf[1] = 0x00; buf[2] = 0x02;
	buf[3] = 0x00; buf[4] = 0x28; buf[5] = 0x3c;
	usb_microdia_control_write(dev, 0x1180, buf, 6);

	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00;
	buf[3] = 0xf0; buf[4] = 0x00;
	usb_microdia_control_write(dev, 0x10fb, buf, 5);

	buf[0] = 0x8c;
	usb_microdia_control_write(dev, 0x1189, buf, 1);

	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x6a; buf[3] = 0x50;
	usb_microdia_control_write(dev, 0x11a1, buf, 4);	// URB 170

	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x50; buf[3] = 0x3c;
	usb_microdia_control_write(dev, 0x11ab, buf, 4);

	buf[0] = 0x03;
	usb_microdia_control_write(dev, 0x1061, buf, 1);

	buf[0] = 0x0a;
	usb_microdia_control_write(dev, 0x11ba, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11b9, buf, 1);

	buf[0] = 0x0b;
	usb_microdia_control_write(dev, 0x11ba, buf, 1);

	buf[0] = 0x01;
	usb_microdia_control_write(dev, 0x1061, buf, 1);

	buf[0] = 0x78;
	usb_microdia_control_write(dev, 0x1000, buf, 1);

	buf[0] = 0x18;
	usb_microdia_control_write(dev, 0x1002, buf, 1);

	buf[0] = 0x18;
	usb_microdia_control_write(dev, 0x1002, buf, 1);

	buf[0] = 0x7b;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 180

	buf[0] = 0x02;
	usb_microdia_control_write(dev, 0x118a, buf, 1);

	buf[0] = 0x02;
	usb_microdia_control_write(dev, 0x0395, buf, 1);

	buf[0] = 0x78;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0xf9;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0xfa;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0x7b;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);		// URB 190

	buf[0] = 0x7c;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0x7b;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);

	for (i = 0; i < 3; i++)					// URB 200-205
		sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2,
			fifth10c0[i][0], dev->sensor_flags, &fifth10c0[i][1]);

	sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 2,
		0x00, dev->sensor_flags, buf);
	/* __u8 result[2] = {buf[3], buf[4]}; */

	buf[0] = 0x03;
	usb_microdia_control_write(dev, 0x1061, buf, 1);

	return 0;
}

/**
 * @brief From startstream.log
 *
 * @param dev
 *
 * @returns 0
 *
 * @author GWater
 *
 * For SN9C201 with SOI968.
 * This is function triggers a working stream of image data and a glowing LED.
 */
int microdia_624e_start_stream(struct usb_microdia *dev)
{
	int i = -1;
	__u8 buf[48];

	__u8 urb10[24] =
		{0x47, 0x18, 0x00, 0x30, 0x00, 0x09, 0x00, 0xed,
		0x0f, 0xda, 0x0f, 0x3a, 0x00, 0x3a, 0x00, 0xd0,
		0x0f, 0xf7, 0x0f, 0x00, 0x00, 0x00, 0x60, 0x0a};

	__u8 urb14[22] =
		{0x00, 0x20, 0x20, 0x20, 0x20, 0x00, 0x24, 0x3b,
		0x4f, 0x61, 0x71, 0x80, 0x8f, 0x9d, 0xab, 0xb8,
		0xc4, 0xd1, 0xdd, 0xe9, 0xf4, 0xff};

	__u8 urb119[17] =
		{0x08, 0x2b, 0x59, 0x79, 0x91, 0x9e, 0xa9, 0xb3,
		0xbd, 0xc6, 0xce, 0xd6, 0xdd, 0xe4, 0xea, 0xf1,
		0xf7};

	__u8 urb120[21] =
		{0x16, 0x00, 0x2d, 0x00, 0x08, 0x00, 0xdd, 0x0f,
		0xbc, 0x0f, 0x68, 0x00, 0x68, 0x00, 0xa9, 0x0f,
		0xf0, 0x0f, 0x00, 0x00, 0x00};

	__u8 qtable1[64] =
		{0x0d, 0x08, 0x08, 0x0d, 0x08, 0x08, 0x0d, 0x0d,
		0x0d, 0x0d, 0x11, 0x0d, 0x0d, 0x11, 0x15, 0x21,
		0x15, 0x15, 0x11, 0x11, 0x15, 0x2a, 0x1d, 0x1d,
		0x19, 0x21, 0x32, 0x2a, 0x32, 0x32, 0x2e, 0x2a,
		0x2e, 0x2e, 0x36, 0x3a, 0x4b, 0x43, 0x36, 0x3a,
		0x47, 0x3a, 0x2e, 0x2e, 0x43, 0x5c, 0x43, 0x47,
		0x4f, 0x53, 0x58, 0x58, 0x58, 0x32, 0x3f, 0x60,
		0x64, 0x5c, 0x53, 0x64, 0x4b, 0x53, 0x58, 0x53};

	__u8 qtable2[64] =
		{0x0d, 0x11, 0x11, 0x15, 0x11, 0x15, 0x26, 0x15,
		0x15, 0x26, 0x53, 0x36, 0x2e, 0x36, 0x53, 0x53,
		0x53, 0x53, 0x53, 0x53, 0x53, 0x53, 0x53, 0x53,
		0x53, 0x53, 0x53, 0x53, 0x53, 0x53, 0x53, 0x53,
		0x53, 0x53, 0x53, 0x53, 0x53, 0x53, 0x53, 0x53,
		0x53, 0x53, 0x53, 0x53, 0x53, 0x53, 0x53, 0x53,
		0x53, 0x53, 0x53, 0x53, 0x53, 0x53, 0x53, 0x53,
		0x53, 0x53, 0x53, 0x53, 0x53, 0x53, 0x53, 0x53};

	__u8 first10c0[20][2] = {
		{0x12, 0x80}, {0x12, 0x00}, {0x0c, 0x00}, {0x0f, 0x1f},
		{0x11, 0x80}, {0x38, 0x52}, {0x1e, 0x00}, {0x33, 0x08},
		{0x35, 0x8c}, {0x36, 0x0c}, {0x37, 0x04}, {0x45, 0x04},
		{0x47, 0xff}, {0x3e, 0x00}, {0x3f, 0x00}, {0x3b, 0x20},
		{0x3a, 0x96}, {0x3d, 0x0a}, {0x14, 0x4e}, {0x13, 0x88}};

	__u8 second10c0a[2] =
		{0x12, 0x40};
	__u8 second10c0b[5] =
		{0x17, 0x13, 0x63, 0x01, 0x79};	// length 4
	__u8 second10c0c[2] =
		{0x1b, 0x00};			// length 0
	__u8 second10c0d[2][2] = {
		{0x32, 0x24},
		{0x03, 0x00}};

	__u8 third10c0a[5][2] = {
		{0x11, 0x40},
		{0x2a, 0x10},
		{0x2b, 0xe0},
		{0x10, 0x32},
		{0x00, 0x00}};
	__u8 third10c0b[3] =
		{0x01, 0x80, 0x80};		// length 2

	__u8 read10c0[4] =
		{0x0a, 0x0b, 0x1c, 0x1d};	// these are four addresses

	// interrupt down

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x1066, buf, 1);	// URB 1

	// interrupt up

	// interrupt down

	buf[0] = 0x78;
	usb_microdia_control_write(dev, 0x1000, buf, 1);

	buf[0] = 0xc7;
	usb_microdia_control_write(dev, 0x1001, buf, 1);

	buf[0] = 0x1c;
	usb_microdia_control_write(dev, 0x1002, buf, 1);

	buf[0] = 0xa0; buf[1] = 0xa0;
	usb_microdia_control_write(dev, 0x1006, buf, 2);

	buf[0] = 0x01;
	usb_microdia_control_write(dev, 0x1061, buf, 1);

	buf[0] = 0x80;
	usb_microdia_control_write(dev, 0x1020, buf, 1);

	buf[0] = 0x40; buf[1] = 0x30; buf[2] = 0x20;
	buf[3] = 0x10; buf[4] = 0x08;
	usb_microdia_control_write(dev, 0x1067, buf, 5);

	buf[0] = 0x92; buf[1] = dev->sensor_slave_address; buf[2] = 0x00;
	buf[3] = 0x00; buf[4] = 0x1d; buf[5] = 0x00;
	buf[6] = 0xa2; buf[7] = 0x10; buf[8] = 0x03;
	usb_microdia_control_write(dev, 0x10c0, buf, 9);

	usb_microdia_control_write(dev, 0x10e0, urb10, 24);	// URB 10

	buf[0] = 0x00; buf[1] = 0x00; buf[3] = 0x00;
	usb_microdia_control_write(dev, 0x10f8, buf, 3);
	
	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00;
	buf[3] = 0xf0; buf[4] = 0x00;
	usb_microdia_control_write(dev, 0x10fb, buf, 5);
	
	buf[0] = 0x8a; buf[1] = 0xcc; buf[2] = 0x08;
	usb_microdia_control_write(dev, 0x1188, buf, 3);
	
	usb_microdia_control_write(dev, 0x118b, urb14, 22);
	
	buf[0] = 0x00; buf[1] = 0x00;
	usb_microdia_control_write(dev, 0x11a1, buf, 2);
	
	buf[0] = 0x60; buf[1] = 0x20; buf[2] = 0x00;
	buf[3] = 0x08; buf[4] = 0x00; buf[5] = 0x40;
	buf[6] = 0x01; buf[7] = 0xf0; buf[8] = 0x00;
	usb_microdia_control_write(dev, 0x11b7, buf, 9);
	
	buf[0] = 0x30;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	
	buf[0] = 0x78;
	usb_microdia_control_write(dev, 0x1000, buf, 1);

	for (i = 0; i < 20; i++)				// URB 9-48
		sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1,
			first10c0[i][0], dev->sensor_flags, &first10c0[i][1]);

	buf[0] = 0x47;
	usb_microdia_control_write(dev, 0x10e0, buf, 1);

	buf[0] = 0x07;
	usb_microdia_control_write(dev, 0x10e0, buf, 1);	// URB 50

	buf[0] = 0xc6;
	usb_microdia_control_write(dev, 0x1001, buf, 1);

	buf[0] = 0xc4;
	usb_microdia_control_write(dev, 0x1001, buf, 1);

	buf[0] = 0x44;
	usb_microdia_control_write(dev, 0x1001, buf, 1);

	buf[0] = 0x8c;
	usb_microdia_control_write(dev, 0x1189, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11bc, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11bd, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11be, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11bf, buf, 1);


	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1,		// URB 59-60
		second10c0a[0], dev->sensor_flags, &second10c0a[1]);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4,
		second10c0b[0], dev->sensor_flags, &second10c0b[1]);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0,
		second10c0c[0], dev->sensor_flags, &second10c0c[1]);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1,
		second10c0d[0][0], dev->sensor_flags, &second10c0d[0][1]);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1,
		second10c0d[1][0], dev->sensor_flags, &second10c0d[1][1]);

	buf[0] = 0x3c; buf[1] = 0x00; buf[2] = 0x0b;
	buf[3] = 0x00; buf[4] = 0x28; buf[5] = 0x3c;
	usb_microdia_control_write(dev, 0x1180, buf, 6);

	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00;
	buf[3] = 0xf0; buf[4] = 0x00;
	usb_microdia_control_write(dev, 0x10fb, buf, 5);	// URB 70

	buf[0] = 0x8c;
	usb_microdia_control_write(dev, 0x1189, buf, 1);

	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x6a; buf[3] = 0x50;
	usb_microdia_control_write(dev, 0x11a1, buf, 4);

	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x50; buf[3] = 0x3c;
	usb_microdia_control_write(dev, 0x11ab, buf, 4);

	buf[0] = 0x7c;
	usb_microdia_control_write(dev, 0x1000, buf, 1);

	buf[0] = 0x1c;
	usb_microdia_control_write(dev, 0x1002, buf, 1);

	buf[0] = 0x1c;
	usb_microdia_control_write(dev, 0x1002, buf, 1);

	buf[0] = 0x38;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);

	buf[0] = 0x05;
	usb_microdia_control_write(dev, 0x118a, buf, 1);

	buf[0] = 0x05;
	usb_microdia_control_write(dev, 0x0395, buf, 1);

	buf[0] = 0x78;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 80
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0xf9;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0x7a;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0x7b;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0x7c;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0xfd;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 90
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0xfe;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0x7f;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0x7a;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);

	for (i = 0; i < 5; i++)					// URB 97-106
		sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1,
			third10c0a[i][0], dev->sensor_flags, &third10c0a[i][1]);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2,
		third10c0b[0], dev->sensor_flags, &third10c0b[1]);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118c, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118d, buf, 1);	// URB 110

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118e, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118f, buf, 1);

	buf[0] = 0x0a;
	usb_microdia_control_write(dev, 0x11ba, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118c, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118d, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118e, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118f, buf, 1);

	buf[0] = 0x0a;
	usb_microdia_control_write(dev, 0x118b, buf, 1);

	usb_microdia_control_write(dev, 0x1190, urb119, 17);

	for (i = 0; i < 9; i++)					// URB 120-128
		usb_microdia_control_write(dev, 0x10e1, urb120, 21);


	buf[0] = 0x07;
	usb_microdia_control_write(dev, 0x10f7, buf, 1);

	buf[0] = 0x60;
	usb_microdia_control_write(dev, 0x10f6, buf, 1);	// URB 130

	usb_microdia_control_write(dev, 0x10e1, urb120, 21);

	buf[0] = 0x14;
	usb_microdia_control_write(dev, 0x10f8, buf, 1);

	buf[0] = 0xff;
	usb_microdia_control_write(dev, 0x10fa, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x10f9, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x10f9, buf, 1);

	buf[0] = 0x0e;
	usb_microdia_control_write(dev, 0x11ba, buf, 1);

	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00;
	usb_microdia_control_write(dev, 0x11bc, buf, 4);

	/* Set whole buf to 0x00 */
	memset(buf, 0, 48);
	usb_microdia_control_write(dev, 0x11c0, buf, 48);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118c, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118d, buf, 1);	// URB 140

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118e, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118f, buf, 1);

	buf[0] = 0x2d; buf[1] = 0x2d; buf[2] = 0x3a;
	buf[3] = 0x05; buf[4] = 0x04; buf[5] = 0x3f;
	usb_microdia_control_write(dev, 0x11a5, buf, 6);

	buf[0] = 0x28; buf[1] = 0xd8; buf[2] = 0x14; buf[3] = 0xec;
	usb_microdia_control_write(dev, 0x11af, buf, 4);

	buf[0] = 0x32; buf[1] = 0xdd; buf[2] = 0x2d; buf[3] = 0xdd;
	usb_microdia_control_write(dev, 0x11b3, buf, 4);

	buf[0] = 0x07;
	usb_microdia_control_write(dev, 0x10e0, buf, 1);

	buf[0] = 0x01;
	usb_microdia_control_write(dev, 0x1061, buf, 1);

	buf[0] = 0x27;
	usb_microdia_control_write(dev, 0x10e0, buf, 1);

	usb_microdia_control_write(dev, 0x1100, qtable1, 64);

	usb_microdia_control_write(dev, 0x1140, qtable2, 64);	// URB 150

	buf[0] = 0x07;
	usb_microdia_control_write(dev, 0x10e0, buf, 1);

	buf[0] = 0x03;
	usb_microdia_control_write(dev, 0x1061, buf, 1);

	buf[0] = 0x0b;
	usb_microdia_control_write(dev, 0x10e0, buf, 1);

	buf[0] = 0x8c;
	usb_microdia_control_write(dev, 0x1189, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11bc, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11bd, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11be, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11bf, buf, 1);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4,		// URB 159-160
		second10c0b[0], dev->sensor_flags, &second10c0b[1]);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0,
		second10c0c[0], dev->sensor_flags, &second10c0c[1]);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1,
		second10c0d[0][0], dev->sensor_flags, &second10c0d[0][1]);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1,
		second10c0d[1][0], dev->sensor_flags, &second10c0d[1][1]);

	buf[0] = 0x3c; buf[1] = 0x00; buf[2] = 0x0b;
	buf[3] = 0x00; buf[4] = 0x28; buf[5] = 0x3c;
	usb_microdia_control_write(dev, 0x1180, buf, 6);

	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00;
	buf[3] = 0xf0; buf[4] = 0x00;
	usb_microdia_control_write(dev, 0x10fb, buf, 5);

	buf[0] = 0x8c;
	usb_microdia_control_write(dev, 0x1189, buf, 1);

	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x6a; buf[3] = 0x50;
	usb_microdia_control_write(dev, 0x11a1, buf, 4);	// URB 170

	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x50; buf[3] = 0x3c;
	usb_microdia_control_write(dev, 0x11ab, buf, 4);

	buf[0] = 0x03;
	usb_microdia_control_write(dev, 0x1061, buf, 1);

	buf[0] = 0x0e;
	usb_microdia_control_write(dev, 0x11ba, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11b9, buf, 1);

	buf[0] = 0x0f;
	usb_microdia_control_write(dev, 0x11ba, buf, 1);

	buf[0] = 0x01;
	usb_microdia_control_write(dev, 0x1061, buf, 1);

	buf[0] = 0x7c;
	usb_microdia_control_write(dev, 0x1000, buf, 1);

	buf[0] = 0x1c;
	usb_microdia_control_write(dev, 0x1002, buf, 1);

	buf[0] = 0x1c;
	usb_microdia_control_write(dev, 0x1002, buf, 1);

	buf[0] = 0x7a;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 180

	buf[0] = 0x05;
	usb_microdia_control_write(dev, 0x118a, buf, 1);

	buf[0] = 0x05;
	usb_microdia_control_write(dev, 0x0395, buf, 1);

	buf[0] = 0x78;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0xf9;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0x7a;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0x7b;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);		// URB 190

	buf[0] = 0x7c;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0x7d;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0xfe;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0x7f;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);
	usb_microdia_control_read(dev, 0x11b8, buf, 1);

	buf[0] = 0x7a;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);

	for (i = 0; i < 3; i++)					// URB 200-205
		sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1,
			third10c0a[i][0], dev->sensor_flags, &third10c0a[i][1]);

	for (i = 0; i < 4; i++)					// URB 206-225
		sn9c20x_read_i2c_data(dev, dev->sensor_slave_address,
			1, read10c0[i], dev->sensor_flags, buf);

	buf[0] = 0x03;
	usb_microdia_control_write(dev, 0x1061, buf, 1);

	buf[0] = 0xa0;
	usb_microdia_control_write(dev, 0x1007, buf, 1);

	buf[0] = 0x80;
	usb_microdia_control_write(dev, 0x1006, buf, 1);

	return 0;
}

int microdia_624f_start_stream(struct usb_microdia *dev)
{
	int ret, i;
	__u8 last_11b8[1];
	__u8 eeprom_slave_id = 0x50;
	__u8 eeprom_flags = SN9C20X_I2C_2WIRE;
	__u16 command;
	__u8 buf[64];
	__u8 unknown1[9] = { dev->sensor_flags, dev->sensor_slave_address,
		0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03 };
	
	buf[0] = 0x20;
	command = 0x1007;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	buf[0] = 0x00;
	command = 0x1006;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	buf[0] = 0x78;
	command = 0x1000;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	buf[0] = 0xc7;
	command = 0x1001;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	buf[0] = 0x1c;
	command = 0x1002;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	buf[0] = 0x01;
	command = 0x1061;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	buf[0] = 0x80;
	command = 0x1020;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	buf[0] = 0x40; buf[1] = 0x30; buf[2] = 0x20;
	buf[3] = 0x10; buf[4] = 0x08;
	command = 0x1067;
	ret = usb_microdia_control_write(dev, command, buf, 5);
	if(ret < 0) goto err;

	/* this is the same register as the I2C write, not sure why
	 * we're writing 9 bytes -- the SN9C102 docs say all writes
	 * must be 8 bytes, but we don't have docs for SN9C20x */
	command = 0x10c0;
	ret = usb_microdia_control_write(dev, command, unknown1, 9);
	if(ret < 0) goto err;
	
	buf[0] = 0x44; buf[1] = 0x18; buf[2] = 0x00; buf[3] = 0x30;
	buf[4] = 0x00; buf[5] = 0x09; buf[6] = 0x00; buf[7] = 0xed;
	buf[8] = 0x0f; buf[9] = 0xda; buf[10] = 0x0f; buf[11] = 0x3a;
	buf[12] = 0x00; buf[13] = 0x3a; buf[14] = 0x00; buf[15] = 0xd0;
	buf[16] = 0x0f; buf[17] = 0xf7; buf[18] = 0x0f; buf[19] = 0x00;
	buf[20] = 0x00; buf[21] = 0x00; buf[22] = 0x60; buf[23] = 0x0a;
	command = 0x10e0;
	ret = usb_microdia_control_write(dev, command, buf, 24);
	if(ret < 0) goto err;

	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00;
	command = 0x10f8;
	ret = usb_microdia_control_write(dev, command, buf, 3);
	if(ret < 0) goto err;
	
	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00;
	buf[3] = 0xf0; buf[4] = 0x00;
	command = 0x10fb;
	ret = usb_microdia_control_write(dev, command, buf, 5);
	if(ret < 0) goto err;

	buf[0] = 0x87; buf[1] = 0xcc; buf[2] = 0x08;
	command = 0x1188;
	ret = usb_microdia_control_write(dev, command, buf, 3);
	if(ret < 0) goto err;
	
	buf[0] = 0x00; buf[1] = 0x20; buf[2] = 0x20; buf[3] = 0x20;
	buf[4] = 0x20; buf[5] = 0x00; buf[6] = 0x24; buf[7] = 0x3b;
	buf[8] = 0x4f; buf[9] = 0x61; buf[10] = 0x71; buf[11] = 0x80;
	buf[12] = 0x8f; buf[13] = 0x9d; buf[14] = 0xab; buf[15] = 0xb8;
	buf[16] = 0xc4; buf[17] = 0xd1; buf[18] = 0xdd; buf[19] = 0xe9;
	buf[20] = 0xf4; buf[21] = 0xff;
	command = 0x118b;
	ret = usb_microdia_control_write(dev, command, buf, 22);
	if(ret < 0) goto err;

	buf[0] = 0x00; buf[1] = 0x00;
	command = 0x11a1;
	ret = usb_microdia_control_write(dev, command, buf, 2);
	if(ret < 0) goto err;
	
	buf[0] = 0x60; buf[1] = 0x20; buf[2] = 0x00; buf[3] = 0x08;
	buf[4] = 0x00; buf[5] = 0x40; buf[6] = 0x01; buf[7] = 0xf0;
	buf[8] = 0x00;
	command = 0x11b7;
	ret = usb_microdia_control_write(dev, command, buf, 9);
	if(ret < 0) goto err;

	buf[0] = 0x30;
	command = 0x11b8;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	buf[0] = 0x78;
	command = 0x1000;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	/* reset all SCCB registers to their default values */
	buf[0] = OV965X_COM7_SCCB_RESET;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_COM7,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x7f; //0x7f; /* AGC setting */
	buf[1] = 0xe0; /* blue gain setting */
	buf[2] = 0x90; /* red gain setting */
	buf[3] = OV965X_VREF_VSTOP_LOW3(0x04) | OV965X_VREF_VSTART_LOW3(0x02);
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, OV965X_CTL_GAIN,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* 
	  this write appears to be unnecessary since we issued
	  the reset above and the spec says that 0x00 is the default
	  for all 4 of the registers we're writing to here
	 */
	memset(buf, 0, 4);
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, OV965X_CTL_COM1,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* first byte still 0, not sure why, RAVE default is 0x00 too */
	buf[0] = 0;
	buf[1] = OV965X_COM2_OUTPUT_DRIVE_CAP_2X;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, OV965X_CTL_RAVE,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x00; buf[1] = 0x00;
	/* not sure why we're setting bit 5 below, spec says it's reserved */
	buf[2] = OV965X_COM5_15FPS_48MHZ_RGB | 0x20;
	/* likewise, spec says 4 & 6 are reserved */
	buf[3] = OV965X_COM6_TIMING_RESET_ON_FMT_CHANGE	| 0x50;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, OV965X_CTL_COM3,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0xff; /* AEC bits 2-9 */
	buf[1] = OV965X_CLKRC_INPUT_CLK_NO_SCALE;
	buf[2] = OV965X_COM7_OUTPUT_RAW_RGB;
	buf[3] = OV965X_COM8_FAST_AGC_AEC | OV965X_COM8_AEC_STEP_SIZE_NOLIMIT |
	   /*OV965X_COM8_AGC_ENABLE |*/ OV965X_COM8_AEC_ENABLE | OV965X_COM8_AWB_ENABLE;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, OV965X_CTL_AECH,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = OV965X_COM9_MAX_AGC_8X | OV965X_COM9_RELAX_EXPOSURE_TIMING |
		OV965X_COM9_DROP_VSYNC_ON_FRAME_DROP |
		OV965X_COM9_DROP_FRAME_ON_BIG_AEC;
	buf[1] = 0x00;
	buf[2] = 0x07; /* hmm, spec says this is reserved */
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 3, OV965X_CTL_COM9,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* this is supposed to be reserved too */
	buf[0] = 0x04; buf[1] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, OV965X_CTL_MVFP,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x78; /* this is the default! */
	buf[1] = 0x68; /* this is the default! */
	buf[2] = 0xd4; /* this is the default! */
	buf[3] = 0x80; /* this is the default! */
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, OV965X_CTL_AEW,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = OV965X_BIAS_SUBTRACT;
	buf[1] = OV965X_Gr_COM_BYPASS_ANALOG_BLC | OV965X_Gr_COM_BYPASS_REGULATOR;
	buf[2] = 0x00; buf[3] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, OV965X_CTL_GbBIAS,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = OV965X_BIAS_SUBTRACT;
	buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, OV965X_CTL_RBIAS,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x08; /* HSYNC rising edge start */
	buf[1] = 0x30; /* HSYNC rising edge end */
	buf[2] = OV965X_HREF_EDGE_OFT_TO_DATA_OUT(2) | OV965X_HREF_HSTART_LOW3(6)
		| OV965X_HREF_HSTOP_LOW3(4);
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 3, OV965X_CTL_HSYST,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* the docs say all 4 of these bytes are reserved */
	buf[0] = 0xe2; buf[1] = 0xbf; buf[2] = 0x81; buf[3] = 0xf9;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, OV965X_CTL_CHLF,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* the docs say these 2 bytes are reserved */
	buf[0] = 0x00; buf[1] = 0x93;
	buf[2] = 0x50; /* bits 4 and 6 are supposedly reserved */
	buf[3] = OV965X_TSLB_OUTPUT_SEQ_UYVY | OV965X_TSLB_DIGITAL_BLC_ENABLE;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, OV965X_CTL_ADC,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = OV965X_COM11_MANUAL_BANDING_FILTER;
	buf[1] = 0x73; /* these bits all reserved */
	buf[2] = OV965X_COM13_ENABLE_COLOR_MATRIX | OV965X_COM13_DELAY_Y_CHANNEL
		| OV965X_COM13_OUTPUT_DELAY(1);
	buf[3] =  OV965X_COM14_YUV_EDGE_ENHANCE
		| OV965X_COM14_EDGE_ENHANCE_FACTOR_DBL
		| 0x0b; /* these last bits supposedly reserved */
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, OV965X_CTL_COM11,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = OV965X_EDGE_EDGE_ENHANCE_LOW4(8)
		| OV965X_EDGE_EDGE_ENHANCE_FACTOR(8);
	buf[1] = OV965X_COM15_OUTPUT_RANGE_O0_TO_FF | 0x01; /* bit reserved */
	buf[2] = 0x00; buf[3] = 0x08; /* this bit supposedly reserved */
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, OV965X_CTL_EDGE,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x00;
	/* OV9653 control register 0x43 is reserved, according to the docs */
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0, 0x43,
		dev->sensor_flags, buf);	// 'Dummy'-write
	if(ret < 0) goto err;

	buf[0] = 0x80; buf[1] = 0x80; buf[2] = 0x40; buf[3] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, OV965X_CTL_MANU,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x0a;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_DBLV,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x06; buf[1] = 0x20; buf[2] = 0x00; buf[3] = 0x00;
	/* OV9653 control register 0x8b is reserved, according to the docs */
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x8b,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0xdf;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_DBLC1,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x88; buf[3] = 0x88;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, OV965X_CTL_DM_LNL,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x04;
	/* control reg 0x96 reserved, according to docs */
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, 0x96,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x80;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_COM26,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x80; buf[1] = 0xb8; buf[2] = 0x92; buf[3] = 0x0a;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, OV965X_CTL_ACOM2,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* reserved, according to docs */
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0, 0xac,
		dev->sensor_flags, NULL);
	if(ret < 0) goto err;

	buf[0] = 0x1b; buf[1] = 0xbb; buf[2] = 0x05; buf[3] = 0x7d;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, OV965X_CTL_HSTART,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_PSHIFT,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* now for some reason it switches to these two reads and
	 * then back to the long init sequence */
	command = 0x10c1;
	ret = usb_microdia_control_read(dev, command, buf, 1); /* reads 30 */
	if(ret < 0) goto err;
	//msg("Should read 0x30, read: %x", buf[0]);

	command = 0x10c1;
	buf[0] = eeprom_slave_id;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	// TRY TO READ FROM EEPROM:
	ret = sn9c20x_read_i2c_data(dev, eeprom_slave_id, 1, 0x00, eeprom_flags, buf);
	if (ret < 0)
	{
		UDIA_INFO("No EEPROM found\n");
	}
	else
	{
		UDIA_INFO("Read from EEPROM successful\n");
		/* returns 0xff -- don't know what we're supposed to do w/ that info */
		//msg("Read from eeprom, should read 0xff, read: %x", buf[0]);
	}

	command = 0x10c1;
	buf[0] = dev->sensor_slave_address;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x10e0;
	buf[0] = 0x47;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x10e0;
	buf[0] = 0x07;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x1001;
	buf[0] = 0xc6;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x1001;
	buf[0] = 0xc4;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x1001;
	buf[0] = 0x44;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x1189;
	buf[0] = 0x8c;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x11bc;
	buf[0] = 0x40;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if (ret < 0) goto err;

	command = 0x11bd;
	buf[0] = 0x01;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if (ret < 0) goto err;

	command = 0x11be;
	buf[0] = 0xf0;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if (ret < 0) goto err;

	command = 0x11bf;
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if (ret < 0) goto err;

	buf[0] = 0x45;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_COM7,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x36;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_VREF,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x24; buf[1] = 0xc5; buf[2] = 0x00; buf[3] = 0x3c;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, OV965X_CTL_HSTART,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0, OV965X_CTL_PSHIFT,
		dev->sensor_flags, NULL);
	if(ret < 0) goto err;

	buf[0] = 0x84;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_HREF,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	command = 0x1180;
	buf[0] = 0x02; buf[1] = 0x00; buf[2] = 0x07; buf[3] = 0x00;
	buf[4] = 0x28; buf[5] = 0x3c;
	ret = usb_microdia_control_write(dev, command, buf, 6);
	if(ret < 0) goto err;

	command = 0x10fb;
	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00; buf[3] = 0xf0;
	buf[4] = 0x00;
	ret = usb_microdia_control_write(dev, command, buf, 5);
	if(ret < 0) goto err;

	command = 0x1189;
	buf[0] = 0x8c;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x11a1;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x6a; buf[3] = 0x50;
	ret = usb_microdia_control_write(dev, command, buf, 4);
	if(ret < 0) goto err;

	command = 0x11ab;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x50; buf[3] = 0x3c;
	ret = usb_microdia_control_write(dev, command, buf, 4);
	if(ret < 0) goto err;

	command = 0x1000;
	buf[0] = 0x78;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x1002;
	buf[0] = 0x18;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x1002;
	buf[0] = 0x18;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	/* urb 297 */
	command = 0x11b8;
	buf[0] = 0x38;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x118a;
	buf[0] = 0x04;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x0395;
	buf[0] = 0x04;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x11b8;
	buf[0] = 0x78;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, command, buf, 1);	// 0x38
	if(ret < 0) goto err;
	if (buf[0] == 0x38)
		buf[0] = 0x79;
	else if (buf[0] == 0xb8)
		buf[0] = 0xf9;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, command, buf, 1);	// 0x39
	if(ret < 0) goto err;
	if (buf[0] == 0x39)
		buf[0] = 0x7a;
	else if (buf[0] == 0xb9)
		buf[0] = 0xfa;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, command, buf, 1);	// 0x3a
	if(ret < 0) goto err;
	if (buf[0] == 0x3a)
		buf[0] = 0x7b;
	else if (buf[0] == 0xba)
		buf[0] = 0xfb;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, command, buf, 1);	// 0x3b
	if(ret < 0) goto err;
	if (buf[0] == 0x3b)
		buf[0] = 0x7c;
	else if (buf[0] == 0xbb)
		buf[0] = 0xfc;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, command, buf, 1);	// 0x3c
	if(ret < 0) goto err;
	last_11b8[0] = buf[0];
	buf[0] = last_11b8[0] & 0xf0;	// 0xf0 or 0x70
	buf[0] = buf[0] | 0x0a;	// => THIS VALUE SEEMS TO DEPEND ON WHAT WAS READ/WRITTEN IN THE LAST 5 READS/WRITES
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	buf[0] = 0x80;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_CLKRC,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* urb 331 */
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_EXHCH,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_EXHCL,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_DM_LNL,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* urb 347 */
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_DM_LNH,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x04;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_MVFP,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* is it really necessary to do this same write again?; 627f does this, too */
	buf[0] = 0x04;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_MVFP,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* urb 365 */
	command = 0x1182;
	buf[0] = 0x07;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	buf[0] = 0x7c;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_AECH,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x03;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_COM1,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;
	
	/* urb 379 */
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_AECHM,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_ADVFL,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* urb 391 */
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_ADVFH,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* urb 397 */
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_GAIN,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* urb 403 */	
	buf[0] = 0x78; /*buf[1] = 0x78;*/
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_BLUE,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;
	/* urb 409 */
	buf[0] = 0x78;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_RED,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;
	// 627f does the last two writes at once (2 byte write to OV965X_CTL_BLUE)
	
	command = 0x118c;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x118d;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x118e;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x118f;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	/* urb 423 */
	command = 0x11ba;
	buf[0] = 0x0a;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x118c;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	/* urb 427 */
	buf[0] = 0x20;
	command = 0x118d;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	buf[0] = 0x20;
	command = 0x118e;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	buf[0] = 0x20;
	command = 0x118f;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	if (dev->webcam_model == MICRODIA_627F)
		buf[0] = 0x0e;
	else	// 624f
		buf[0] = 0x00;
	command = 0x118b;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	/* urb 435 */
	/* Gamma control? */
	command = 0x1190;
	if (dev->webcam_model == MICRODIA_627F)
	{
		buf[0] = 0x08; buf[1] = 0x23; buf[2] = 0x37; buf[3] = 0x48;
		buf[4] = 0x59; buf[5] = 0x68; buf[6] = 0x77; buf[7] = 0x85;
		buf[8] = 0x93; buf[9] = 0xa1; buf[10] = 0xae; buf[11] = 0xba;
		buf[12] = 0xc7; buf[13] = 0xd3; buf[14] = 0xe0; buf[15] = 0xeb;
		buf[16] = 0xf7;
	}
	else	// 624f
	{
		/*buf[0] = 0x0a; buf[1] = 0x37; buf[2] = 0x58; buf[3] = 0x70;
		buf[4] = 0x81; buf[5] = 0x8f; buf[6] = 0x9c; buf[7] = 0xa9;
		buf[8] = 0xb5; buf[9] = 0xbe; buf[10] = 0xc8; buf[11] = 0xd1;
		buf[12] = 0xda; buf[13] = 0xe2; buf[14] = 0xec; buf[15] = 0xf0;
		buf[16] = 0xf5;*/
	
		/* Old one */
		buf[0] = 0x00; buf[1] = 0x2a; buf[2] = 0x42; buf[3] = 0x56;
		buf[4] = 0x67; buf[5] = 0x78; buf[6] = 0x87; buf[7] = 0x95;
		buf[8] = 0xa2; buf[9] = 0xaf; buf[10] = 0xbc; buf[11] = 0xc8;
		buf[12] = 0xd4; buf[13] = 0xdf; buf[14] = 0xea; buf[15] = 0xf5;
		buf[16] = 0xff;
	}
	ret = usb_microdia_control_write(dev, command, buf, 17);
	if(ret < 0) goto err;

	if (dev->webcam_model == MICRODIA_627F)
	{
		buf[0] = 0xf3; buf[1] = 0x0f; buf[2] = 0xf3; buf[3] = 0x0f;
		buf[4] = 0x0f; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00;
		buf[8] = 0x00; buf[9] = 0x00; buf[10] = 0x00; buf[11] = 0x00;
		buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00; buf[15] = 0x00;
		buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x0c; buf[19] = 0x00;
		buf[20] = 0x00;
	}
	else	// 624f
		memset(buf, 0, 21);
	command = 0x10e1;
	for(i = 0; i < 8; i++)
		ret = usb_microdia_control_write(dev, command, buf, 21);

	if (dev->webcam_model == MICRODIA_627F)
	{
		buf[0] = 0x0f; buf[1] = 0x00; buf[2] = 0x15; buf[3] = 0x00;
		buf[4] = 0x1a; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00;
		buf[8] = 0x00; buf[9] = 0x00; buf[10] = 0x00; buf[11] = 0x00;
		buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00; buf[15] = 0x00;
		buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x0c; buf[19] = 0x00;
		buf[20] = 0x00;
	}
	else	// 624f
	{
		/* urb 453 */
		buf[0] = 0x15; buf[2] = 0x29;
	}
	ret = usb_microdia_control_write(dev, command, buf, 21);
	if (ret < 0) goto err;

	command = 0x10f7;
	buf[0] = 0x05;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x10f6;	// picture sharpness
	if (dev->webcam_model == MICRODIA_627F)
		buf[0] = 0x80;
	else	// 624f
		buf[0] = 0x00;	// 624f-log: 0xc0 !
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	/* urb 459 */
	command = 0x10e1;
	if (dev->webcam_model == MICRODIA_627F)
	{
		buf[0] = 0x0f; buf[1] = 0x00; buf[2] = 0x15; buf[3] = 0x00;
		buf[4] = 0x1a; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00;
		buf[8] = 0x00; buf[9] = 0x00; buf[10] = 0x00; buf[11] = 0x00;
		buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00; buf[15] = 0x00;
		buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x0c; buf[19] = 0x00;
		buf[20] = 0x00;
	}
	else	// 624f
	{
		memset(buf, 0, 21);
		buf[0] = 0x15; buf[2] = 0x29; buf [4] = 0x08;
	}
	ret = usb_microdia_control_write(dev, command, buf, 21);
	if(ret < 0) goto err;

	command = 0x10f8;
	buf[0] = 0x14;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	/* urb 463 */
	command = 0x10fa;
	buf[0] = 0xff;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x10f9;
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	/* urb 467 */
	command = 0x10f9;
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	command = 0x11ba;
	buf[0] = 0x0e;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	//memset(buf, 0, 4);
	command = 0x11bc;
	buf[0] = 0x80; buf[1] = 0x02; buf[2] = 0xe0; buf[3] = 0x01;
	ret = usb_microdia_control_write(dev, command, buf, 4);
	if(ret < 0) goto err;

	command = 0x11c0;
	if (dev->webcam_model == MICRODIA_627F)
	{
		buf[0] = 0x19; buf[1] = 0x31; buf[2] = 0x44; buf[3] = 0x50;
		buf[4] = 0x52; buf[5] = 0x56; buf[6] = 0x5b; buf[7] = 0x61;
		buf[8] = 0x65; buf[9] = 0x6a; buf[10] = 0x6c; buf[11] = 0x6e;
		buf[12] = 0x70; buf[13] = 0x72; buf[14] = 0x74; buf[15] = 0x76;
		buf[16] = 0x1a; buf[17] = 0x31; buf[18] = 0x44; buf[19] = 0x51;
		buf[20] = 0x54; buf[21] = 0x57; buf[22] = 0x5c; buf[23] = 0x61;
		buf[24] = 0x66; buf[25] = 0x6b; buf[26] = 0x6d; buf[27] = 0x6f;
		buf[28] = 0x71; buf[29] = 0x73; buf[30] = 0x75; buf[31] = 0x76;
		buf[32] = 0x1b; buf[33] = 0x32; buf[34] = 0x44; buf[35] = 0x50;
		buf[36] = 0x50; buf[37] = 0x54; buf[38] = 0x59; buf[39] = 0x5e;
		buf[40] = 0x63; buf[41] = 0x67; buf[42] = 0x6a; buf[43] = 0x6c;
		buf[44] = 0x6e; buf[45] = 0x70; buf[46] = 0x72; buf[47] = 0x73;
	}
	else	// 624f
	{
		buf[0] = 0x10; buf[1] = 0x21; buf[2] = 0x34; buf[3] = 0x40;
		buf[4] =0x47; buf[5] = 0x4F; buf[6] = 0x57; buf[7] = 0x5F;
		buf[8] = 0x64; buf[9] = 0x68; buf[10] = 0x6D; buf[11] = 0x73;
		buf[12] = 0x79; buf[13] = 0x80; buf[14] = 0x89; buf[15] = 0x97;
		buf[16] = 0x0D; buf[17] = 0x1C; buf[18] = 0x2A; buf[19] = 0x33;
		buf[20] = 0x38; buf[21] = 0x3D; buf[22] = 0x44; buf[23] = 0x4A;
		buf[24] = 0x4E; buf[25] = 0x52; buf[26] = 0x56; buf[27] = 0x5B;
		buf[28] = 0x61; buf[29] = 0x68; buf[30] = 0x6F; buf[31] = 0x7A;
		buf[32] = 0x0D; buf[33] = 0x1A; buf[34] = 0x2A; buf[35] = 0x31;
		buf[36] = 0x36; buf[37] = 0x3B; buf[38] = 0x41; buf[39] = 0x47;
		buf[40] = 0x4A; buf[41] = 0x4E; buf[42] = 0x53; buf[43] = 0x58;
		buf[44] = 0x5D; buf[45] = 0x64; buf[46] = 0x6B; buf[47] = 0x76;
	}
	ret = usb_microdia_control_write(dev, command, buf, 48);

	command = 0x118c;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	/* urb 477 */
	command = 0x118d;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	command = 0x118e;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x118f;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	/* urb 483 */
	command = 0x11a5;
	buf[0] = 0x2d; buf[1] = 0x2d; buf[2] = 0x3a; buf[3] = 0x05; 
	buf[4] = 0x04; buf[5] = 0x3f;
	ret = usb_microdia_control_write(dev, command, buf, 6);
	if(ret < 0) goto err;
	
	/* urb 485 */
	command = 0x11af;
	buf[0] = 0x28; buf[1] = 0xd8; buf[2] = 0x14; buf[3] = 0xec;
	ret = usb_microdia_control_write(dev, command, buf, 4);
	if(ret < 0) goto err;

	command = 0x11b3;
	buf[0] = 0x32; buf[1] = 0xdd; buf[2] = 0x32; buf[3] = 0xdd;
	ret = usb_microdia_control_write(dev, command, buf, 4);
	if(ret < 0) goto err;

	/* urb 489 */
	command = 0x10e0;
	buf[0] = 0x07;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	command = 0x1061;
	buf[0] = 0x01;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	command = 0x10e0;
	buf[0] = 0x27;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	
//////////////////////////////////////////////////////////////////////
	/* urb 495 */
	command = 0x1100;
	buf[0] = 0x0d; buf[1] = 0x08; buf[2] = 0x08; buf[3] = 0x0d;
	buf[4] = 0x08; buf[5] = 0x08; buf[6] = 0x0d; buf[7] = 0x0d;
	buf[8] = 0x0d; buf[9] = 0x0d; buf[10] = 0x11; buf[11] = 0x0d;
	buf[12] = 0x0d; buf[13] = 0x11; buf[14] = 0x15; buf[15] = 0x21;
	buf[16] = 0x15; buf[17] = 0x15; buf[18] = 0x11; buf[19] = 0x11;
	buf[20] = 0x15; buf[21] = 0x2a; buf[22] = 0x1d; buf[23] = 0x1d;
	buf[24] = 0x19; buf[25] = 0x21; buf[26] = 0x32; buf[27] = 0x2a;
	buf[28] = 0x32; buf[29] = 0x32; buf[30] = 0x2e; buf[31] = 0x2a;
	buf[32] = 0x2e; buf[33] = 0x2e; buf[34] = 0x36; buf[35] = 0x3a;
	buf[36] = 0x4b; buf[37] = 0x43; buf[38] = 0x36; buf[39] = 0x3a;
	buf[40] = 0x47; buf[41] = 0x3a; buf[42] = 0x2e; buf[43] = 0x2e;
	buf[44] = 0x43; buf[45] = 0x5c; buf[46] = 0x43; buf[47] = 0x47;
	buf[48] = 0x4f; buf[49] = 0x54; buf[50] = 0x58; buf[51] = 0x58;
	buf[52] = 0x58; buf[53] = 0x32; buf[54] = 0x3f; buf[55] = 0x60;
	buf[56] = 0x64; buf[57] = 0x5c; buf[58] = 0x54; buf[59] = 0x64;
	buf[60] = 0x4b; buf[61] = 0x54; buf[62] = 0x58; buf[63] = 0x54;
	ret = usb_microdia_control_write(dev, command, buf, 64);
	if(ret < 0) goto err;

	command = 0x1140;
	buf[0] = 0x0d; buf[1] = 0x11; buf[2] = 0x11; buf[3] = 0x15;
	buf[4] = 0x11; buf[5] = 0x15; buf[6] = 0x26; buf[7] = 0x15;
	buf[8] = 0x15; buf[9] = 0x26; buf[10] = 0x54; buf[11] = 0x36;
	buf[12] = 0x2e; buf[13] = 0x36; buf[14] = 0x54; buf[15] = 0x54;
	buf[16] = 0x54; buf[17] = 0x54; buf[18] = 0x54; buf[19] = 0x54;
	buf[20] = 0x54; buf[21] = 0x54; buf[22] = 0x54; buf[23] = 0x54;
	buf[24] = 0x54; buf[25] = 0x54; buf[26] = 0x54; buf[27] = 0x54;
	buf[28] = 0x54; buf[29] = 0x54; buf[30] = 0x54; buf[31] = 0x54;
	buf[32] = 0x54; buf[33] = 0x54; buf[34] = 0x54; buf[35] = 0x54;
	buf[36] = 0x54; buf[37] = 0x54; buf[38] = 0x54; buf[39] = 0x54;
	buf[40] = 0x54; buf[41] = 0x54; buf[42] = 0x54; buf[43] = 0x54;
	buf[44] = 0x54; buf[45] = 0x54; buf[46] = 0x54; buf[47] = 0x54;
	buf[48] = 0x54; buf[49] = 0x54; buf[50] = 0x54; buf[51] = 0x54;
	buf[52] = 0x54; buf[53] = 0x54; buf[54] = 0x54; buf[55] = 0x54;
	buf[56] = 0x54; buf[57] = 0x54; buf[58] = 0x54; buf[59] = 0x54;
	buf[60] = 0x54; buf[61] = 0x54; buf[62] = 0x54; buf[63] = 0x54;
	ret = usb_microdia_control_write(dev, command, buf, 64);
	if(ret < 0) goto err;

	/* urb 499 */
	command = 0x10e0;
	buf[0] = 0x07;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x1061;
	buf[0] = 0x03;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	/* urb 503 */
	command = 0x10e0;
	buf[0] = 0x0b;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x1189;
	buf[0] = 0x8c;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	buf[0] = 0x40;
	command = 0x11bc;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	buf[0] = 0x01;
	command = 0x11bd;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	
	buf[0] = 0xf0;
	command = 0x11be;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	buf[0] = 0x00;
	command = 0x11bf;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	/* urb 515 */
	command = 0x1180;
	buf[0] = 0x02; buf[1] = 0x00; buf[2] = 0x07; buf[3] = 0x00;
	buf[4] = 0x28; buf[5] = 0x3c;
	ret = usb_microdia_control_write(dev, command, buf, 6);
	if(ret < 0) goto err;

	command = 0x10fb;
	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00; buf[3] = 0xf0;
	buf[4] = 0x00;
	ret = usb_microdia_control_write(dev, command, buf, 5);
	if(ret < 0) goto err;

	command = 0x1189;
	buf[0] = 0x8c;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x11a1;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x6a; buf[3] = 0x50;
	ret = usb_microdia_control_write(dev, command, buf, 4);
	if(ret < 0) goto err;

	/* 523 */
	command = 0x11ab;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x50; buf[3] = 0x3c;
	ret = usb_microdia_control_write(dev, command, buf, 4);
	if(ret < 0) goto err;

	command = 0x1061;
	buf[0] = 0x03;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x11ba;
	buf[0] = 0x0e;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x11b9;
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x11ba;
	buf[0] = 0x0f;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x1061;
	buf[0] = 0x01;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x1000;
	buf[0] = 0x78;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x1002;
	buf[0] = 0x18;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	/* urb 539 */
	command = 0x1002;
	buf[0] = 0x18;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x11b8;
	buf[0] = 0x7a;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x118a;
	buf[0] = 0x04;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	/* urb 545 */
	command = 0x0395;
	buf[0] = 0x04;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x11b8;
	if (last_11b8[0] == 0x3c)
		buf[0] = 0x78;
	else if (last_11b8[0] == 0xbc)
		buf[0] = 0xf8;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, command, buf, 1);	// 624f: 0xb8	627f: 0x38
	if(ret < 0) goto err;
	if (buf[0] == 0x38)
		buf[0] = 0x79;
	else if (buf[0] == 0xb8)
		buf[0] = 0xf9;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, command, buf, 1);	// 624f: 0xb9	627f: 0x39
	if(ret < 0) goto err;
	if (buf[0] == 0x39)
		buf[0] = 0x7a;
	else if (buf[0] == 0xb9)
		buf[0] = 0xfa;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, command, buf, 1);	// 0x3a
	if(ret < 0) goto err;
	if (buf[0] == 0x3a)
		buf[0] = 0x7b;
	else if (buf[0] == 0xba)
		buf[0] = 0xfb;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, command, buf, 1);	// 624f: 0x3b	627f: 0xbb
	if(ret < 0) goto err;
	if (buf[0] == 0x3b)
		buf[0] = 0x7c;
	else if (buf[0] == 0xbb)
		buf[0] = 0xfc;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, command, buf, 1);	// 624f: 0x3c	627f: 0xbc
	if(ret < 0) goto err;
	buf[0] = buf[0] & 0xf0;	// 0xf0 or 0x70
	if (dev->webcam_model == MICRODIA_627F)
		buf[0] = buf[0] | 0x09;	// THIS VALUE SEEMS TO DEPEND ON WHAT WAS READ/WRITTEN IN THE LAST 5 READS/WRITES
	else	// 624f
		buf[0] = buf[0] | 0x08;	// THIS VALUE SEEMS TO DEPEND ON WHAT WAS READ/WRITTEN IN THE LAST 5 READS/WRITES
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	buf[0] = 0x80;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_CLKRC,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_EXHCH,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* urb 581 */
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_EXHCL,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* urb 587 */
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_DM_LNL,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	/* urb 593 */
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_DM_LNH,
		dev->sensor_flags, buf);
	if(ret < 0) goto err;

	ret = sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_PID,
		dev->sensor_flags, buf);
	/* returns 96, no idea what we're supposed to do with it */
	if(ret < 0) goto err;
	//msg("sensor pid, should be 0x96: %x", buf[0]);

	/* urb 609 */
	ret = sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_VER,
		dev->sensor_flags, buf);
	/* returns 52, no idea what we're supposed to do with it */
	if(ret < 0) goto err;
	//msg("sensor ver, should be 0x52: %x", buf[0]);

	/* urb 619 */
	ret = sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_MIDH,
		dev->sensor_flags, buf);
	/* returns 7f, no idea what we're supposed to do with it */
	if(ret < 0) goto err;
	//msg("sensor midh, should be 0x7f: %x", buf[0]);

	/* urb 629 */
	ret = sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 1, OV965X_CTL_MIDL,
		dev->sensor_flags, buf);
	/* returns a2, no idea what we're supposed to do with it */
	if(ret < 0) goto err;
	//msg("sensor midl, should be 0xa2: %x", buf[0]);

	command = 0x1061;
	buf[0] = 0x03;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;
//////////////////////////////////////////////////////////////////////////////////
//STREAM STARTED!
//////////////////////////////////////////////////////////////////////////////////

/* --- ADJUSTMENTS OF SHARPNESS, COLORS, CONTRAST, BRIGHTNESS ---
 * SHOULD BE REMOVED (writes to 0x10e1 and 0x10f6 are done again afterwards by dev_microdia_camera_settings() )
 * => NOT done by device 627f
 * => NOT in the 624f-log from 15-feb-08 at files.zenum.net
 */
/*
	command = 0x10f6;
	buf[0] = 0x80;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	command = 0x10e1;
	buf[0] = 0x15; buf[1] = 0x00; buf[2] = 0x29; buf[3] = 0x00;
	buf[4] = 0x08; buf[5] = 0x00; buf[6] = 0xF7; buf[7] = 0x0F;
	buf[8] = 0xC2; buf[9] = 0x0F; buf[10] = 0x47; buf[11] = 0x00; 
	buf[12] = 0x36; buf[13] = 0x00; buf[14] = 0xDD ; buf[15] = 0x0F;
	buf[16] = 0xEE; buf[17] = 0x0F; buf[18] = 0x00; buf[19] = 0x00; 
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, command, buf, 21);
	if(ret < 0) goto err;

	buf[0] = 0x7d; buf[1] = 0x00;
	ret = sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 1, 0x2d,
		dev->sensor_flags, buf);
	
	buf[0] = 0x0f;
	ret = sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 1, 0x00,
		dev->sensor_flags, buf);

	// urb 679
	command = 0x10e1;
	buf[0] = 0x15; buf[1] = 0x00; buf[2] = 0x29; buf[3] = 0x00;
	buf[4] = 0x08; buf[5] = 0x00; buf[6] = 0xF7; buf[7] = 0x0F;
	buf[8] = 0xC2; buf[9] = 0x0F; buf[10] = 0x47; buf[11] = 0x00; 
	buf[12] = 0x36; buf[13] = 0x00; buf[14] = 0xDD ; buf[15] = 0x0F;
	buf[16] = 0xEE; buf[17] = 0x0F; buf[18] = 0x00; buf[19] = 0x00; 
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, command, buf, 21);
	if(ret < 0) goto err;
	
	// These 2 regs controls brightness of image 
	// (night mode or something like that)

	ret = sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 2, 0x2d,
		dev->sensor_flags, buf);
	//msg("Old brightness value: %x %x", buf[0], buf[1]);
	buf[0] = 0x00; buf[1] = 0x04;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x2d,
		dev->sensor_flags, buf);

	// urb 743
	command = 0x10f6;
	buf[0] = 0x74;
	ret = usb_microdia_control_write(dev, command, buf, 1);

	// urb 745
	command = 0x10e1;
	buf[0] = 0x15; buf[1] = 0x00; buf[2] = 0x29; buf[3] = 0x00;
	buf[4] = 0x08; buf[5] = 0x00; buf[6] = 0xF7; buf[7] = 0x0F;
	buf[8] = 0xC2; buf[9] = 0x0F; buf[10] = 0x47; buf[11] = 0x00; 
	buf[12] = 0x36; buf[13] = 0x00; buf[14] = 0xdd ; buf[15] = 0x0F;
	buf[16] = 0xEE; buf[17] = 0x0F; buf[18] = 0x00; buf[19] = 0x00; 
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, command, buf, 21);
	if(ret < 0) goto err;
	return ret;

	buf[0] = 0xe8;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, 0x01,
		dev->sensor_flags, buf);
	// THIS WRITE WILL NEVER BE DONE, because function returned before
*/
	return 0;

err:
	UDIA_ERROR("start stream: command %x failed (%d)!\n", command, ret);
	return ret;
}

int microdia_6260_start_stream(struct usb_microdia *dev)
{
	int ret;
	__u8 buf[32];

	ret = usb_microdia_control_read(dev, 0x1045, buf, 1);
	buf[0] = 0x1f;
	ret = usb_microdia_control_write(dev, 0x1045, buf, 1);
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, 0x1066, buf, 1);
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, 0x1007, buf, 1);
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, 0x1006, buf, 1);
	buf[0] = 0x78;
	ret = usb_microdia_control_write(dev, 0x1000, buf, 1);
	buf[0] = 0xc7;
	ret = usb_microdia_control_write(dev, 0x1001, buf, 1);
	buf[0] = 0x18;
	ret = usb_microdia_control_write(dev, 0x1002, buf, 1);
	buf[0] = 0x01;
	ret = usb_microdia_control_write(dev, 0x1061, buf, 1);
	buf[0] = 0x80;
	ret = usb_microdia_control_write(dev, 0x1020, buf, 1);
	buf[0] = 0x40;
	buf[1] = 0x30;
	buf[2] = 0x20;
	buf[3] = 0x10;
	buf[4] = 0x08;
	ret = usb_microdia_control_write(dev, 0x1067, buf, 5);
	buf[0] = 0x80;
	buf[1] = 0x21;
	buf[2] = 0x00;
	buf[3] = 0x00;
	buf[4] = 0x00;
	buf[5] = 0x00;
	buf[6] = 0x00;
	buf[7] = 0x00;
	buf[8] = 0x03;
	ret = usb_microdia_control_write(dev, 0x10c0, buf, 9);
	buf[0] = 0x45;
	buf[1] = 0x18;
	buf[2] = 0x00;
	buf[3] = 0x30;
	buf[4] = 0x00;
	buf[5] = 0x09;
	buf[6] = 0x00;
	buf[7] = 0xed;
	buf[8] = 0x0f;
	buf[9] = 0xda;
	buf[10] = 0x0f;
	buf[11] = 0x3a;
	buf[12] = 0x00;
	buf[13] = 0x3a;
	buf[14] = 0x00;
	buf[15] = 0xd0;
	buf[16] = 0x0f;
	buf[17] = 0xf7;
	buf[18] = 0x0f;
	buf[19] = 0x00;
	buf[20] = 0x00;
	buf[21] = 0x00;
	buf[22] = 0x60;
	buf[23] = 0x0a;
	ret = usb_microdia_control_write(dev, 0x10e0, buf, 24);
	buf[0] = 0x00;
	buf[1] = 0x00;
	buf[2] = 0x00;
	ret = usb_microdia_control_write(dev, 0x10f8, buf, 3);
	buf[0] = 0x00;
	buf[1] = 0xa0;
	buf[2] = 0x00;
	buf[3] = 0xf0;
	buf[4] = 0x00;
	ret = usb_microdia_control_write(dev, 0x10fb, buf, 5);
	buf[0] = 0x8a;
	buf[1] = 0x8c;
	buf[2] = 0x08;
	ret = usb_microdia_control_write(dev, 0x1188, buf, 3);
	buf[0] = 0x00;
	buf[1] = 0x20;
	buf[2] = 0x20;
	buf[3] = 0x20;
	buf[4] = 0x20;
	buf[5] = 0x00;
	buf[6] = 0x24;
	buf[7] = 0x3b;
	buf[8] = 0x4f;
	buf[9] = 0x61;
	buf[10] = 0x71;
	buf[11] = 0x80;
	buf[12] = 0x8f;
	buf[13] = 0x9d;
	buf[14] = 0xab;
	buf[15] = 0xb8;
	buf[16] = 0xc4;
	buf[17] = 0xd1;
	buf[18] = 0xdd;
	buf[19] = 0xe9;
	buf[20] = 0xf4;
	buf[21] = 0xff;
	ret = usb_microdia_control_write(dev, 0x118b, buf, 22);
	buf[0] = 0x00;
	buf[1] = 0x00;
	ret = usb_microdia_control_write(dev, 0x11a1, buf, 2);
	buf[0] = 0x60;
	buf[1] = 0x28;
	buf[2] = 0x00;
	buf[3] = 0x00;
	buf[4] = 0x00;
	buf[5] = 0x40;
	buf[6] = 0x01;
	buf[7] = 0xf0;
	buf[8] = 0x00;
	ret = usb_microdia_control_write(dev, 0x11b7, buf, 9);
	buf[0] = 0x78;
	ret = usb_microdia_control_write(dev, 0x1000, buf, 1);
	buf[0] = 0x38;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);

	ret = ov7670_initialise(dev);

	ret = sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 1, 0x3b, dev->sensor_flags, buf);
	ret = usb_microdia_control_read(dev, 0x10c1, buf, 1);
	buf[0] = 0x50;
	ret = usb_microdia_control_write(dev, 0x10c1, buf, 1);

	// TRY TO READ FROM EEPROM:
	ret = sn9c20x_read_i2c_data(dev, 0x50, 1, 0x00, dev->sensor_flags, buf);
	if (ret < 0)
	{
		UDIA_INFO("No EEPROM found\n");
	}
	else
	{
		UDIA_INFO("Read from EEPROM successful\n");
	}

	buf[0] = 0x21;
	ret = usb_microdia_control_write(dev, 0x10c1, buf, 1);
	buf[0] = 0x45;
	ret = usb_microdia_control_write(dev, 0x10e0, buf, 1);
	buf[0] = 0xc6;
	ret = usb_microdia_control_write(dev, 0x1001, buf, 1);
	buf[0] = 0xc4;
	ret = usb_microdia_control_write(dev, 0x1001, buf, 1);
	buf[0] = 0x00;
	buf[1] = 0x00;
	buf[2] = 0x01;
	buf[3] = 0x00;
	buf[4] = 0x50;
	buf[5] = 0x3c;
	ret = usb_microdia_control_write(dev, 0x1180, buf, 6);
	buf[0] = 0x00;
	buf[1] = 0x40;
	buf[2] = 0x00;
	buf[3] = 0xf0;
	buf[4] = 0x01;
	ret = usb_microdia_control_write(dev, 0x10fb, buf, 5);
	buf[0] = 0x80;
	ret = usb_microdia_control_write(dev, 0x1189, buf, 1);
	buf[0] = 0x00;
	buf[1] = 0x00;
	buf[2] = 0xd5;
	buf[3] = 0x50;
	ret = usb_microdia_control_write(dev, 0x11a1, buf, 4);
	buf[0] = 0x00;
	buf[1] = 0x00;
	buf[2] = 0xa0;
	buf[3] = 0x3c;
	ret = usb_microdia_control_write(dev, 0x11ab, buf, 4);
	buf[0] = 0x78;
	ret = usb_microdia_control_write(dev, 0x1000, buf, 1);
	buf[0] = 0x10;
	ret = usb_microdia_control_write(dev, 0x1002, buf, 1);
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, 0x1002, buf, 1);
	buf[0] = 0x38;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	buf[0] = 0x04;
	ret = usb_microdia_control_write(dev, 0x118a, buf, 1);
	buf[0] = 0x04;
	ret = usb_microdia_control_write(dev, 0x0395, buf, 1);
	buf[0] = 0x78;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	ret = usb_microdia_control_read(dev, 0x11b8, buf, 1);
	buf[0] = 0xf9;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	ret = usb_microdia_control_read(dev, 0x11b8, buf, 1);
	buf[0] = 0xfa;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	ret = usb_microdia_control_read(dev, 0x11b8, buf, 1);
	buf[0] = 0x7b;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	ret = usb_microdia_control_read(dev, 0x11b8, buf, 1);
	buf[0] = 0x7c;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	ret = usb_microdia_control_read(dev, 0x11b8, buf, 1);
	buf[0] = 0x7d;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	ret = usb_microdia_control_read(dev, 0x11b8, buf, 1);
	buf[0] = 0x7b;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	/*
	 * OV7670 Intialise Part 2
	 */
	buf[0] = 0x00;
	buf[1] = 0x00;
	buf[2] = 0x01;
	buf[3] = 0x00;
	buf[4] = 0x50;
	buf[5] = 0x3c;
	ret = usb_microdia_control_write(dev, 0x1180, buf, 6);
	buf[0] = 0x00;
	buf[1] = 0x40;
	buf[2] = 0x00;
	buf[3] = 0xf0;
	buf[4] = 0x01;
	ret = usb_microdia_control_write(dev, 0x10fb, buf, 5);
	buf[0] = 0x80;
	ret = usb_microdia_control_write(dev, 0x1189, buf, 1);
	buf[0] = 0x00;
	buf[1] = 0x00;
	buf[2] = 0xd5;
	buf[3] = 0x50;
	ret = usb_microdia_control_write(dev, 0x11a1, buf, 4);
	buf[0] = 0x00;
	buf[1] = 0x00;
	buf[2] = 0xa0;
	buf[3] = 0x3c;
	ret = usb_microdia_control_write(dev, 0x11ab, buf, 4);
	buf[0] = 0x03;
	ret = usb_microdia_control_write(dev, 0x1061, buf, 1);
	buf[0] = 0x01;
	ret = usb_microdia_control_write(dev, 0x1061, buf, 1);
	buf[0] = 0x78;
	ret = usb_microdia_control_write(dev, 0x1000, buf, 1);
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, 0x1002, buf, 1);
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, 0x1002, buf, 1);
	buf[0] = 0x7b;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	buf[0] = 0x04;
	ret = usb_microdia_control_write(dev, 0x118a, buf, 1);
	buf[0] = 0x04;
	ret = usb_microdia_control_write(dev, 0x0395, buf, 1);
	buf[0] = 0x78;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	ret = usb_microdia_control_read(dev, 0x11b8, buf, 1);
	buf[0] = 0x79;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	ret = usb_microdia_control_read(dev, 0x11b8, buf, 1);
	buf[0] = 0x7a;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	ret = usb_microdia_control_read(dev, 0x11b8, buf, 1);
	buf[0] = 0x7b;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	ret = usb_microdia_control_read(dev, 0x11b8, buf, 1);
	buf[0] = 0x7c;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	ret = usb_microdia_control_read(dev, 0x11b8, buf, 1);
	buf[0] = 0x7d;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	ret = usb_microdia_control_read(dev, 0x11b8, buf, 1);
	buf[0] = 0xfa;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	/*
	 * OV7670 Initialisation Part 3 Goes Here
	 */
	buf[0] = 0x03;
	ret = usb_microdia_control_write(dev, 0x1061, buf, 1);
	buf[0] = 0x00;
	buf[1] = 0x00;
	buf[2] = 0x00;
	buf[3] = 0x00;
	buf[4] = 0x00;
	buf[5] = 0x00;
	buf[6] = 0x00;
	buf[7] = 0x00;
	buf[8] = 0x00;
	buf[9] = 0x00;
	buf[10] = 0x00;
	buf[11] = 0x00;
	buf[12] = 0x00;
	buf[13] = 0x00;
	buf[14] = 0x00;
	buf[15] = 0x00;
	buf[16] = 0x00;
	buf[17] = 0x00;
	buf[18] = 0x0a;
	buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, 0x10e1, buf, 21);
	buf[0] = 0x00;
	buf[1] = 0x00;
	buf[2] = 0x00;
	buf[3] = 0x00;
	buf[4] = 0x00;
	buf[5] = 0x00;
	buf[6] = 0x00;
	buf[7] = 0x00;
	buf[8] = 0x00;
	buf[9] = 0x00;
	buf[10] = 0x00;
	buf[11] = 0x00;
	buf[12] = 0x00;
	buf[13] = 0x00;
	buf[14] = 0x00;
	buf[15] = 0x00;
	buf[16] = 0x00;
	buf[17] = 0x00;
	buf[18] = 0x0a;
	buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, 0x10e1, buf, 21);
	buf[0] = 0x00;
	buf[1] = 0x00;
	buf[2] = 0x00;
	buf[3] = 0x00;
	buf[4] = 0x00;
	buf[5] = 0x00;
	buf[6] = 0x00;
	buf[7] = 0x00;
	buf[8] = 0x00;
	buf[9] = 0x00;
	buf[10] = 0x00;
	buf[11] = 0x00;
	buf[12] = 0x00;
	buf[13] = 0x00;
	buf[14] = 0x00;
	buf[15] = 0x00;
	buf[16] = 0x00;
	buf[17] = 0x00;
	buf[18] = 0x0a;
	buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, 0x10e1, buf, 21);
	buf[0] = 0x00;
	buf[1] = 0x00;
	buf[2] = 0x00;
	buf[3] = 0x00;
	buf[4] = 0x00;
	buf[5] = 0x00;
	buf[6] = 0x00;
	buf[7] = 0x00;
	buf[8] = 0x00;
	buf[9] = 0x00;
	buf[10] = 0x00;
	buf[11] = 0x00;
	buf[12] = 0x00;
	buf[13] = 0x00;
	buf[14] = 0x00;
	buf[15] = 0x00;
	buf[16] = 0x00;
	buf[17] = 0x00;
	buf[18] = 0x0a;
	buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, 0x10e1, buf, 21);

	return ret;
}

/**
 * @brief From 7 start-stop-logs (made by Alex Carlos, Boris Borisov, bohzan and Comer352l)
 *
 * @param dev
 *
 * @returns 0 (ok) or -1 (error)
 *
 * @author Comer352l
 *
 * Windows driver versions: 5.7.23.000
 * Windows versions: 2000 and XP
 * USB-Logger: SniffUSB 1.8 and 2.0, USBTrace 2.0, SnoopyPro
 * All logs were made using AMCAP with 640x480, RGB24
 */
int microdia_6270_start_stream(struct usb_microdia *dev)
{
	__u8 buf[64];
	__u8 last_11b8[1];
	int k;
	int retI2C;

	// Check if sensor slave address is valid:
	if ((dev->sensor_slave_address != MT9V111_I2C_SLAVE_ADDRESS) && (dev->sensor_slave_address != MT9V011_I2C_SLAVE_ADDRESS))
	{
		UDIA_INFO("microdia_6270_start_stream(): Invalid sensor slave address !\n");
		return -1;
	}

	// <= INTERRUPT COMING BACK (URB 1953)
	// SWITCH TO ALTERNATE SETTINGS 8 (URB 1954)	// NOT IN ALEX'S LOG
	// => INTERRUPT GOING DOWN (URB 1955)

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x1066, buf, 1);		// URB 1956

	// <= INTERRUPT COMING BACK (URB 1955)
	// => INTERRUPT GOING DOWN (URB 1957)

	buf[0] = 0x78;
	usb_microdia_control_write(dev, 0x1000, buf, 1);		// URB 1958
	buf[0] = 0xc7;
	usb_microdia_control_write(dev, 0x1001, buf, 1);		// URB 1959
	buf[0] = 0x18;
	usb_microdia_control_write(dev, 0x1002, buf, 1);		// URB 1960
	buf[0] = 0x01;
	usb_microdia_control_write(dev, 0x1061, buf, 1);		// URB 1961
	buf[0] = 0x80;
	usb_microdia_control_write(dev, 0x1020, buf, 1);		// URB 1962
	buf[0] = 0x40; buf[1] = 0x30; buf[2] = 0x20; buf[3] = 0x10; buf[4] = 0x08;
	usb_microdia_control_write(dev, 0x1067, buf, 5);		// URB 1963

	// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// STRANGE I2C MESSAGE:
	// - 9 bytes
	// - byte 8 = 0x00 at first start + first cycle
	// - procedure not complete: no wait/check for ack/error, no readout of 0x10c2
	// - dummy writes at first cycle and read at second cycle, but there are many I2C messages in the meantime, so this should not work
	//
	// AT FIRST START + FIRST CYCLE:
	buf[0] = 0x80; buf[1] = dev->sensor_slave_address; buf[2] = 0x00; buf[3] = 0x00;
	buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x03;
	// AT ALL FURTHER STARTS + FIRST CYCLE:
	//MT9V111: I2C MESSAGE => IFP Register 0x33: Auto exposure
	//buf[0] = 0x90; buf[1] = dev->sensor_slave_address; buf[2] = 0x33; buf[3] = 0x00;
	//buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; buf[8] = 0x03;
	//MT9V011: ?????????????????????????
	//buf[0] = 0x90; buf[1] = dev->sensor_slave_address; buf[2] = 0x07; buf[3] = 0x00;
	//buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; buf[8] = 0x03;
	// ALWAYS AT 2nd AND FURTHER CYCLES:
	//MT9V111:
	//buf[0] = 0xa2; buf[1] = 0x5c; buf[2] = 0x00; buf[3] = 0x36;
	//buf[4] = 0x00; buf[5] = 0x82; buf[6] = 0x3a; buf[7] = 0x10; buf[8] = 0x03;
	// => buf[3] to buf[6] simply contain bytes read with URB 2215
	//MT9V011:
	//buf[0] = 0xb0; buf[1] = 0x5d; buf[2] = 0x07; buf[3] = 0x00;
	//buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; buf[8] = 0x03;
	usb_microdia_control_write(dev, 0x10c0, buf, 9);		// URB 1964
	// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	buf[0] = 0x45; buf[1] = 0x18; buf[2] = 0x00; buf[3] = 0x30;
	buf[4] = 0x00; buf[5] = 0x09; buf[6] = 0x00; buf[7] = 0xed;
	buf[8] = 0x0f; buf[9] = 0xda; buf[10] = 0x0f; buf[11] = 0x3a;
	buf[12] = 0x00; buf[13] = 0x3a; buf[14] = 0x00; buf[15] = 0xd0;
	buf[16] = 0x0f; buf[17] = 0xf7; buf[18] = 0x0f; buf[19] = 0x00;
	buf[20] = 0x00; buf[21] = 0x00; buf[22] = 0x60; buf[23] = 0x0a;
	usb_microdia_control_write(dev, 0x10e0, buf, 24);		// URB 1965
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00;
	usb_microdia_control_write(dev, 0x10f8, buf, 3);		// URB 1966
	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00; buf[3] = 0xf0; buf[4] = 0x00;
	usb_microdia_control_write(dev, 0x10fb, buf, 5);		// URB 1967
	buf[0] = 0x06; buf[1] = 0x0c; buf[2] = 0x08;
	usb_microdia_control_write(dev, 0x1188, buf, 3);		// URB 1968
	buf[0] = 0x00; buf[1] = 0x20; buf[2] = 0x20; buf[3] = 0x20;
	buf[4] = 0x20; buf[5] = 0x00; buf[6] = 0x24; buf[7] = 0x3b;
	buf[8] = 0x4f; buf[9] = 0x61; buf[10] = 0x71; buf[11] = 0x80;
	buf[12] = 0x8f; buf[13] = 0x9d; buf[14] = 0xab; buf[15] = 0xb8;
	buf[16] = 0xc4; buf[17] = 0xd1; buf[18] = 0xdd; buf[19] = 0xe9;
	buf[20] = 0xf4; buf[21] = 0xff;
	usb_microdia_control_write(dev, 0x118b, buf, 22);		// URB 1969
	buf[0] = 0x00; buf[1] = 0x00;
	usb_microdia_control_write(dev, 0x11a1, buf, 2);		// URB 1970
	buf[0] = 0x60; buf[1] = 0x28; buf[2] = 0x00; buf[3] = 0x08;
	buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00;
	buf[8] = 0x00;
	usb_microdia_control_write(dev, 0x11b7, buf, 9);		// URB 1971
	buf[0] = 0x38;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);		// URB 1972
	buf[0] = 0x78;
	usb_microdia_control_write(dev, 0x1000, buf, 1);		// URB 1973

	// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// I2C MESSAGES
	retI2C = 0;
	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		// Write to IFP register	0x0d: Color Correction Register 8
		buf[0] = 0x00; buf[1] = 0x01;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x0d, dev->sensor_flags, buf);	// URBs 1974-1976
		// Write to IFP register	0x0d: Color Correction Register 8
		buf[0] = 0x00; buf[1] = 0x00;	// ???
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x0d, dev->sensor_flags, buf);	// URBs 1977-1979
		// Writes to IFP registers	0x01: Register Address Space Selection
		//				0x02: Color Correction Register 1
		buf[0] = 0x00; buf[1] = 0x01;	// select IFP address space
		buf[2] = 0x00; buf[3] = 0x16;	// ???	// AT FIRST START + FIRST CYCLE: 0x00 0x00
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x01, dev->sensor_flags, buf);	// URBs 1980-1982
		// Writes to IFP registers	0x03: Color Correction Register 3
		//				0x04: Color Correction Register 4
		buf[0] = 0x01; buf[1] = 0xe1;	// ???	// AT FIRST START + FIRST CYCLE:  0x00 0x00
		buf[2] = 0x02; buf[3] = 0x81;	// ???	// AT FIRST START + FIRST CYCLE:  0x00 0x00
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x03, dev->sensor_flags, buf);	// URBs 1983-1985
		// Writes to IFP registers	0x05: Aperture Correction (Sharpening)
		//				0x06: Operating Mode Control
		buf[0] = 0x00; buf[1] = 0x04;	// 100% sharpening, no automatic sharpness reduction at low light	// AT FIRST START + FIRST CYCLE: 0x00 0x00: no sharpening
		buf[2] = 0x00; buf[3] = 0x00;	// stop AWB at the current values, no on-the-fly defect correction, no auto exposure, + RESREVED option
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x05, dev->sensor_flags, buf);	// URBs 1986-1988
		// Writes to IFP registers	0x07: Image Flow Processor Soft Reset
		//				0x08: Output Format Control
		buf[0] = 0x30; buf[1] = 0x02;	// reset	// AT FIRST START + FIRST CYCLE: 0x00 0x00
		buf[2] = 0x04; buf[3] = 0x80;	// bypass entire image processing, raw 8+2 Bayer data output directly
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x07, dev->sensor_flags, buf);	// URBs 1989-1991
		// "Dummy" write to IFP Register 0x11: Color Correction Register 12
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0, 0x11, dev->sensor_flags, NULL);	// URBs 1992-1993
		// Write to register		0x01: Register address space selection
		// Write to sensor register	0x02: Column Start
		buf[0] = 0x00; buf[1] = 0x04;	// select sensor address space
		buf[2] = 0x00; buf[3] = 0x16;	// start with column 22
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x01, dev->sensor_flags, buf);	// URBs 1994-1996
		// Writes to sensor registers	0x03: Window Height
		//				0x04: Window Width
		buf[0] = 0x01; buf[1] = 0xe6;	// 486
		buf[2] = 0x02; buf[3] = 0x86;	// 646
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x03, dev->sensor_flags, buf);	// URBs 1997-1999
		// Writes to sensor registers	0x05: Horizontal Blanking
		//				0x06: Vertical Blanking
		buf[0] = 0x00; buf[1] = 0x04;	// 4 columns (pixel clocks)	// AT FIRST START + FIRST CYCLE: 0x00 0x00
		buf[2] = 0x00; buf[3] = 0x00;	// 0 rows
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x05, dev->sensor_flags, buf);	// URBs 2000-2002
		// Writes to sensor registers	0x07: Output Control
		//				0x08: Row Start
		buf[0] = 0x30; buf[1] = 0x02;	// normal operation + chip enable + RESERVED options
		buf[2] = 0x00; buf[3] = 0x08;	// row 8
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x07, dev->sensor_flags, buf);	// URBs 2003-2005
		// "Dummy" write to sensor Register 0x11: UNDOCUMENTED
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0, 0x11, dev->sensor_flags, NULL);	// URBs 2006-2007
		// Writes to sensor registers	0x0c: Shutter Delay
		//				0x0d: Reset (Soft) (from MT9V011 datasheet)
		buf[0] = 0x00; buf[1] = 0x00;	// 0 master clocks
		buf[2] = 0x00; buf[3] = 0x00;	// return to normal operation
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x0c, dev->sensor_flags, buf);	// URBs 2008-2010
		// Writes to sensor registers	0x0e: UNDOCUMENTED
		//				0x0f: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x0e, dev->sensor_flags, buf);	// URBs 2011-2013
		// Writes to sensor registers	0x10: UNDOCUMENTED
		//				0x11: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x10, dev->sensor_flags, buf);	// URBs 2014-2016
		// Writes to sensor registers	0x12: 2X Zoom Col Start		=> 0x1e bit 0 must be set to activate zoom
		//				0x13: 2X Zoom Row Start		=> 0x1e bit 0 must be set to activate zoom
		buf[0] = 0x00; buf[1] = 0xb0;	// column 176
		buf[2] = 0x00; buf[3] = 0x7c;	// row 124
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x12, dev->sensor_flags, buf);	// URBs 2017-2018
		// Writes to sensor registers	0x14: UNDOCUMENTED
		//				0x15: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x14, dev->sensor_flags, buf);	// URBs 2019-2021
		// Writes to sensor registers	0x16: UNDOCUMENTED
		//				0x17: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x16, dev->sensor_flags, buf);	// URBs 2022-2024
		// Writes to sensor registers	0x18: UNDOCUMENTED
		//				0x19: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x18, dev->sensor_flags, buf);	// URBs 2025-2027
		// Writes to sensor registers	0x1a: UNDOCUMENTED
		//				0x1b: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x1a, dev->sensor_flags, buf);	// URBs 2028-2030
		// Writes to sensor registers	0x1c: UNDOCUMENTED
		//				0x1d: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x1c, dev->sensor_flags, buf);	// URBs 2031-2033
		// Write to sensor register	0x30: RESERVED
		buf[0] = 0x00; buf[1] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x30, dev->sensor_flags, buf);	// URBs 2034-2036
		// Write to sensor register	0x20: Read Mode
		buf[0] = 0x00; buf[1] = 0x00;	// normal readout
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x20, dev->sensor_flags, buf);	// URBs 2037-2039
		// Writes to sensor registers	0x30: RESERVED
		//				0x31: RESERVED
		buf[0] = 0x00; buf[1] = 0x05;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x30, dev->sensor_flags, buf);	// URBs 2040-2042
		// "Dummy" write to sensor Register 0x34: RESERVED
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0, 0x34, dev->sensor_flags, NULL);	// URBs 2043-2044
	}
	else if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		// Write to sensor register	0x07: Output Control
		buf[0] = 0x00; buf[1] = 0x02;	// chip enable, normal operation
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x07, dev->sensor_flags, buf);
		// Write to sensor register	0x0d: Soft Reset
		buf[0] = 0x00; buf[1] = 0x01;	// reset
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x0d, dev->sensor_flags, buf);
		// Write to sensor register	0x0d: Soft Reset
		buf[0] = 0x00; buf[1] = 0x00;	// resume operation
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x0d, dev->sensor_flags, buf);
		// Writes to sensor registers	0x01: Row start
		//				0x02: Column Start
		buf[0] = 0x00; buf[1] = 0x08;	// start with row 8
		buf[2] = 0x00; buf[3] = 0x16;	// start with column 22
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x01, dev->sensor_flags, buf);
		// Writes to sensor registers	0x03: Window Height
		//				0x04: Window Width
		buf[0] = 0x01; buf[1] = 0xe1;	// 481
		buf[2] = 0x02; buf[3] = 0x81;	// 641
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x03, dev->sensor_flags, buf);
		// Writes to sensor registers	0x05: Horizontal Blanking
		//				0x06: Vertical Blanking
		buf[0] = 0x00; buf[1] = 0x83;	// 131 columns (pixel clocks)
		buf[2] = 0x00; buf[3] = 0x06;	// 6 rows
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x05, dev->sensor_flags, buf);
		// Write to sensor register	0x0d: Soft Reset
		buf[0] = 0x00; buf[1] = 0x02;	// UNKNOWN
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x0d, dev->sensor_flags, buf);
		// Writes to sensor registers	0x0a: Pixel Clock Speed
		//				0x0b: Frame Restart
		buf[0] = 0x00; buf[1] = 0x00;	// default
		buf[2] = 0x00; buf[3] = 0x00;	// (has no effect/no restart)
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x0a, dev->sensor_flags, buf);
		// Writes to sensor registers	0x0c: Shutter Delay
		//				0x0d: Soft Reset
		buf[0] = 0x00; buf[1] = 0x00;	// 0 master clocks
		buf[2] = 0x00; buf[3] = 0x00;	// resume operation
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x0c, dev->sensor_flags, buf);
		// Writes to sensor registers	0x0e: UNDOCUMENTED
		//				0x0f: UNDOCUMENTED 
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x0e, dev->sensor_flags, buf);
		// Writes to sensor registers	0x10: UNDOCUMENTED
		//				0x11: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x10, dev->sensor_flags, buf);
		// Writes to sensor registers	0x12: 2X Zoom Column Start (from MT9V111 datasheet)
		//				0x13: 2X Zoom Row Start (from MT9V111 datasheet)
		buf[0] = 0x00; buf[1] = 0x00;	// column 0 => bit0 of reg 0x1e must be set to activate zoom
		buf[2] = 0x00; buf[3] = 0x00;	// row 0    => bit0 of reg 0x1e must be set to activate zoom
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x12, dev->sensor_flags, buf);
		// Writes to sensor registers	0x14: UNDOCUMENTED
		//				0x15: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x14, dev->sensor_flags, buf);
		// Writes to sensor registers	0x16: UNDOCUMENTED
		//				0x17: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x16, dev->sensor_flags, buf);
		// Writes to sensor registers	0x18: UNDOCUMENTED
		//				0x19: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x18, dev->sensor_flags, buf);
		// Writes to sensor registers	0x1a: UNDOCUMENTED
		//				0x1b: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x1a, dev->sensor_flags, buf);
		// Writes to sensor registers	0x1c: UNDOCUMENTED
		//				0x1d: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x1c, dev->sensor_flags, buf);
		// Write to sensor register	0x32: RESERVED
		buf[0] = 0x00; buf[1] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x32, dev->sensor_flags, buf);
		// Writes to sensor registers	0x20: Read Mode
		//				0x21: RESERVED
		buf[0] = 0x11; buf[1] = 0x01;	// output all frames (including bad frames) + UNDOCUMENTED
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x20, dev->sensor_flags, buf);
		// Writes to sensor registers	0x22: RESERVED
		//				0x23: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x22, dev->sensor_flags, buf);
		// Writes to sensor registers	0x24: UNDOCUMENTED
		//				0x25: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x24, dev->sensor_flags, buf);
		// Writes to sensor registers	0x26: UNDOCUMENTED
		//				0x27: RESERVED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x24;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x26, dev->sensor_flags, buf);
		// "Dummy" write to sensor Register 0x30: RESERVED
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0, 0x30, dev->sensor_flags, NULL);
		// Writes to sensor registers	0x2f: RESERVED
		//				0x30: RESERVED
		buf[0] = 0xf7; buf[1] = 0xb0;
		buf[2] = 0x00; buf[3] = 0x05;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x2f, dev->sensor_flags, buf);
		// Writes to sensor registers	0x31: RESERVED
		//				0x32: RESERVED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x31, dev->sensor_flags, buf);
		// Writes to sensor registers	0x33: RESERVED
		//				0x34: RESERVED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x01; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x33, dev->sensor_flags, buf);
		// "Dummy" write to sensor Register 0x3b: RESERVED
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0, 0x3b, dev->sensor_flags, NULL);
		// Write to sensor register	0x3d: RESERVED
		buf[0] = 0x06; buf[1] = 0x8f;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x3d, dev->sensor_flags, buf);
		// Writes to sensor registers	0x40: RESERVED
		//				0x41: RESERVED
		buf[0] = 0x01; buf[1] = 0xe0;
		buf[2] = 0x00; buf[3] = 0xd1;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x40, dev->sensor_flags, buf);
		// Write to sensor register	0x44: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x82;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x44, dev->sensor_flags, buf);
		// Writes to sensor registers	0x5a: RESERVED
		//				0x5b: RESERVED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x5a, dev->sensor_flags, buf);
		// Writes to sensor registers	0x5c: RESERVED
		//				0x5d: RESERVED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0x00; buf[3] = 0x00;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x5c, dev->sensor_flags, buf);
		// Writes to sensor registers	0x5e: RESERVED
		//				0x5f: RESERVED
		buf[0] = 0x00; buf[1] = 0x00;
		buf[2] = 0xa3; buf[3] = 0x1d;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x5e, dev->sensor_flags, buf);
		// "Dummy" write to sensor Register 0x68: UNDOCUMENTED
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0, 0x68, dev->sensor_flags, NULL);
		// Write to sensor register	0x62: RESERVED
		buf[0] = 0x06; buf[1] = 0x11;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x62, dev->sensor_flags, buf);
	}
	if (retI2C < 0)
	{
		UDIA_INFO("1 or more errors occured in I2C-message-block 1 !\n");
		return -1;
	}
	// END OF I2C MESSAGES
	// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x40;
		usb_microdia_control_write(dev, 0x1007, buf, 8);
		buf[0] = 0x40;
		usb_microdia_control_write(dev, 0x1006, buf, 8);
	}


	// *** TEMPORARY SWITCH TO SLAVE ADDRESS 0x50, I2C READ ATTEMPT:
	// - WE CURRENTLY KNOW NO 6270 CAM THAT IS SUCCESSFULLY READ HERE
	// - IF READ FAILS, THIS IS NEVER DONE AGAIN (IN NEXT CYCLE OR FURTHER STARTS)
	usb_microdia_control_read(dev, 0x10c1, buf, 1);		// returns 0x5c
	buf[0] = 0x50;
	usb_microdia_control_write(dev, 0x10c1, buf, 1);
	retI2C = sn9c20x_read_i2c_data(dev, 0x50, 1, 0x00, SN9C20X_I2C_2WIRE, buf);
	if (retI2C >= 0)
		UDIA_INFO("Read of address 0x00 from slave 0x50 successful:\n=> Please report this to microdia@googlegroups.com (http://groups.google.com/group/microdia) !\n");
	buf[0] = 0x5c;
	usb_microdia_control_write(dev, 0x10c1, buf, 1);


	buf[0] = 0x47;		// ALEX: 0x44
	usb_microdia_control_write(dev, 0x10e0, buf, 1);	// URB 2045
	buf[0] = 0x47;		// ALEX: 0x04
	usb_microdia_control_write(dev, 0x10e0, buf, 1);	// URB 2046
	buf[0] = 0xc6;
	usb_microdia_control_write(dev, 0x1001, buf, 1);	// URB 2047
	buf[0] = 0xc4;
	usb_microdia_control_write(dev, 0x1001, buf, 1);	// URB 2048
	buf[0] = 0x84;		// ALEX: 0xc0
	usb_microdia_control_write(dev, 0x1001, buf, 1);	// URB 2049
	/* ALEX:
	buf[0] = 0x40;
	usb_microdia_control_write(dev, 0x1001, buf, 1);
	*/

	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		// I2C MESSAGES
		retI2C = 0;
		// Writes to sensor registers	0x02: Column Start
		//				0x03: Window Height
		buf[0] = 0x00; buf[1] = 0x16;	// coulmn 22
		buf[2] = 0x01; buf[3] = 0xe1;	// 481
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x02, dev->sensor_flags, buf);	// URBs 2050-2052
		// Writes to sensor registers	0x04: Window Width
		//				0x05: Horizontal Blanking
		buf[0] = 0x02; buf[1] = 0x81;	// 641
		buf[2] = 0x00; buf[3] = 0x04;	// 4 columns (pixel clocks)	// AT FIRST START + FIRST CYCLE: 0x00 0x00
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x04, dev->sensor_flags, buf);	// URBs 2053-2055
		// Writes to sensor registers	0x06: Vertical Blanking
		//				0x07: Output Control
		buf[0] = 0x00; buf[1] = 0x00;	// 0 rows
		buf[2] = 0x30; buf[3] = 0x02;	// normal operation + chip enable + RESERVED options
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x06, dev->sensor_flags, buf);	// URBs 2056-2058
	// Write to sensor register	0x0e: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x08;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x0e, dev->sensor_flags, buf);	// URBs 2059-2061
		if (retI2C < 0)
		{
			UDIA_INFO("1 or more errors occured in I2C-message-block 2 !\n");
			return -1;
		}
		// END OF I2C MESSAGES
		// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	}

	buf[0] = 0x02; buf[1] = 0x00; buf[2] = 0x02; buf[3] = 0x00;
	buf[4] = 0x28; buf[5] = 0x3c;
	usb_microdia_control_write(dev, 0x1180, buf, 6);	// URB 2062
	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00; buf[3] = 0xf0; buf[4] = 0x00;
	usb_microdia_control_write(dev, 0x10fb, buf, 5);	// URB 2063
	buf[0] = 0x0c;
	usb_microdia_control_write(dev, 0x1189, buf, 1);	// URB 2064
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x6a; buf[3] = 0x50;
	usb_microdia_control_write(dev, 0x11a1, buf, 4);	// URB 2065
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x50; buf[3] = 0x3c;
	usb_microdia_control_write(dev, 0x11ab, buf, 4);	// URB 2066
	buf[0] = 0x78;		// ALEX: 0x7c
	usb_microdia_control_write(dev, 0x1000, buf, 1);	// URB 2067
	buf[0] = 0x18;		// ALEX: 0x1c
	usb_microdia_control_write(dev, 0x1002, buf, 1);	// URB 2068
	buf[0] = 0x18;		// ALEX: 0x1c
	usb_microdia_control_write(dev, 0x1002, buf, 1);	// URB 2069
	buf[0] = 0x38;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2070
	buf[0] = 0x04;
	usb_microdia_control_write(dev, 0x118a, buf, 1);	// URB 2071
	buf[0] = 0x04;
	usb_microdia_control_write(dev, 0x0395, buf, 1);	// URB 2072

	buf[0] = 0x78;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2073
	usb_microdia_control_read(dev, 0x11b8, buf, 1);		// URB 2074	// 0x38
	if (buf[0] == 0x38)
		buf[0] = 0x79;
	else if (buf[0] == 0xb8)
		buf[0] = 0xf9;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2075
	usb_microdia_control_read(dev, 0x11b8, buf, 1);		// URB 2076	// 0xb9
	if (buf[0] == 0x39)
		buf[0] = 0x7a;
	else if (buf[0] == 0xb9)
		buf[0] = 0xfa;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2077
	usb_microdia_control_read(dev, 0x11b8, buf, 1);		// URB 2078
	if (buf[0] == 0x3a)
		buf[0] = 0x7b;
	else if (buf[0] == 0xba)
		buf[0] = 0xfb;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2079
	usb_microdia_control_read(dev, 0x11b8, buf, 1);		// URB 2080	// 0x3b
	if (buf[0] == 0x3b)
		buf[0] = 0x7c;
	else if (buf[0] == 0xbb)
		buf[0] = 0xfc;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2081
	usb_microdia_control_read(dev, 0x11b8, buf, 1);		// URB 2082	// 0x3c
	if (buf[0] == 0x3c)
		buf[0] = 0x7d;
	else if (buf[0] == 0xbc)
		buf[0] = 0xfd;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2083
	usb_microdia_control_read(dev, 0x11b8, buf, 1);		// URB 2084	// 0x3d
	last_11b8[0] = buf[0];
	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		if (buf[0] == 0x3d)
			buf[0] = 0x7e;
		else if (buf[0] == 0xbd)
			buf[0] = 0xfe;
		usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2085
		usb_microdia_control_read(dev, 0x11b8, buf, 1);		// URB 2086
		if (buf[0] == 0x3e)
			buf[0] = 0x7f;
		else if (buf[0] == 0xbe)
			buf[0] = 0xff;
		usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2087
		usb_microdia_control_read(dev, 0x11b8, last_11b8, 1);	// URB 2088
	}
	buf[0] = last_11b8[0] & 0xf0;	// 0xf0 or 0x70
	buf[0] = buf[0] | 0x0d;	// 0x0c => THIS VALUE SEEMS TO DEPEND ON WHAT WAS READ/WRITTEN IN THE LAST 8 READS/WRITES
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2089


	// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// I2C MESSAGES
	retI2C = 0;
	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		// Write to sensor register	0x06: Vertical Blanking
		buf[0] = 0x00; buf[1] = 0x2d;	// 45 rows
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x06, dev->sensor_flags, buf);	// URBs 2090-2092
		// Write to sensor register	0x05: Horizontal Blanking
		buf[0] = 0x00; buf[1] = 0x04;	// 4 columns (pixel clocks)
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x05, dev->sensor_flags, buf);	// URBs 2093-2095
		// Write to sensor register	0x20: Read Mode
		buf[0] = 0x00; buf[1] = 0x00;	// normal readout
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x20, dev->sensor_flags, buf);	// URBs 2096-2098
	}
	else if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		// Write to sensor register	0x0a: Pixel Clock Speed
		buf[0] = 0x00; buf[1] = 0x00;	// default
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x0a, dev->sensor_flags, buf);
		// Write to sensor register	0x06: Vertical Blanking
		buf[0] = 0x00; buf[1] = 0x29;	// 41 rows
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x06, dev->sensor_flags, buf);
		// Write to sensor register	0x05: Horizontal Blanking
		buf[0] = 0x00; buf[1] = 0x09;	// 9 columns (pixel clocks)
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x05, dev->sensor_flags, buf);
		// Write to sensor register	0x20: Read Mode
		buf[0] = 0x11; buf[1] = 0x01;	// output all frames (including bad ones) + UNKNOWN
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x20, dev->sensor_flags, buf);
	}
	if (retI2C < 0)
	{
		UDIA_INFO("1 or more errors occured in I2C-message-block 3 !\n");
		return -1;
	}
	// END OF I2C MESSAGES
	// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


	buf[0] = 0x02;
	usb_microdia_control_write(dev, 0x1180, buf, 1);	// URB 2099

	// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// I2C MESSAGE
	retI2C = 0;
	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		// Write to sensor register	0x20: Read Mode
		buf[0] = 0x00; buf[1] = 0x00;	// normal readout
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x20, dev->sensor_flags, buf);	// URBs 2100-2102
	}
	else if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		// Write to sensor register	0x20: Read Mode
		buf[0] = 0x11; buf[1] = 0x01;	// output all frames (including bad ones) + UNKNOWN
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x20, dev->sensor_flags, buf);
	}
	if (retI2C < 0)
	{
		UDIA_INFO("1 or more errors occured in I2C-message-block 4 !\n");
		return -1;
	}
	// END OF I2C MESSAGES
	// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


	buf[0] = 0x02;
	usb_microdia_control_write(dev, 0x1182, buf, 1);	// URB 2103


	// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// I2C MESSAGES
	retI2C = 0;
	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		// Write to sensor register	0x09: Shutter Width
		buf[0] = 0x00; buf[1] = 0x64;	// integration of 100 rows
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x09, dev->sensor_flags, buf);	// URBs 2104-2106
		// Write to sensor register	0x2b: Green 1 Gain
		//				0x2c: Blue Gain
		buf[0] = 0x00; buf[1] = 0xa0;	// 32*0.03125*2 = 2
		buf[2] = 0x00; buf[3] = 0xa0;	// 32*0.03125*2 = 2
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x2b, dev->sensor_flags, buf);	// URBs 2107-2109
		// Write to sensor register	0x2d: Red Gain
		//				0x2e: Green 2 Gain
		buf[0] = 0x00; buf[1] = 0xa0;	// 32*0.03125*2 = 2
		buf[2] = 0x00; buf[3] = 0xa0;	// 32*0.03125*2 = 2
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x2d, dev->sensor_flags, buf);	// URBs 2110-2112
		// "Dummy" write to sensor Register 0x33: RESERVED
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0, 0x33, dev->sensor_flags, NULL);	// URBs 2113-2114
	}
	else if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		// Write to sensor register	0x09: Shutter Width
		buf[0] = 0x00; buf[1] = 0x64;	// integration of 100 rows
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x09, dev->sensor_flags, buf);
		// Write to sensor register	0x07: Output Control
		buf[0] = 0x00; buf[1] = 0x03;	// dont update changes until bit0=0, chip enable, normal operation
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x07, dev->sensor_flags, buf);
		// Write to sensor register	0x2b: Green 1 Gain
		//				0x2c: Blue Gain
		buf[0] = 0x00; buf[1] = 0x33;	// 51*0.03125*1 = 1.59375
		buf[2] = 0x00; buf[3] = 0xa0;	// 32*0.03125*2 = 2
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x2b, dev->sensor_flags, buf);
		// Write to sensor register	0x2d: Red Gain
		//				0x2e: Green 2 Gain
		buf[0] = 0x00; buf[1] = 0xa0;	// 32*0.03125*2 = 2
		buf[2] = 0x00; buf[3] = 0x33;	// 51*0.03125*1 = 1.59375
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x2d, dev->sensor_flags, buf);
		// "Dummy" write to sensor Register 0x33: RESERVED
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0, 0x33, dev->sensor_flags, NULL);
		// Write to sensor register	0x07: Output Control
		buf[0] = 0x00; buf[1] = 0x02;	// chip enable, normal operation
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x07, dev->sensor_flags, buf);
	}
	if (retI2C < 0)
	{
		UDIA_INFO("1 or more errors occured in I2C-message-block 5 !\n");
		return -1;
	}
	// END OF I2C MESSAGES
	// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118c, buf, 1);	// URB 2115
	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118d, buf, 1);	// URB 2116
	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118e, buf, 1);	// URB 2117
	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118f, buf, 1);	// URB 2118
	buf[0] = 0x0a;
	usb_microdia_control_write(dev, 0x11ba, buf, 1);	// URB 2119
	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118c, buf, 1);	// URB 2120
	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118d, buf, 1);	// URB 2121
	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118e, buf, 1);	// URB 2122
	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118f, buf, 1);	// URB 2123

	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x14;
	}
	else if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x0a;
	}
	usb_microdia_control_write(dev, 0x118b, buf, 1);	// URB 2124


	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x0a; buf[1] = 0x36; buf[2] = 0x56; buf[3] = 0x6e;
		buf[4] = 0x7f; buf[5] = 0x8d; buf[6] = 0x9b; buf[7] = 0xa8;
		buf[8] = 0xb4; buf[9] = 0xbd; buf[10] = 0xc7; buf[11] = 0xd0;
		buf[12] = 0xd9; buf[13] = 0xe2; buf[14] = 0xec; buf[15] = 0xf0;
		buf[16] = 0xf5;
	}
	else if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x08; buf[1] = 0x35; buf[2] = 0x5a; buf[3] = 0x6d;
		buf[4] = 0x7d; buf[5] = 0x8b; buf[6] = 0x97; buf[7] = 0xa3;
		buf[8] = 0xad; buf[9] = 0xb7; buf[10] = 0xc1; buf[11] = 0xca;
		buf[12] = 0xd3; buf[13] = 0xdb; buf[14] = 0xe3; buf[15] = 0xeb;
		buf[16] = 0xf2;
	}
	usb_microdia_control_write(dev, 0x1190, buf, 17);	// URB 2125

	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x16; buf[1] = 0x00; buf[2] = 0x2b; buf[3] = 0x00;
		buf[4] = 0x08; buf[5] = 0x00; buf[6] = 0xf6; buf[7] = 0x0f;
		buf[8] = 0xd2; buf[9] = 0x0f; buf[10] = 0x38; buf[11] = 0x00;
		buf[12] = 0x34; buf[13] = 0x00; buf[14] = 0xcf; buf[15] = 0x0f;
		buf[16] = 0xfd; buf[17] = 0x0f; buf[18] = 0x00; buf[19] = 0x00;
		buf[20] = 0x00;
	}
	else if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x16; buf[1] = 0x00; buf[2] = 0x2b; buf[3] = 0x00;
		buf[4] = 0x08; buf[5] = 0x00; buf[6] = 0xf0; buf[7] = 0x0f;
		buf[8] = 0xd0; buf[9] = 0x0f; buf[10] = 0x41; buf[11] = 0x00;
		buf[12] = 0x38; buf[13] = 0x00; buf[14] = 0xc7; buf[15] = 0x0f;
		buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00;
		buf[20] = 0x00;
	}
	for (k=0; k<9; k++)	// AT FIRST START + FIRST CYCLE: first 8 writes 21 bytes 0x00
	{
        	usb_microdia_control_write(dev, 0x10e1, buf, 21);	// URB 2126 - 2134
	}

	buf[0] = 0x07;
	usb_microdia_control_write(dev, 0x10f7, buf, 1);	// URB 2135
	buf[0] = 0x18;	// ALEX: 0x00
	usb_microdia_control_write(dev, 0x10f6, buf, 1);	// URB 2136

	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x16; buf[1] = 0x00; buf[2] = 0x2b; buf[3] = 0x00;
		buf[4] = 0x08; buf[5] = 0x00; buf[6] = 0xf6; buf[7] = 0x0f;
		buf[8] = 0xd2; buf[9] = 0x0f; buf[10] = 0x38; buf[11] = 0x00;
		buf[12] = 0x34; buf[13] = 0x00; buf[14] = 0xcf; buf[15] = 0x0f;
		buf[16] = 0xfd; buf[17] = 0x0f; buf[18] = 0x00; buf[19] = 0x00;
		buf[20] = 0x00;
	}
	else if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x16; buf[1] = 0x00; buf[2] = 0x2b; buf[3] = 0x00;
		buf[4] = 0x08; buf[5] = 0x00; buf[6] = 0xf0; buf[7] = 0x0f;
		buf[8] = 0xd0; buf[9] = 0x0f; buf[10] = 0x41; buf[11] = 0x00;
		buf[12] = 0x38; buf[13] = 0x00; buf[14] = 0xc7; buf[15] = 0x0f;
		buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00;
		buf[20] = 0x00;
	}
	usb_microdia_control_write(dev, 0x10e1, buf, 21);	// URB 2137

	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x01;
	}
	else if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x14;
	}
	usb_microdia_control_write(dev, 0x10f8, buf, 1);	// URB 2138

	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x0a;
	}
	else if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0xff;
	}
	usb_microdia_control_write(dev, 0x10fa, buf, 1);	// URB 2139

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x10f9, buf, 1);	// URB 2140
	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x10f9, buf, 1);	// URB 2141
	buf[0] = 0x0a;
	usb_microdia_control_write(dev, 0x11ba, buf, 1);	// URB 2142
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00;
	usb_microdia_control_write(dev, 0x11bc, buf, 4);	// URB 2143
	for (k=0; k<48; k++) buf[k] = 0x00;
	usb_microdia_control_write(dev, 0x11c0, buf, 48);	// URB 2144
	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118c, buf, 1);	// URB 2145
	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118d, buf, 1);	// URB 2146
	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118e, buf, 1);	// URB 2147
	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x118f, buf, 1);	// URB 2148
	buf[0] = 0x2d; buf[1] = 0x2d; buf[2] = 0x3a; buf[3] = 0x05;
	buf[4] = 0x04; buf[5] = 0x3f;
	usb_microdia_control_write(dev, 0x11a5, buf, 6);	// URB 2149

	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x14; buf[1] = 0xec; buf[2] = 0x0a; buf[3] = 0xf6;
	}
	else if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x28; buf[1] = 0xd8; buf[2] = 0x14; buf[3] = 0xec;
	}
	usb_microdia_control_write(dev, 0x11af, buf, 4);	// URB 2150

	buf[0] = 0x32; buf[1] = 0xdd; buf[2] = 0x2d; buf[3] = 0xdd;
	usb_microdia_control_write(dev, 0x11b3, buf, 4);	// URB 2151
	buf[0] = 0x47;	// ALEX: 0x04
	usb_microdia_control_write(dev, 0x10e0, buf, 1);	// URB 2152
	buf[0] = 0x01;
	usb_microdia_control_write(dev, 0x1061, buf, 1);	// URB 2153
	buf[0] = 0x67;	// ALEX: 0x24
	usb_microdia_control_write(dev, 0x10e0, buf, 1);	// URB 2154
	buf[0] = 0x0d; buf[1] = 0x08; buf[2] = 0x08; buf[3] = 0x0d;
	buf[4] = 0x08; buf[5] = 0x08; buf[6] = 0x0d; buf[7] = 0x0d;
	buf[8] = 0x0d; buf[9] = 0x0d; buf[10] = 0x11; buf[11] = 0x0d;
	buf[12] = 0x0d; buf[13] = 0x11; buf[14] = 0x15; buf[15] = 0x21;
	buf[16] = 0x15; buf[17] = 0x15; buf[18] = 0x11; buf[19] = 0x11;
	buf[20] = 0x15; buf[21] = 0x2a; buf[22] = 0x1d; buf[23] = 0x1d;
	buf[24] = 0x19; buf[25] = 0x21; buf[26] = 0x32; buf[27] = 0x2a;
	buf[28] = 0x32; buf[29] = 0x32; buf[30] = 0x2e; buf[31] = 0x2a;
	buf[32] = 0x2e; buf[33] = 0x2e; buf[34] = 0x36; buf[35] = 0x3a;
	buf[36] = 0x4b; buf[37] = 0x43; buf[38] = 0x36; buf[39] = 0x3a;
	buf[40] = 0x47; buf[41] = 0x3a; buf[42] = 0x2e; buf[43] = 0x2e;
	buf[44] = 0x43; buf[45] = 0x5c; buf[46] = 0x43; buf[47] = 0x47;
	buf[48]= 0x4f; buf[49] = 0x53; buf[50] = 0x58; buf[51] = 0x58;
	buf[52] = 0x58; buf[53] = 0x32; buf[54] = 0x3f; buf[55] = 0x60;
	buf[56] = 0x64; buf[57] = 0x5c; buf[58] = 0x53; buf[59] = 0x64;
	buf[60] = 0x4b; buf[61] = 0x53; buf[62] = 0x58; buf[63] = 0x53;
	usb_microdia_control_write(dev, 0x1100, buf, 64);	// URB 2155
	buf[0] = 0x0d; buf[1] = 0x11; buf[2] = 0x11; buf[3] = 0x15;
	buf[4] = 0x11; buf[5] = 0x15; buf[6] = 0x26; buf[7] = 0x15;
	buf[8] = 0x15; buf[9] = 0x26; buf[10] = 0x53; buf[11] = 0x36;
	buf[12] = 0x2e; buf[13] = 0x36; buf[14] = 0x53; buf[15] = 0x53;
	buf[16] = 0x53; buf[17] = 0x53; buf[18] = 0x53; buf[19] = 0x53;
	buf[20] = 0x53; buf[21] = 0x53; buf[22] = 0x53; buf[23] = 0x53;
	buf[24] = 0x53; buf[25] = 0x53; buf[26] = 0x53; buf[27] = 0x53;
	buf[28] = 0x53; buf[29] = 0x53; buf[30] = 0x53; buf[31] = 0x53;
	buf[32] = 0x53; buf[33] = 0x53; buf[34] = 0x53; buf[35] = 0x53;
	buf[36] = 0x53; buf[37] = 0x53; buf[38] = 0x53; buf[39] = 0x53;
	buf[40] = 0x53; buf[41] = 0x53; buf[42] = 0x53; buf[43] = 0x53;
	buf[44] = 0x53; buf[45] = 0x53; buf[46] = 0x53; buf[47] = 0x53;
	buf[48] = 0x53; buf[49] = 0x53; buf[50] = 0x53; buf[51] = 0x53;
	buf[52] = 0x53; buf[53] = 0x53; buf[54] = 0x53; buf[55] = 0x53;
	buf[56] = 0x53; buf[57] = 0x53; buf[58] = 0x53; buf[59] = 0x53;
	buf[60] = 0x53; buf[61] = 0x53; buf[62] = 0x53; buf[63] = 0x53;
	usb_microdia_control_write(dev, 0x1140, buf, 64);	// URB 2156
	buf[0] = 0x47;	// ALEX: 0x04
	usb_microdia_control_write(dev, 0x10e0, buf, 1);	// URB 2157
	buf[0] = 0x03;
	usb_microdia_control_write(dev, 0x1061, buf, 1);	// URB 2158
	buf[0] = 0x4b;	// ALEX: 0x08
	usb_microdia_control_write(dev, 0x10e0, buf, 1);	// URB 2159

	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
		// I2C MESSAGES
		retI2C = 0;
		// Writes to sensor registers	0x02: Column Start
		//				0x03: Window Hight
		buf[0] = 0x00; buf[1] = 0x16;	// coulmn 22
		buf[2] = 0x01; buf[3] = 0xe1;	// 481
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x02, dev->sensor_flags, buf);	// URBs 2160-2162
		// Writes to sensor registers	0x04: Window Width
		//				0x05: Horizontal Blanking
		buf[0] = 0x02; buf[1] = 0x81;	// 641
		buf[2] = 0x00; buf[3] = 0x04;	// 4 columns (pixel clocks)
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x04, dev->sensor_flags, buf);	// URBs 2163-2165
		// Writes to sensor registers	0x06: Vertical Blanking
		//				0x07: Output Control
		buf[0] = 0x00; buf[1] = 0x2d;	// 45 rows
		buf[2] = 0x30; buf[3] = 0x02;	// RESERVED options
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, 0x06, dev->sensor_flags, buf);	// URBs 2166-2167
		// Writes to sensor register	0x0e: UNDOCUMENTED
		buf[0] = 0x00; buf[1] = 0x08;
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x0e, dev->sensor_flags, buf);	// URBs 2168-2170
		if (retI2C < 0)
		{
			UDIA_INFO("1 or more errors occured in I2C-message-block 6 !\n");
			return -1;
		}
		// END OF I2C MESSAGES
		// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	}
	else if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x40;
		usb_microdia_control_write(dev, 0x1007, buf, 1);
		buf[0] = 0x40;
		usb_microdia_control_write(dev, 0x1006, buf, 1);
	}


	buf[0] = 0x02; buf[1] = 0x00; buf[2] = 0x02; buf[3] = 0x00;
	buf[4] = 0x28; buf[5] = 0x3c;
	usb_microdia_control_write(dev, 0x1180, buf, 6);	// URB 2171
	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00; buf[3] = 0xf0; buf[4] = 0x00;
	usb_microdia_control_write(dev, 0x10fb, buf, 5);	// URB 2172
	buf[0] = 0x0c;
	usb_microdia_control_write(dev, 0x1189, buf, 1);	// URB 2173
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x6a; buf[3] = 0x50;
	usb_microdia_control_write(dev, 0x11a1, buf, 4);	// URB 2174
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x50; buf[3] = 0x3c;
	usb_microdia_control_write(dev, 0x11ab, buf, 4);	// URB 2175
	buf[0] = 0x03;
	usb_microdia_control_write(dev, 0x1061, buf, 1);	// URB 2176
	buf[0] = 0x0a;
	usb_microdia_control_write(dev, 0x11ba, buf, 1);	// URB 2177
	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x11b9, buf, 1);	// URB 2178
	buf[0] = 0x0b;
	usb_microdia_control_write(dev, 0x11ba, buf, 1);	// URB 2179
	buf[0] = 0x01;
	usb_microdia_control_write(dev, 0x1061, buf, 1);	// URB 2180
	buf[0] = 0x78;	// ALEX: 0x7c
	usb_microdia_control_write(dev, 0x1000, buf, 1);	// URB 2181
	buf[0] = 0x18;	// ALEX: 0x1c
	usb_microdia_control_write(dev, 0x1002, buf, 1);	// URB 2182
	buf[0] = 0x18;	// ALEX: 0x1c
	usb_microdia_control_write(dev, 0x1002, buf, 1);	// URB 2183
	buf[0] = 0x7d;	// ALEX: 0xfc
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2184
	buf[0] = 0x04;
	usb_microdia_control_write(dev, 0x118a, buf, 1);	// URB 2185
	buf[0] = 0x04;
	usb_microdia_control_write(dev, 0x0395, buf, 1);	// URB 2186

	if ((last_11b8[0] == 0x3d) || (last_11b8[0] == 0x3f))
		buf[0] = 0x78;
	else if ((last_11b8[0] == 0xbd) || (last_11b8[0] == 0xbf))
		buf[0] = 0xf8;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2187
	usb_microdia_control_read(dev, 0x11b8, buf, 1);		// URB 2188	// 0x38
	if (buf[0] == 0x38)
		buf[0] = 0x79;
	else if (buf[0] == 0xb8)
		buf[0] = 0xf9;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2189
	usb_microdia_control_read(dev, 0x11b8, buf, 1);		// URB 2190	// 0xb9
	if (buf[0] == 0x39)
		buf[0] = 0x7a;
	else if (buf[0] == 0xb9)
		buf[0] = 0xfa;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2191
	usb_microdia_control_read(dev, 0x11b8, buf, 1);		// URB 2192	// 0xba
	if (buf[0] == 0x3a)
		buf[0] = 0x7b;
	else if (buf[0] == 0xba)
		buf[0] = 0xfb;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2193
	usb_microdia_control_read(dev, 0x11b8, buf, 1);		// URB 2194	// 0x3b
	if (buf[0] == 0x3b)
		buf[0] = 0x7c;
	else if (buf[0] == 0xbb)
		buf[0] = 0xfc;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2195
	usb_microdia_control_read(dev, 0x11b8, buf, 1);		// URB 2196	// 0x3c
	if (buf[0] == 0x3c)
		buf[0] = 0x7d;
	else if (buf[0] == 0xbc)
		buf[0] = 0xfd;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2197
	usb_microdia_control_read(dev, 0x11b8, buf, 1);		// URB 2198	// 0x3d
	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		if (buf[0] == 0x3d)
			buf[0] = 0x7e;
		else if (buf[0] == 0xbd)
			buf[0] = 0xfe;
		usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2199
		usb_microdia_control_read(dev, 0x11b8, buf, 1);		// URB 2200
		if (buf[0] == 0x3e)
			buf[0] = 0x7f;
		else if (buf[0] == 0xbe)
			buf[0] = 0xff;
		usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2201
		usb_microdia_control_read(dev, 0x11b8, buf, 1);	// URB 2202
	}
	buf[0] = buf[0] & 0xf0;	// 0xf0 or 0x70
	buf[0] = buf[0] | 0x0d;	// 0x0c => THIS VALUE SEEMS TO DEPEND ON WHAT WAS READ/WRITTEN IN THE LAST 8 READS/WRITES
	usb_microdia_control_write(dev, 0x11b8, buf, 1);	// URB 2089

	// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	// I2C MESSAGES
	retI2C = 0;
	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		// Write to sensor register	0x06: Vertical Blanking
		buf[0] = 0x00; buf[1] = 0x2d;	// 45 rows
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x06, dev->sensor_flags, buf);	// URBs 2204-2206
		// Write to sensor register	0x05: Horizontal Blanking
		buf[0] = 0x00; buf[1] = 0x04;	// 4 columns (pixel clocks)
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x05, dev->sensor_flags, buf);	// URBs 2207-2209
		// Read of sensor register	0x36: Chip Version (mirror of reg0xff)
		retI2C += sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 2, 0x36, dev->sensor_flags, buf);	// URBs 2211-2215   // 0x82 0x3a
	}
	else if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		// Write to sensor register	0x0a: Pixel Clock Speed
		buf[0] = 0x00; buf[1] = 0x00;	// default
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x06, dev->sensor_flags, buf);
		// Write to sensor register	0x06: Vertical Blanking
		buf[0] = 0x00; buf[1] = 0x29;	// 41 rows
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x06, dev->sensor_flags, buf);
		// Write to sensor register	0x05: Horizontal Blanking
		buf[0] = 0x00; buf[1] = 0x09;	// 9 columns (pixel clocks)
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x05, dev->sensor_flags, buf);
		// Read of sensor register	0x00: Chip Version (=reg0xff)
		retI2C += sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 2, 0x00, dev->sensor_flags, buf);	// 0x82 0x43
	}
	if (retI2C < 0)
	{
		UDIA_INFO("1 or more errors occured in I2C-message-block 7 !\n");
		return -1;
	}
	// END OF I2C MESSAGES
	// +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++


	buf[0] = 0x03;
	usb_microdia_control_write(dev, 0x1061, buf, 1);	// URB 2216


	/*** NOW DRIVER DOES STOP-SEQUENCE 
		- WITHOUT WRITES TO 0x1007 AND 0x1006 (URBs 3544-3550)   => MT9V111
		- WRITING 0x40 INSTEAD OF 0x60 TO 0x1007 AND 0x1006	 => MT9V011
	 ***/
	/*** 2nd, 3rd, ... CYCLE: ALL MESSAGES (URBs 1953 - URBs 2216) ARE REPEATED ***/
	/*** ----- WE DON'T DO THIS, IT SEEMS TO BE UNNECESSARY ----- ***/

	// Setup IFP registers for AE and AWB (new, not in the logs):
	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		mt9v111_setup_autoexposure(dev);
		mt9v111_setup_autowhitebalance(dev);
		mt9v111_set_autocorrections(dev, 1);
	}

	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x20;
	}
	else if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x60;
	}
	usb_microdia_control_write(dev, 0x1007, buf, 1);	// URB 2489
	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x00;
	}
	else if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x40;
	}
	usb_microdia_control_write(dev, 0x1006, buf, 1);	// URB 2490
	// => These two writes seem to cause the cam to start sending isochronus USB messages

	return 0;
}

int microdia_627b_start_stream(struct usb_microdia *dev)
{
	int ret;
	//int actual;
	__u16 reg;
	__u8 buf[64];

	buf[0] = 0x00; /** Will have to be set for every single interrupt. */
	//ret = usb_interrupt_msg(dev->udev, 0x00000083, buf, 0x0000001, &actual, 1000); // URB 43

	reg = 0x1066;
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 1);			// URB 44
	if(ret < 0) goto err;

	//ret = usb_set_interface(dev->udev, 0, 8);
	//if(ret < 0) goto err;

	//buf[0] = 0x00; /** Will have to be set for every single interrupt. */
	//ret = usb_interrupt_msg(dev->udev, 0x00000083, buf, 0x0000001, &actual, 1000); // URB 45

	reg = 0x1000;
	buf[0] = 0x78;
	ret = usb_microdia_control_write(dev, reg, buf, 1);			// URB 46
	if(ret < 0) goto err;

	reg = 0x1001;
	buf[0] = 0xc7;
	ret = usb_microdia_control_write(dev, reg, buf, 1);			// URB 47
	if(ret < 0) goto err;

	reg = 0x1002;
	buf[0] = 0x18;
	ret = usb_microdia_control_write(dev, reg, buf, 1);			// URB 48
	if(ret < 0) goto err;

	reg = 0x1061;
	buf[0] = 0x01;
	ret = usb_microdia_control_write(dev, reg, buf, 1);			// URB 49
	if(ret < 0) goto err;

	reg = 0x1020;
	buf[0] = 0x80;
	ret = usb_microdia_control_write(dev, reg, buf, 1);			// URB 50
	if(ret < 0) goto err;

	reg = 0x1067;
	buf[0] = 0x40; buf[1] = 0x30; buf[2] = 0x20; buf[3] = 0x10; buf[4] = 0x08;
	ret = usb_microdia_control_write(dev, reg, buf, 5);			// URB 51
	if(ret < 0) goto err;

	reg = 0x10c0;
	buf[0] = 0x80; buf[1] = 0x21; buf[2] = 0x00; buf[3] = 0x00;
	buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00;
	buf[8] = 0x03;
	ret = usb_microdia_control_write(dev, reg, buf, 9);			// URB 52
	if(ret < 0) goto err;
	//udelay(400); //delay test

	//transferbufferlength is 18, but data is 24 bytes long, what to do ?
	reg = 0x10e0;
	buf[0] = 0x45; buf[1] = 0x18; buf[2] = 0x00; buf[3] = 0x30; buf[4] = 0x00;
	buf[5] = 0x09; buf[6] = 0x00; buf[7] = 0xed; buf[8] = 0x0f; buf[9] = 0xda; 
	buf[10] = 0x0f; buf[11] = 0x3a; buf[12] = 0x00; buf[13] = 0x3a; buf[14] = 0x00;
	buf[15] = 0xd0; buf[16] = 0x0f; buf[17] = 0xf7; buf[18] = 0x0f; buf[19] = 0x00;
	buf[20] = 0x00; buf[21] = 0x00; buf[22] = 0x60; buf[23] = 0x0a;
	ret = usb_microdia_control_write(dev, reg, buf, 24);			// URB 53
	if(ret < 0) goto err;

	reg = 0x10f8;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 3);			// URB 54
	if(ret < 0) goto err;

	reg = 0x10fb;
	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00; buf[3] = 0xf0; buf[4] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 5);			// URB 55
	if(ret < 0) goto err;

	reg = 0x1188;
	buf[0] = 0x86; buf[1] = 0x8c; buf[2] = 0x08;
	ret = usb_microdia_control_write(dev, reg, buf, 3);			// URB 56
	if(ret < 0) goto err;

	reg = 0x118b;
	buf[0] = 0x00; buf[1] = 0x20; buf[2] = 0x20; buf[3] = 0x20; buf[4] = 0x20;
	buf[5] = 0x00; buf[6] = 0x24; buf[7] = 0x3b; buf[8] = 0x4f; buf[9] = 0x61;
	buf[10] = 0x71; buf[11] = 0x80; buf[12] = 0x8f; buf[13] = 0x9d; buf[14] = 0xab;
	buf[15] = 0xb8; buf[16] = 0xc4; buf[17] = 0xd1; buf[18] = 0xdd; buf[19] = 0xe9;
	buf[20] = 0xf4; buf[21] = 0xff;
	ret = usb_microdia_control_write(dev, reg, buf, 22);			// URB 57
	if(ret < 0) goto err;

	reg = 0x11a1;
	buf[0] = 0x00; buf[1] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 2);			// URB 58
	if(ret < 0) goto err;

	reg = 0x11b7;
	buf[0] = 0x60; buf[1] = 0x28; buf[2] = 0x00; buf[3] = 0x08; buf[4] = 0x00;
	buf[5] = 0x40; buf[6] = 0x01; buf[7] = 0xf0; buf[8] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 9);			// URB 59
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0x38;
	ret = usb_microdia_control_write(dev, reg, buf, 1);			// URB 60	
	if(ret < 0) goto err;

	reg = 0x1000;
	buf[0] = 0x78;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x12;
	buf[0] = 0x80;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x00;
	buf[0] = 0x01; buf[1] = 0x74; buf[2] = 0x92; buf[3] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x04;
	buf[0] = 0x00; buf[1] = 0x7d; buf[2] = 0x62; buf[3] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x08;
	buf[0] = 0x83; buf[1] = 0x01;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x0c;
	buf[0] = 0x00; buf[1] = 0x08; buf[2] = 0x04; buf[3] = 0x62;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x10;
	buf[0] = 0x00; buf[1] = 0x40; buf[2] = 0x05; buf[3] = 0xf8;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x14;
	buf[0] = 0x2e; buf[1] = 0x00; buf[2] = 0x02;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 3, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x17;
	buf[0] = 0x10; buf[1] = 0x60; buf[2] = 0x02; buf[3] = 0x7b;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x1b;
	buf[0] = 0x06;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x1e;
	buf[0] = 0x01; buf[1] = 0x0e;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x20;
	buf[0] = 0x07; buf[1] = 0x07; buf[2] = 0x07; buf[3] = 0x07;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x24;
	buf[0] = 0x68; buf[1] = 0x58; buf[2] = 0xd4; buf[3] = 0x80;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x28;
	buf[0] = 0x80; buf[1] = 0x3c; buf[2] = 0x00; buf[3] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x2c;
	buf[0] = 0x80; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x62;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x30;
	buf[0] = 0x08; buf[1] = 0x30; buf[2] = 0xb4;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 3, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x33;
	buf[0] = 0x00; buf[1] = 0x07; buf[2] = 0x84; buf[3] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x37;
	buf[0] = 0x0c; buf[1] = 0x02; buf[2] = 0x43; buf[3] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x3b;
	buf[0] = 0x02; buf[1] = 0x6c; buf[2] = 0x19; buf[3] = 0x0e;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x3f;
	buf[0] = 0x41; buf[1] = 0xc1; buf[2] = 0x22; buf[3] = 0x08;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	// "Dummy" write
	reg = 0x43;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0, reg,
					dev->sensor_flags, NULL);
	if(ret < 0) goto errI2C;

	reg = 0x43;
	buf[0] = 0xf0; buf[1] = 0x10; buf[2] = 0x78; buf[3] = 0xa8;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x47;
	buf[0] = 0x60; buf[1] = 0x80; buf[2] = 0x00; buf[3] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x4b;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	// "Dummy" write
	reg = 0x4f;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0, reg,
					dev->sensor_flags, NULL);
	if(ret < 0) goto errI2C;

	reg = 0x59;
	buf[0] = 0xba; buf[1] = 0x9a; buf[2] = 0x22; buf[3] = 0xb9;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x5d;
	buf[0] = 0x9b; buf[1] = 0x10; buf[2] = 0xf0; buf[3] = 0x05;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x61;
	buf[0] = 0x60;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x62;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x50; buf[3] = 0x30;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x66;
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x67;
	buf[0] = 0x80; buf[1] = 0x7a; buf[2] = 0x90; buf[3] = 0x80;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x6b;
	buf[0] = 0x0a;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x8b;
	buf[0] = 0x99; buf[1] = 0x99; buf[2] = 0xcf;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 3, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x92;
	buf[0] = 0x00; buf[1] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x1007;
	buf[0] = 0x40;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1006;
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c1;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c1;
	buf[0] = 0x50;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	// TRY TO READ FROM EEPROM:
	reg = 0x00;
	ret = sn9c20x_read_i2c_data(dev, 0x50, 1, reg, SN9C20X_I2C_2WIRE, buf);
	if(ret < 0)
	{
		UDIA_INFO("No EEPROM found\n");
	}
	else
	{
		UDIA_INFO("Read from EEPROM successful\n");
	}

	reg = 0x10c1;
	buf[0] = 0x21;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10e0;
	buf[0] = 0x47;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10e0;
	buf[0] = 0x47;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1001;
	buf[0] = 0xc6;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1001;
	buf[0] = 0xc4;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1001;
	buf[0] = 0x84;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x03;
	buf[0] = 0x08;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x17;
	buf[0] = 0x10; buf[1] = 0x61; buf[2] = 0x02; buf[3] = 0x7a;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 4, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	// "Dummy" write:
	reg = 0x1b;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 0, reg,
					dev->sensor_flags, NULL);
	if(ret < 0) goto errI2C;

	reg = 0x32;
	buf[0] = 0x84;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x32;
	buf[0] = 0x84;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x32;
	buf[0] = 0x84;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x32;
	buf[0] = 0x84;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x32;
	buf[0] = 0x84;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x1180;
	buf[0] = 0x01; buf[1] = 0x00; buf[2] = 0x01; buf[3] = 0x00; buf[4] = 0x28;
	buf[5] = 0x3c;
	ret = usb_microdia_control_write(dev, reg, buf, 6);
	if(ret < 0) goto err;

	reg = 0x10fb;
	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00; buf[3] = 0xf0; buf[4] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 5);
	if(ret < 0) goto err;

	reg = 0x1189;
	buf[0] = 0x8c;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11a1;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x6a; buf[3] = 0x50;
	ret = usb_microdia_control_write(dev, reg, buf, 4);
	if(ret < 0) goto err;

	reg = 0x11ab;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x50; buf[3] = 0x3c;
	ret = usb_microdia_control_write(dev, reg, buf, 4);
	if(ret < 0) goto err;

	reg = 0x1000;
	buf[0] = 0x78;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1002;
	buf[0] = 0x18;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1002;
	buf[0] = 0x18;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0x38;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118a;
	buf[0] = 0x04;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x0395;
	buf[0] = 0x04;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0x78;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0xf9;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0xfa;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0x7b;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0x7c;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0x7d;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0x7b;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;
	//ret = usb_microdia_control_read(dev, reg, buf, 1);
	//if(ret < 0) goto err;

	reg = 0x11;
	buf[0] = 0x40;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x2a;
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x2b;
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x92;
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x93;
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x1e;
	buf[0] = 0x01;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x1e;
	buf[0] = 0x01;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x03;
	buf[0] = 0x08;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x03;
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x10;
	buf[0] = 0x7f;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x04;
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x2d;
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x2e;
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x00;
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x01;
	buf[0] = 0x78; buf[1] = 0x78;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x118c;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118d;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118e;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118f;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11ba;
	buf[0] = 0x0a;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118c;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118d;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118e;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118f;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118b;
	buf[0] = 0x0c;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1190;
	buf[0] = 0x00; buf[1] = 0x33; buf[2] = 0x53; buf[3] = 0x6b; buf[4] = 0x7c;
	buf[5] = 0x8b; buf[6] = 0x99; buf[7] = 0xa6; buf[8] = 0xb2; buf[9] = 0xbd;
	buf[10] = 0xc8; buf[11] = 0xd2; buf[12] = 0xdc; buf[13] = 0xe5; buf[14] = 0xee;
	buf[15] = 0xf7; buf[16] = 0xff;
	ret = usb_microdia_control_write(dev, reg, buf, 17);
	if(ret < 0) goto err;
	//transferbufferlength was only 11 ?

	reg = 0x10e1;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00;
	buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00;
	buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00;
	buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0xee; buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;
	//transferbufferlength was only 15 ?

	reg = 0x10e1;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00;
	buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00;
	buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00;
	buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0xee; buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;
	//transferbufferlength was only 15 ?

	reg = 0x10e1;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00;
	buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00;
	buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00;
	buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0xee; buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;
	//transferbufferlength was only 15 ?

	reg = 0x10e1;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00;
	buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00;
	buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00;
	buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0xee; buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;
	//transferbufferlength was only 15 ?

	reg = 0x10e1;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00;
	buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00;
	buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00;
	buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0xee; buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;
	//transferbufferlength was only 15 ?

	reg = 0x10e1;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00;
	buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00;
	buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00;
	buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0xee; buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;
	//transferbufferlength was only 15 ?

	reg = 0x10e1;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00;
	buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00;
	buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00;
	buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0xee; buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;
	//transferbufferlength was only 15 ?

	reg = 0x10e1;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00;
	buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00;
	buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00;
	buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0xee; buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;
	//transferbufferlength was only 15 ?

	reg = 0x10e1;
	buf[0] = 0x15; buf[1] = 0x00; buf[2] = 0x29; buf[3] = 0x00; buf[4] = 0x08;
	buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00;
	buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00;
	buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0xee; buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;
	//transferbufferlength was only 15 ?

	reg = 0x10f7;
	buf[0] = 0x05;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;
	
	reg = 0x10f6;
	buf[0] = 0x1b;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10e1;
	buf[0] = 0x15; buf[1] = 0x00; buf[2] = 0x29; buf[3] = 0x00; buf[4] = 0x08;
	buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00;
	buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00;
	buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0xee; buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;
	//transferbufferlength was only 15 ?

	reg = 0x10f8;
	buf[0] = 0x14;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10fa;
	buf[0] = 0xff;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10f9;
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10f9;
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11ba;
	buf[0] = 0x0a;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11bc;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 4);
	if(ret < 0) goto err;

	reg = 0x11c0;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00;
	buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00;
	buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00;
	buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00;
	buf[20] = 0x00; buf[21] = 0x00; buf[22] = 0x00; buf[23] = 0x00; buf[24] = 0x00;
	buf[25] = 0x00; buf[26] = 0x00; buf[27] = 0x00; buf[28] = 0x00; buf[29] = 0x00;
	buf[30] = 0x00; buf[31] = 0x00; buf[32] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 33);
	if(ret < 0) goto err;
	//transferbufferlength was only 30 ?

	reg = 0x118c;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118d;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118e;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118f;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11a5;
	buf[0] = 0x2d; buf[1] = 0x2d; buf[2] = 0x3a; buf[3] = 0x05; buf[4] = 0x04;
	buf[5] = 0x3f;
	ret = usb_microdia_control_write(dev, reg, buf, 6);
	if(ret < 0) goto err;

	reg = 0x11af;
	buf[0] = 0x28; buf[1] = 0xd8; buf[2] = 0x14; buf[3] = 0xec;
	ret = usb_microdia_control_write(dev, reg, buf, 4);
	if(ret < 0) goto err;

	reg = 0x11b3;
	buf[0] = 0x32; buf[1] = 0xdd; buf[2] = 0x32; buf[3] = 0xdd;
	ret = usb_microdia_control_write(dev, reg, buf, 4);
	if(ret < 0) goto err;

	reg = 0x10e0;
	buf[0] = 0x47;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1061;
	buf[0] = 0x01;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10e0;
	buf[0] = 0x67;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1100;
	buf[0] = 0x0d; buf[1] = 0x08; buf[2] = 0x08; buf[3] = 0x0d; buf[4] = 0x08;
	buf[5] = 0x08; buf[6] = 0x0d; buf[7] = 0x0d; buf[8] = 0x0d; buf[9] = 0x0d;
	buf[10] = 0x11; buf[11] = 0x0d; buf[12] = 0x0d; buf[13] = 0x11; buf[14] = 0x15;
	buf[15] = 0x21; buf[16] = 0x15; buf[17] = 0x15; buf[18] = 0x11; buf[19] = 0x11;
	buf[20] = 0x15; buf[21] = 0x2a; buf[22] = 0x1d; buf[23] = 0x1d; buf[24] = 0x19;
	buf[25] = 0x21; buf[26] = 0x32; buf[27] = 0x2a; buf[28] = 0x32; buf[29] = 0x32;
	buf[30] = 0x2e; buf[31] = 0x2a; buf[32] = 0x2e; buf[33] = 0x2e; buf[34] = 0x36;
	buf[35] = 0x3a; buf[36] = 0x4b; buf[37] = 0x43; buf[38] = 0x36; buf[39] = 0x3a;
	buf[40] = 0x47; buf[41] = 0x3a; buf[42] = 0x2e; buf[43] = 0x2e; buf[44] = 0x43;
	buf[45] = 0x5c; buf[46] = 0x43; buf[47] = 0x47; buf[48] = 0x4f; buf[49] = 0x53;
	buf[50] = 0x58; buf[51] = 0x58; buf[52] = 0x58; buf[53] = 0x32; buf[54] = 0x3f;
	buf[55] = 0x60; buf[56] = 0x64; buf[57] = 0x5c; buf[58] = 0x53; buf[59] = 0x64;
	buf[60] = 0x4b; buf[61] = 0x53; buf[62] = 0x58; buf[63] = 0x53;
	ret = usb_microdia_control_write(dev, reg, buf, 64);
	if(ret < 0) goto err;
	//transferbufferlength is only 40 ?

	reg = 0x1140;
	buf[0] = 0x0d; buf[1] = 0x11; buf[2] = 0x11; buf[3] = 0x15; buf[4] = 0x11;
	buf[5] = 0x15; buf[6] = 0x26; buf[7] = 0x15; buf[8] = 0x15; buf[9] = 0x26;
	buf[10] = 0x53; buf[11] = 0x36; buf[12] = 0x2e; buf[13] = 0x36; buf[14] = 0x53;
	buf[15] = 0x53; buf[16] = 0x53; buf[17] = 0x53; buf[18] = 0x53; buf[19] = 0x53;
	buf[20] = 0x53; buf[21] = 0x53; buf[22] = 0x53; buf[23] = 0x53; buf[24] = 0x53;
	buf[25] = 0x53; buf[26] = 0x53; buf[27] = 0x53; buf[28] = 0x53; buf[29] = 0x53;
	buf[30] = 0x53; buf[31] = 0x53; buf[32] = 0x53; buf[33] = 0x53; buf[34] = 0x53;
	buf[35] = 0x53; buf[36] = 0x53; buf[37] = 0x53; buf[38] = 0x53; buf[39] = 0x53;
	buf[40] = 0x53; buf[41] = 0x53; buf[42] = 0x53; buf[43] = 0x53; buf[44] = 0x53;
	buf[45] = 0x53; buf[46] = 0x53; buf[47] = 0x53; buf[48] = 0x53; buf[49] = 0x53;
	buf[50] = 0x53; buf[51] = 0x53; buf[52] = 0x53; buf[53] = 0x53; buf[54] = 0x53;
	buf[55] = 0x53; buf[56] = 0x53; buf[57] = 0x53; buf[58] = 0x53; buf[59] = 0x53;
	buf[60] = 0x53; buf[61] = 0x53; buf[62] = 0x53; buf[63] = 0x53;
	ret = usb_microdia_control_write(dev, reg, buf, 64);
	if(ret < 0) goto err;
	//transferbufferlength is only 40 ?

	reg = 0x10e0;
	buf[0] = 0x47;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1061;
	buf[0] = 0x03;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10e0;
	buf[0] = 0x4b;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x03;
	buf[0] = 0x08;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x1180;
	buf[0] = 0x01; buf[1] = 0x00; buf[2] = 0x01; buf[3] = 0x00; buf[4] = 0x28;
	buf[5] = 0x3c;
	ret = usb_microdia_control_write(dev, reg, buf, 6);
	if(ret < 0) goto err;

	reg = 0x10fb;
	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00; buf[3] = 0xf0; buf[4] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 5);
	if(ret < 0) goto err;

	reg = 0x1189;
	buf[0] = 0x8c;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11a1;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x6a; buf[3] = 0x50;
	ret = usb_microdia_control_write(dev, reg, buf, 4);
	if(ret < 0) goto err;

	reg = 0x11ab;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x50; buf[3] = 0x3c;
	ret = usb_microdia_control_write(dev, reg, buf, 4);
	if(ret < 0) goto err;

	reg = 0x1061;
	buf[0] = 0x03;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11ba;
	buf[0] = 0x0a;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b9;
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11ba;
	buf[0] = 0x0b;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1061;
	buf[0] = 0x01;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1000;
	buf[0] = 0x78;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1002;
	buf[0] = 0x18;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1002;
	buf[0] = 0x18;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0x7b;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118a;
	buf[0] = 0x04;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x0395;
	buf[0] = 0x04;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0x78;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0xf9;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0xfa;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0x7b;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0x7c;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0x7d;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0x7b;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11;
	buf[0] = 0x40;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x2a;
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x2b;
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x92;
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x93;
	buf[0] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x0a;
	ret = sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x0b;
	ret = sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x1c;
	ret = sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x1d;
	ret = sn9c20x_read_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x1061;
	buf[0] = 0x03;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1007;
	buf[0] = 0x60;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1006;
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1007;
	buf[0] = 0x60;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1006;
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10e1;
	buf[0] = 0x15; buf[1] = 0x00; buf[2] = 0x29; buf[3] = 0x00; buf[4] = 0x08;
	buf[5] = 0x00; buf[6] = 0xf9; buf[7] = 0x0f; buf[8] = 0xc4; buf[9] = 0x0f;
	buf[10] = 0x44; buf[11] = 0x00; buf[12] = 0x3f; buf[13] = 0x00; buf[14] = 0xd8;
	buf[15] = 0x0f; buf[16] = 0xe9; buf[17] = 0x0f; buf[18] = 0xeb; buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;
	//transferbufferlength is only 15 ?

	reg = 0x10;
	buf[0] = 0x7f;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x04;
	buf[0] = 0x01;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x10e1;
	buf[0] = 0x15; buf[1] = 0x00; buf[2] = 0x29; buf[3] = 0x00; buf[4] = 0x08;
	buf[5] = 0x00; buf[6] = 0xf9; buf[7] = 0x0f; buf[8] = 0xc4; buf[9] = 0x0f;
	buf[10] = 0x44; buf[11] = 0x00; buf[12] = 0x3f; buf[13] = 0x00; buf[14] = 0xd8;
	buf[15] = 0x0f; buf[16] = 0xe9; buf[17] = 0x0f; buf[18] = 0xeb; buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;
	//transferbufferlength is only 15 ?

	reg = 0x00;
	buf[0] = 0x13;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x10e1;
	buf[0] = 0x15; buf[1] = 0x00; buf[2] = 0x29; buf[3] = 0x00; buf[4] = 0x08;
	buf[5] = 0x00; buf[6] = 0xf9; buf[7] = 0x0f; buf[8] = 0xc4; buf[9] = 0x0f;
	buf[10] = 0x44; buf[11] = 0x00; buf[12] = 0x3f; buf[13] = 0x00; buf[14] = 0xd8;
	buf[15] = 0x0f; buf[16] = 0xe9; buf[17] = 0x0f; buf[18] = 0xeb; buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;
	//transferbufferlength is only 15 ?

	reg = 0x2d;
	buf[0] = 0x00; buf[1] = 0x01;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x00;
	buf[0] = 0x1e;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x01;
	buf[0] = 0x50; buf[1] = 0x60;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x118c;
	buf[0] = 0x1e;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118d;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118e;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118f;
	buf[0] = 0x21;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x2d;
	buf[0] = 0xfc; buf[1] = 0x04;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x10e1;
	buf[0] = 0x15; buf[1] = 0x00; buf[2] = 0x29; buf[3] = 0x00; buf[4] = 0x08;
	buf[5] = 0x00; buf[6] = 0xf9; buf[7] = 0x0f; buf[8] = 0xc8; buf[9] = 0x0f;
	buf[10] = 0x3f; buf[11] = 0x00; buf[12] = 0x3b; buf[13] = 0x00; buf[14] = 0xdb;
	buf[15] = 0x0f; buf[16] = 0xeb; buf[17] = 0x0f; buf[18] = 0xeb; buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;
	//transferbufferlength is only 15 ?

	reg = 0x00;
	buf[0] = 0x1f; buf[1] = 0x00;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x01;
	buf[0] = 0x48; buf[1] = 0x58;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x118c;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118d;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118e;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118f;
	buf[0] = 0x1f;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x2d;
	buf[0] = 0xf6; buf[1] = 0x0b;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x10e1;
	buf[0] = 0x15; buf[1] = 0x00; buf[2] = 0x29; buf[3] = 0x00; buf[4] = 0x08;
	buf[5] = 0x00; buf[6] = 0xfc; buf[7] = 0x0f; buf[8] = 0xdc; buf[9] = 0x0f;
	buf[10] = 0x29; buf[11] = 0x00; buf[12] = 0x26; buf[13] = 0x00; buf[14] = 0xe8;
	buf[15] = 0x0f; buf[16] = 0xf3; buf[17] = 0x0f; buf[18] = 0xeb; buf[19] = 0x00;
	buf[20] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;
	//transferbufferlength is only 15 ?

	reg = 0x118c;
	buf[0] = 0x21;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118d;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118e;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118f;
	buf[0] = 0x21;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1006;
	buf[0] = 0x40;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x01;
	buf[0] = 0x50; buf[1] = 0x60;
	ret = sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, reg,
					dev->sensor_flags, buf);
	if(ret < 0) goto errI2C;

	reg = 0x118c;
	buf[0] = 0x1f;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118d;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118e;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118f;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;
	
	return ret;
err:
	UDIA_ERROR("r/w in bridge register %x failed (%d)!\n", reg, ret);
	return ret;

errI2C:
	UDIA_ERROR("r/w in sensor register %x failed (%d)!\n", reg, ret);
	return ret;

}

/** 
 * @brief From UsbSnoop-plugin-parsed.log
 * 
 * @param dev 
 * 
 * @returns 0 (OK) or <0 (Error)
 *
 * @author Vincent, Kuzja
 */
int microdia_6288_start_stream(struct usb_microdia *dev)
{
	int ret;
	__u16 reg;
	__u8 buf[64];

	
	
	reg = 0x1066 ;
	buf[0] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1000 ;
	buf[0] = 0x78; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1001 ;
	buf[0] = 0xc7; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1002 ;
	buf[0] = 0x1c; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1061 ;
	buf[0] = 0x01; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1020 ;
	buf[0] = 0x80; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1067 ;
	buf[0] = 0x40; buf[1] = 0x30; buf[2] = 0x20; buf[3] = 0x10; buf[4] = 0x08; 
	ret = usb_microdia_control_write(dev, reg, buf, 5);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x80; buf[1] = 0x30; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x03; 
	ret = usb_microdia_control_write(dev, reg, buf, 9);
	if(ret < 0) goto err;

	reg = 0x10e0 ;
	buf[0] = 0x44; buf[1] = 0x18; buf[2] = 0x00; buf[3] = 0x30; buf[4] = 0x00; buf[5] = 0x09; buf[6] = 0x00; buf[7] = 0xed; buf[8] = 0x0f; buf[9] = 0xda; buf[10] = 0x0f; buf[11] = 0x3a; buf[12] = 0x00; buf[13] = 0x3a; buf[14] = 0x00; buf[15] = 0xd0; buf[16] = 0x0f; buf[17] = 0xf7; buf[18] = 0x0f; buf[19] = 0x00; buf[20] = 0x00; buf[21] = 0x00; buf[22] = 0x60; buf[23] = 0x0a; 
	ret = usb_microdia_control_write(dev, reg, buf, 24);
	if(ret < 0) goto err;

	reg = 0x10f8 ;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 3);
	if(ret < 0) goto err;

	reg = 0x10fb ;
	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00; buf[3] = 0xf0; buf[4] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 5);
	if(ret < 0) goto err;

	reg = 0x1188 ;
	buf[0] = 0x86; buf[1] = 0xcc; buf[2] = 0x08; 
	ret = usb_microdia_control_write(dev, reg, buf, 3);
	if(ret < 0) goto err;

	reg = 0x118b ;
	buf[0] = 0x00; buf[1] = 0x20; buf[2] = 0x20; buf[3] = 0x20; buf[4] = 0x20; buf[5] = 0x00; buf[6] = 0x24; buf[7] = 0x3b; buf[8] = 0x4f; buf[9] = 0x61; buf[10] = 0x71; buf[11] = 0x80; buf[12] = 0x8f; buf[13] = 0x9d; buf[14] = 0xab; buf[15] = 0xb8; buf[16] = 0xc4; buf[17] = 0xd1; buf[18] = 0xdd; buf[19] = 0xe9; buf[20] = 0xf4; buf[21] = 0xff; 
	ret = usb_microdia_control_write(dev, reg, buf, 22);
	if(ret < 0) goto err;

	reg = 0x11a1 ;
	buf[0] = 0x00; buf[1] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 2);
	if(ret < 0) goto err;

	reg = 0x11b7 ;
	buf[0] = 0x60; buf[1] = 0x20; buf[2] = 0x00; buf[3] = 0x08; buf[4] = 0x00; buf[5] = 0x40; buf[6] = 0x01; buf[7] = 0xf0; buf[8] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 9);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0x30; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1000 ;
	buf[0] = 0x78; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x12; buf[3] = 0x80; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x12; buf[3] = 0x01; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x0d; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x0e; buf[3] = 0x61; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x11; buf[3] = 0x80; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x13; buf[3] = 0xb8; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x14; buf[3] = 0x3e; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x16; buf[3] = 0x24; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x1e; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x1e; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x1e; buf[3] = 0x04; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x27; buf[3] = 0x08; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x28; buf[3] = 0x08; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x29; buf[3] = 0x15; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x2c; buf[3] = 0x08; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x32; buf[3] = 0x80; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x34; buf[3] = 0x3d; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x35; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x36; buf[3] = 0xf8; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x38; buf[3] = 0x12; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x39; buf[3] = 0x57; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x3a; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x3b; buf[3] = 0xcc; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x3c; buf[3] = 0x0c; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x3d; buf[3] = 0x19; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x3e; buf[3] = 0x0c; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x3f; buf[3] = 0x01; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x41; buf[3] = 0x40; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x42; buf[3] = 0x80; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x45; buf[3] = 0x46; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x46; buf[3] = 0x62; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x47; buf[3] = 0x2a; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x48; buf[3] = 0x3c; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x4a; buf[3] = 0xf0; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x4b; buf[3] = 0xdc; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x4c; buf[3] = 0xdc; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x4d; buf[3] = 0xdc; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x4e; buf[3] = 0xdc; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x69; buf[3] = 0x02; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x6c; buf[3] = 0x04; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x6f; buf[3] = 0x9e; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x70; buf[3] = 0x05; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x71; buf[3] = 0x78; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x77; buf[3] = 0x02; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x8a; buf[3] = 0x23; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x8c; buf[3] = 0x0d; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x90; buf[3] = 0x7e; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x91; buf[3] = 0x7c; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x9f; buf[3] = 0x6e; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xa0; buf[3] = 0x6e; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xa4; buf[3] = 0x50; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xa5; buf[3] = 0x68; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xa6; buf[3] = 0x60; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xa8; buf[3] = 0xc1; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xa9; buf[3] = 0xfa; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xaa; buf[3] = 0x92; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xab; buf[3] = 0x04; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xac; buf[3] = 0x80; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xad; buf[3] = 0x80; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xae; buf[3] = 0x80; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xaf; buf[3] = 0x80; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xb2; buf[3] = 0xf2; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xb3; buf[3] = 0x20; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xb4; buf[3] = 0x20; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xb5; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xb6; buf[3] = 0xaf; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xbb; buf[3] = 0xae; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xbc; buf[3] = 0x44; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xbd; buf[3] = 0x44; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xbe; buf[3] = 0x3b; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xbf; buf[3] = 0x3a; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xc0; buf[3] = 0xe2; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xc1; buf[3] = 0xc8; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xc2; buf[3] = 0x01; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xc4; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xc6; buf[3] = 0x85; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xc7; buf[3] = 0x81; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xc9; buf[3] = 0xe0; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xca; buf[3] = 0xe8; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xcc; buf[3] = 0xd8; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xcd; buf[3] = 0x93; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c1 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c1 ;
	buf[0] = 0x50; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x90; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x90; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x90; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x90; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x90; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x90; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x90; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x90; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x90; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x92; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x92; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x92; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x92; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x92; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x92; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x92; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x92; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x92; buf[1] = 0x50; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c2 ;
	ret = usb_microdia_control_read(dev, reg, buf, 5);
	if(ret < 0) goto err;

	reg = 0x10c1 ;
	buf[0] = 0x30; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10e0 ;
	buf[0] = 0x47; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10e0 ;
	buf[0] = 0x47; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1001 ;
	buf[0] = 0xc6; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1001 ;
	buf[0] = 0xc4; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1001 ;
	buf[0] = 0x84; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1189 ;
	buf[0] = 0xcc; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11bc ;
	buf[0] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11bd ;
	buf[0] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11be ;
	buf[0] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11bf ;
	buf[0] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x03; buf[3] = 0x14; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xd0; buf[1] = 0x30; buf[2] = 0x17; buf[3] = 0x1b; buf[4] = 0xbc; buf[5] = 0x01; buf[6] = 0x7a; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x90; buf[1] = 0x30; buf[2] = 0x1b; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x32; buf[3] = 0x82; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1180 ;
	buf[0] = 0x05; buf[1] = 0x00; buf[2] = 0x01; buf[3] = 0x00; buf[4] = 0x50; buf[5] = 0x78; 
	ret = usb_microdia_control_write(dev, reg, buf, 6);
	if(ret < 0) goto err;

	reg = 0x10fb ;
	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00; buf[3] = 0xf0; buf[4] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 5);
	if(ret < 0) goto err;

	reg = 0x1189 ;
	buf[0] = 0xdc; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1189 ;
	buf[0] = 0xd0; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11a1 ;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0xd5; buf[3] = 0xa0; 
	ret = usb_microdia_control_write(dev, reg, buf, 4);
	if(ret < 0) goto err;

	reg = 0x11ab ;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0xa0; buf[3] = 0x78; 
	ret = usb_microdia_control_write(dev, reg, buf, 4);
	if(ret < 0) goto err;

	reg = 0x1000 ;
	buf[0] = 0x78; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1002 ;
	buf[0] = 0x18; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1002 ;
	buf[0] = 0x08; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0x38; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118a ;
	buf[0] = 0x02; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x0395 ;
	buf[0] = 0x02; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0x78; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0xf9; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0x7a; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0x7b; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0x7c; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0xfd; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0xfa; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x11; buf[3] = 0x80; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x2a; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x2b; buf[3] = 0x06; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x92; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x93; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x1e; buf[3] = 0x04; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x1e; buf[3] = 0x04; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1182 ;
	buf[0] = 0x01; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x10; buf[3] = 0x7d; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x04; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xa1; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x2d; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x2e; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x01; buf[3] = 0x80; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x02; buf[3] = 0x80; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118c ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118d ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118e ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118f ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11ba ;
	buf[0] = 0x0a; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118c ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118d ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118e ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118f ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118b ;
	buf[0] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1190 ;
	buf[0] = 0x08; buf[1] = 0x2f; buf[2] = 0x46; buf[3] = 0x58; buf[4] = 0x69; buf[5] = 0x78; buf[6] = 0x86; buf[7] = 0x94; buf[8] = 0xa0; buf[9] = 0xac; buf[10] = 0xb8; buf[11] = 0xc3; buf[12] = 0xce; buf[13] = 0xd9; buf[14] = 0xe3; buf[15] = 0xed; buf[16] = 0xf7; 
	ret = usb_microdia_control_write(dev, reg, buf, 17);
	if(ret < 0) goto err;

	reg = 0x10e1 ;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00; buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00; buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00; buf[20] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;

	reg = 0x10e1 ;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00; buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00; buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00; buf[20] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;

	reg = 0x10e1 ;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00; buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00; buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00; buf[20] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;

	reg = 0x10e1 ;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00; buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00; buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00; buf[20] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;

	reg = 0x10e1 ;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00; buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00; buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00; buf[20] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;

	reg = 0x10e1 ;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00; buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00; buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00; buf[20] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;

	reg = 0x10e1 ;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00; buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00; buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00; buf[20] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;

	reg = 0x10e1 ;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00; buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00; buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00; buf[20] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;

	reg = 0x10e1 ;
	buf[0] = 0x16; buf[1] = 0x00; buf[2] = 0x2d; buf[3] = 0x00; buf[4] = 0x08; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00; buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00; buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00; buf[20] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;

	reg = 0x10f7 ;
	buf[0] = 0x05; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10f6 ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10e1 ;
	buf[0] = 0x16; buf[1] = 0x00; buf[2] = 0x2d; buf[3] = 0x00; buf[4] = 0x08; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00; buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00; buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00; buf[20] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;

	reg = 0x10f8 ;
	buf[0] = 0x14; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10fa ;
	buf[0] = 0xff; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10f9 ;
	buf[0] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10f9 ;
	buf[0] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11ba ;
	buf[0] = 0x0a; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11bc ;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 4);
	if(ret < 0) goto err;

	reg = 0x11c0 ;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x00; buf[8] = 0x00; buf[9] = 0x00; buf[10] = 0x00; buf[11] = 0x00; buf[12] = 0x00; buf[13] = 0x00; buf[14] = 0x00; buf[15] = 0x00; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00; buf[20] = 0x00; buf[21] = 0x00; buf[22] = 0x00; buf[23] = 0x00; buf[24] = 0x00; buf[25] = 0x00; buf[26] = 0x00; buf[27] = 0x00; buf[28] = 0x00; buf[29] = 0x00; buf[30] = 0x00; buf[31] = 0x00; buf[32] = 0x00; buf[33] = 0x00; buf[34] = 0x00; buf[35] = 0x00; buf[36] = 0x00; buf[37] = 0x00; buf[38] = 0x00; buf[39] = 0x00; buf[40] = 0x00; buf[41] = 0x00; buf[42] = 0x00; buf[43] = 0x00; buf[44] = 0x00; buf[45] = 0x00; buf[46] = 0x00; buf[47] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 48);
	if(ret < 0) goto err;

	reg = 0x118c ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118d ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118e ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118f ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11a5 ;
	buf[0] = 0x2d; buf[1] = 0x2d; buf[2] = 0x3a; buf[3] = 0x05; buf[4] = 0x04; buf[5] = 0x3f; 
	ret = usb_microdia_control_write(dev, reg, buf, 6);
	if(ret < 0) goto err;

	reg = 0x11af ;
	buf[0] = 0x28; buf[1] = 0xd8; buf[2] = 0x14; buf[3] = 0xec; 
	ret = usb_microdia_control_write(dev, reg, buf, 4);
	if(ret < 0) goto err;

	reg = 0x11b3 ;
	buf[0] = 0x32; buf[1] = 0xdd; buf[2] = 0x32; buf[3] = 0xdd; 
	ret = usb_microdia_control_write(dev, reg, buf, 4);
	if(ret < 0) goto err;

	reg = 0x10e0 ;
	buf[0] = 0x47; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1061 ;
	buf[0] = 0x01; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10e0 ;
	buf[0] = 0x67; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1100 ;
	buf[0] = 0x0d; buf[1] = 0x08; buf[2] = 0x08; buf[3] = 0x0d; buf[4] = 0x08; buf[5] = 0x08; buf[6] = 0x0d; buf[7] = 0x0d; buf[8] = 0x0d; buf[9] = 0x0d; buf[10] = 0x11; buf[11] = 0x0d; buf[12] = 0x0d; buf[13] = 0x11; buf[14] = 0x15; buf[15] = 0x21; buf[16] = 0x15; buf[17] = 0x15; buf[18] = 0x11; buf[19] = 0x11; buf[20] = 0x15; buf[21] = 0x2a; buf[22] = 0x1d; buf[23] = 0x1d; buf[24] = 0x19; buf[25] = 0x21; buf[26] = 0x32; buf[27] = 0x2a; buf[28] = 0x32; buf[29] = 0x32; buf[30] = 0x2e; buf[31] = 0x2a; buf[32] = 0x2e; buf[33] = 0x2e; buf[34] = 0x36; buf[35] = 0x3a; buf[36] = 0x4b; buf[37] = 0x43; buf[38] = 0x36; buf[39] = 0x3a; buf[40] = 0x47; buf[41] = 0x3a; buf[42] = 0x2e; buf[43] = 0x2e; buf[44] = 0x43; buf[45] = 0x5c; buf[46] = 0x43; buf[47] = 0x47; buf[48] = 0x4f; buf[49] = 0x53; buf[50] = 0x58; buf[51] = 0x58; buf[52] = 0x58; buf[53] = 0x32; buf[54] = 0x3f; buf[55] = 0x60; buf[56] = 0x64; buf[57] = 0x5c; buf[58] = 0x53; buf[59] = 0x64; buf[60] = 0x4b; buf[61] = 0x53; buf[62] = 0x58; buf[63] = 0x53; 
	ret = usb_microdia_control_write(dev, reg, buf, 64);
	if(ret < 0) goto err;

	reg = 0x1140 ;
	buf[0] = 0x0d; buf[1] = 0x11; buf[2] = 0x11; buf[3] = 0x15; buf[4] = 0x11; buf[5] = 0x15; buf[6] = 0x26; buf[7] = 0x15; buf[8] = 0x15; buf[9] = 0x26; buf[10] = 0x53; buf[11] = 0x36; buf[12] = 0x2e; buf[13] = 0x36; buf[14] = 0x53; buf[15] = 0x53; buf[16] = 0x53; buf[17] = 0x53; buf[18] = 0x53; buf[19] = 0x53; buf[20] = 0x53; buf[21] = 0x53; buf[22] = 0x53; buf[23] = 0x53; buf[24] = 0x53; buf[25] = 0x53; buf[26] = 0x53; buf[27] = 0x53; buf[28] = 0x53; buf[29] = 0x53; buf[30] = 0x53; buf[31] = 0x53; buf[32] = 0x53; buf[33] = 0x53; buf[34] = 0x53; buf[35] = 0x53; buf[36] = 0x53; buf[37] = 0x53; buf[38] = 0x53; buf[39] = 0x53; buf[40] = 0x53; buf[41] = 0x53; buf[42] = 0x53; buf[43] = 0x53; buf[44] = 0x53; buf[45] = 0x53; buf[46] = 0x53; buf[47] = 0x53; buf[48] = 0x53; buf[49] = 0x53; buf[50] = 0x53; buf[51] = 0x53; buf[52] = 0x53; buf[53] = 0x53; buf[54] = 0x53; buf[55] = 0x53; buf[56] = 0x53; buf[57] = 0x53; buf[58] = 0x53; buf[59] = 0x53; buf[60] = 0x53; buf[61] = 0x53; buf[62] = 0x53; buf[63] = 0x53; 
	ret = usb_microdia_control_write(dev, reg, buf, 64);
	if(ret < 0) goto err;

	reg = 0x10e0 ;
	buf[0] = 0x47; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1061 ;
	buf[0] = 0x03; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10e0 ;
	buf[0] = 0x4b; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1189 ;
	buf[0] = 0xd0; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11bc ;
	buf[0] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11bd ;
	buf[0] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11be ;
	buf[0] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11bf ;
	buf[0] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x03; buf[3] = 0x14; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xd0; buf[1] = 0x30; buf[2] = 0x17; buf[3] = 0x1b; buf[4] = 0xbc; buf[5] = 0x01; buf[6] = 0x7a; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x90; buf[1] = 0x30; buf[2] = 0x1b; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x32; buf[3] = 0x82; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1180 ;
	buf[0] = 0x05; buf[1] = 0x00; buf[2] = 0x01; buf[3] = 0x00; buf[4] = 0x50; buf[5] = 0x78; 
	ret = usb_microdia_control_write(dev, reg, buf, 6);
	if(ret < 0) goto err;

	reg = 0x10fb ;
	buf[0] = 0x00; buf[1] = 0xa0; buf[2] = 0x00; buf[3] = 0xf0; buf[4] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 5);
	if(ret < 0) goto err;

	reg = 0x1189 ;
	buf[0] = 0xd0; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11a1 ;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0xd5; buf[3] = 0xa0; 
	ret = usb_microdia_control_write(dev, reg, buf, 4);
	if(ret < 0) goto err;

	reg = 0x11ab ;
	buf[0] = 0x00; buf[1] = 0x00; buf[2] = 0xa0; buf[3] = 0x78; 
	ret = usb_microdia_control_write(dev, reg, buf, 4);
	if(ret < 0) goto err;

	reg = 0x1061 ;
	buf[0] = 0x03; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11ba ;
	buf[0] = 0x0a; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b9 ;
	buf[0] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11ba ;
	buf[0] = 0x0b; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1061 ;
	buf[0] = 0x01; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1000 ;
	buf[0] = 0x78; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1002 ;
	buf[0] = 0x08; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1002 ;
	buf[0] = 0x08; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0xfa; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118a ;
	buf[0] = 0x02; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x0395 ;
	buf[0] = 0x02; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0xf8; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0xf9; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0x7a; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0x7b; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0x7c; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0xfd; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0xfa; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x11; buf[3] = 0x80; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x2a; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x2b; buf[3] = 0x70; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x92; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x93; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x90; buf[1] = 0x30; buf[2] = 0x0a; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x92; buf[1] = 0x30; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c2 ;
	ret = usb_microdia_control_read(dev, reg, buf, 5);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x90; buf[1] = 0x30; buf[2] = 0x0b; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x92; buf[1] = 0x30; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c2 ;
	ret = usb_microdia_control_read(dev, reg, buf, 5);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x90; buf[1] = 0x30; buf[2] = 0x1c; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x92; buf[1] = 0x30; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c2 ;
	ret = usb_microdia_control_read(dev, reg, buf, 5);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x90; buf[1] = 0x30; buf[2] = 0x1d; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0x92; buf[1] = 0x30; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c2 ;
	ret = usb_microdia_control_read(dev, reg, buf, 5);
	if(ret < 0) goto err;

	reg = 0x1061 ;
	buf[0] = 0x03; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1007 ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1006 ;
	buf[0] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10e1 ;
	buf[0] = 0x16; buf[1] = 0x00; buf[2] = 0x2d; buf[3] = 0x00; buf[4] = 0x08; buf[5] = 0x00; buf[6] = 0xdd; buf[7] = 0x0f; buf[8] = 0xcc; buf[9] = 0x0f; buf[10] = 0x58; buf[11] = 0x00; buf[12] = 0x4b; buf[13] = 0x00; buf[14] = 0xb5; buf[15] = 0x0f; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00; buf[20] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0xa1; buf[3] = 0x01; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x10; buf[3] = 0x06; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x04; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x2d; buf[3] = 0x36; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x2e; buf[3] = 0x00; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10e1 ;
	buf[0] = 0x16; buf[1] = 0x00; buf[2] = 0x2d; buf[3] = 0x00; buf[4] = 0x08; buf[5] = 0x00; buf[6] = 0xdd; buf[7] = 0x0f; buf[8] = 0xcc; buf[9] = 0x0f; buf[10] = 0x58; buf[11] = 0x00; buf[12] = 0x4b; buf[13] = 0x00; buf[14] = 0xb5; buf[15] = 0x0f; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00; buf[20] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x00; buf[3] = 0x01; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x15; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10e1 ;
	buf[0] = 0x16; buf[1] = 0x00; buf[2] = 0x2d; buf[3] = 0x00; buf[4] = 0x08; buf[5] = 0x00; buf[6] = 0xdd; buf[7] = 0x0f; buf[8] = 0xcc; buf[9] = 0x0f; buf[10] = 0x58; buf[11] = 0x00; buf[12] = 0x4b; buf[13] = 0x00; buf[14] = 0xb5; buf[15] = 0x0f; buf[16] = 0x00; buf[17] = 0x00; buf[18] = 0x00; buf[19] = 0x00; buf[20] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 21);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x00; buf[3] = 0x06; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x15; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x01; buf[3] = 0x94; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x02; buf[3] = 0x8d; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118c ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118d ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118e ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118f ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x00; buf[3] = 0x09; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x15; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x01; buf[3] = 0x95; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x02; buf[3] = 0x85; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118c ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118d ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118e ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118f ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x00; buf[3] = 0x0c; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x15; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x01; buf[3] = 0x96; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x02; buf[3] = 0x80; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118c ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118d ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118e ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118f ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	buf[0] = 0xa0; buf[1] = 0x30; buf[2] = 0x02; buf[3] = 0x7c; buf[4] = 0x00; buf[5] = 0x00; buf[6] = 0x00; buf[7] = 0x10; 
	ret = usb_microdia_control_write(dev, reg, buf, 8);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x10c0 ;
	ret = usb_microdia_control_read(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118c ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118d ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118e ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x118f ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1007 ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1006 ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;




	/* END OF MICRODIA 6288 START STREAM */

	return 0;
err:
	UDIA_ERROR("r/w in Register %x failed (%d)!\n", reg, ret);
	return ret;
}

/** 
 * @brief From UsbSnoop-plugin-parsed.log
 * 
 * @param dev 
 * 
 * @returns 0 if all OK
 *
 * this function starts video stream Microdia 6128 webcam
 * Bridge SN9C325 + OM6802 CMOS sensor
 * note: comments are my observations so they could be wrong
 */
int microdia_6128_start_stream(struct usb_microdia *dev)
{
	int i;
	int ret = 0;
	__u8 buf[8];

	__u8 data_15[] = {
		0x00, 0x1a, 0x34, 0x27, 0x20, 0xa0, 0x34, 0x69, 0x00, 0x00, 0x00, 0x00, 0x00, 0x03, 0x00
	};

	__u8 data_17[] = {
		0x08, 0x22, 0x44, 0x63, 0x7d, 0x92, 0xa3, 0xaf, 0xbc, 0xc4, 0xcd, 0xd5, 0xdc, 0xe1, 0xe8, 0xef,
		0xf7
	};

	__u8 data_21[] = {
		0x15, 0x00, 0x2b, 0x00, 0x08, 0x00, 0xeb, 0x0f, 0xd7, 0x0f, 0x43, 0x00, 0x3d, 0x00, 0xca, 0x0f, 
		0xf7, 0x0f, 0x0c, 0x00, 0x00
	};

	__u8 qtable_1[] = {
		0x06, 0x04, 0x04, 0x06, 0x04, 0x04, 0x06, 0x06, 0x06, 0x06, 0x08, 0x06, 0x06, 0x08, 0x0a, 0x10, 
		0x0a, 0x0a, 0x08, 0x08, 0x0a, 0x13, 0x0e, 0x0e, 0x0c, 0x10, 0x17, 0x13, 0x17, 0x17, 0x15, 0x13, 
		0x15, 0x15, 0x19, 0x1b, 0x23, 0x1f, 0x19, 0x1b, 0x21, 0x1b, 0x15, 0x15, 0x1f, 0x2b, 0x1f, 0x21, 
		0x25, 0x27, 0x29, 0x29, 0x29, 0x17, 0x1d, 0x2d, 0x2f, 0x2b, 0x27, 0x2f, 0x23, 0x27, 0x29, 0x27
	};


	__u8 qtable_2[] = {
		0x06, 0x08, 0x08, 0x0a, 0x08, 0x0a, 0x12, 0x0a, 0x0a, 0x12, 0x27, 0x19, 0x15, 0x19, 0x27, 0x27,
		0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27,
		0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27,
		0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27, 0x27,
	};

	__u8 om6802_sensor[][8] = {
		{0xa0, dev->sensor_slave_address, 0xd6, 0x02, 0x00, 0x00, 0x00, 0x10},
		{0xa0, dev->sensor_slave_address, 0xae, 0xe2, 0x00, 0x00, 0x00, 0x10},
		{0xa0, dev->sensor_slave_address, 0xdf, 0x6d, 0x00, 0x00, 0x00, 0x10}, 	// Enter factory mode
		{0xa0, dev->sensor_slave_address, 0xdd, 0x18, 0x00, 0x00, 0x00, 0x10},	// RGB, uyvy1,Raw RGB, n/1/2, n/clp
		{0xa0, dev->sensor_slave_address, 0x5a, 0xc0, 0x00, 0x00, 0x00, 0x10},
		{0xa0, dev->sensor_slave_address, 0xfb, 0x11, 0x00, 0x00, 0x00, 0x10},	
		{0xa0, dev->sensor_slave_address, 0xf0, 0x03, 0x00, 0x00, 0x00, 0x10},
		{0xa0, dev->sensor_slave_address, 0xe9, 0x00, 0x00, 0x00, 0x00, 0x10},
		{0xa0, dev->sensor_slave_address, 0xe4, 0xff, 0x00, 0x00, 0x00, 0x10},
		{0xa0, dev->sensor_slave_address, 0xfb, 0xee, 0x00, 0x00, 0x00, 0x10},
		{0xa0, dev->sensor_slave_address, 0x5d, 0x80, 0x00, 0x00, 0x00, 0x10},
		{0xa0, dev->sensor_slave_address, 0x71, 0x84, 0x00, 0x00, 0x00, 0x10},
		{0xa0, dev->sensor_slave_address, 0x72, 0x00, 0x00, 0x00, 0x00, 0x10},
		{0xa0, dev->sensor_slave_address, 0x68, 0x80, 0x00, 0x00, 0x00, 0x10},
		{0xa0, dev->sensor_slave_address, 0x69, 0x00, 0x00, 0x00, 0x00, 0x10},
		{0xa0, dev->sensor_slave_address, 0x71, 0x18, 0x00, 0x00, 0x00, 0x10},
		{0xa0, dev->sensor_slave_address, 0x72, 0x04, 0x00, 0x00, 0x00, 0x10},
		{0xa0, dev->sensor_slave_address, 0x68, 0xde, 0x00, 0x00, 0x00, 0x10},
		{0xa0, dev->sensor_slave_address, 0x69, 0x00, 0x00, 0x00, 0x00, 0x10},	// 18
		{0xa0, dev->sensor_slave_address, 0x68, 0x80, 0x00, 0x00, 0x00, 0x10},
		{0xa0, dev->sensor_slave_address, 0x69, 0x00, 0x00, 0x00, 0x00, 0x10},
		{0, 0, 0, 0, 0, 0, 0, 0}	// Terminating string
	};

	buf[0] = 0x00;
	ret += usb_microdia_control_write(dev, 0xf1, buf, 1);	// Reset cam
	
	buf[0] = 0x23;
	ret += usb_microdia_control_write(dev, 0x01, buf, 1);	// Sensor off, video off, clk 12MHz
	
	ret += usb_microdia_control_read(dev, 0x00, buf, 1);	// Check if cam is working, must write 0x12 in buf[0]
	
	if(buf[0] != 0x12){
		UDIA_ERROR("Microdia 6128 webcam not connected or working !!!\n");
		return -ENODEV;
	}
	
	ret += usb_microdia_control_read(dev, 0x01, buf, 1);	// Must read 23
	
	if(buf[0] != 0x23){
		UDIA_ERROR("Microdia 6128 webcam is not initialized !!!\n");
		return -ENODEV;
	}

	buf[0] = 0x23; buf[1] = 0x72;
	ret += usb_microdia_control_write(dev, 0x01, buf, 2);	// Configure GPIO

	buf[0] = 0x34; buf[1] = 0x27; buf[2] = 0x20; buf[3] = 0xa0; buf[4] = 0x34;
	ret += usb_microdia_control_write(dev, 0x05, buf, 5);

	buf[0] = 0x44; buf[1] = 0x45; buf[2] = 0x00; buf[3] = 0x00; buf[4] = 0x00;
	ret += usb_microdia_control_write(dev, 0x17, buf, 5);

	buf[0] = 0x08; buf[1] = 0x40; buf[2] = 0x20; buf[3] = 0x00; buf[4] = 0x00;
	ret += usb_microdia_control_write(dev, 0x9a, buf, 5);	

	buf[0] = 0x60; buf[1] = 0x00; buf[2] = 0x00;
	ret += usb_microdia_control_write(dev, 0xd4, buf, 3);
	ret += usb_microdia_control_write(dev, 0x03, data_15, 15);

	buf[0] = 0x73;
	ret += usb_microdia_control_write(dev, 0x02, buf, 1);

	buf[0] = 0x64;
	ret += usb_microdia_control_write(dev, 0x17, buf, 1);

	buf[0] = 0x22; buf[1] = 0x62;
	ret += usb_microdia_control_write(dev, 0x01, &buf[0], 1);
	ret += usb_microdia_control_write(dev, 0x01, &buf[1], 1);

	buf[0] = 0x64;
	ret += usb_microdia_control_write(dev, 0x17, buf, 1);
	ret += usb_microdia_control_write(dev, 0x17, buf, 1);

	buf[0] = 0x42;		// Sensor on, clk 24MHz, video off
	ret += usb_microdia_control_write(dev, 0x01, buf, 1);
	ret += usb_microdia_control_write(dev, 0x01, buf, 1);

	// Sensor stuff
	for (i = 0; i < 2; i++){
		ret += usb_microdia_control_write(dev, 0x08, om6802_sensor[i], 8); 
		msleep(1);
	}	

	buf[0] = 0x71;
	ret += usb_microdia_control_write(dev, 0x02, buf, 1);

	// Sensor stuff
	for (i = 2; i < 11; i++){
		ret += usb_microdia_control_write(dev, 0x08, om6802_sensor[i], 8); 
		msleep(1);
	}

	buf[0] = 0x71;
	ret += usb_microdia_control_write(dev, 0x02, buf, 1);
	ret += usb_microdia_control_write(dev, 0x02, buf, 1);		// URB 1780

	buf[0] = 0x28;
	ret += usb_microdia_control_write(dev, 0x15, buf, 1);

	buf[0] = 0x1e;
	ret += usb_microdia_control_write(dev, 0x16, buf, 1);

	buf[0] = 0x51;
	ret += usb_microdia_control_write(dev, 0x12, buf, 1);

	buf[0] = 0x01;
	ret += usb_microdia_control_write(dev, 0x13, buf, 1);

	buf[0] = 0x45;
	ret += usb_microdia_control_write(dev, 0x18, buf, 1);

	buf[0] = 0x6a;
	ret += usb_microdia_control_write(dev, 0xd2, buf, 1);

	buf[0] = 0x50;
	ret += usb_microdia_control_write(dev, 0xd3, buf, 1);

	buf[0] = 0x00;
	ret += usb_microdia_control_write(dev, 0xc6, buf, 1);
	ret += usb_microdia_control_write(dev, 0xc7, buf, 1);

	buf[0] = 0x50;
	ret += usb_microdia_control_write(dev, 0xc8, buf, 1);

	buf[0] = 0x3c;
	ret += usb_microdia_control_write(dev, 0xc9, buf, 1);

	buf[0] = 0x45;
	ret += usb_microdia_control_write(dev, 0x18, buf, 1);

	buf[0] = 0x64;
	ret += usb_microdia_control_write(dev, 0x17, buf, 1);

	buf[0] = 0x20;
	ret += usb_microdia_control_write(dev, 0x05, buf, 1);
	ret += usb_microdia_control_write(dev, 0x07, buf, 1);
	ret += usb_microdia_control_write(dev, 0x06, buf, 1);

	buf[0] = 0x00;
	ret += usb_microdia_control_write(dev, 0x14, buf, 1);

	ret += usb_microdia_control_write(dev, 0x20, data_17, 17);
	ret += usb_microdia_control_write(dev, 0x20, data_17, 17);  

	for (i = 0; i < 8; ++i)	// writing the same buffer 8 times, why ??
		ret += usb_microdia_control_write(dev, 0x84, data_21, 21);  

	data_21[0] = 0x16; data_21[10] = 40; data_21[12] = 0x3f;
	ret += usb_microdia_control_write(dev, 0x84, data_21, 21);

	buf[0] = 0x08;
	ret += usb_microdia_control_write(dev, 0x9a, buf, 1);

	buf[0] = 0x10;
	ret += usb_microdia_control_write(dev, 0x99, buf, 1);

	ret += usb_microdia_control_write(dev, 0x84, data_21, 21);	// URB 1811

	buf[0] = 0x20;
	ret += usb_microdia_control_write(dev, 0x05, buf, 1);
	ret += usb_microdia_control_write(dev, 0x07, buf, 1);
	ret += usb_microdia_control_write(dev, 0x06, buf, 1);  

	// Sensor Stuff
	for (i = 11; i < 15; i++){
		ret += usb_microdia_control_write(dev, 0x08, om6802_sensor[i], 8);
		msleep(1);
	}

	buf[0] = 0x34;
	ret += usb_microdia_control_write(dev, 0x05, buf, 1);
	buf[0] = 0x27;
	ret += usb_microdia_control_write(dev, 0x06, buf, 1);

	buf[0] = buf[1] = 0x2d; buf[2] = 0x3a; buf[3] = 0x05; buf[4] = 0x04; buf[5] = 0x3f;
	ret += usb_microdia_control_write(dev, 0xc0, buf, 6);

	buf[0] = 0x28; buf[1] = 0xd8; buf[2] = 0x14; buf[3] = 0xec;
	ret += usb_microdia_control_write(dev, 0xca, buf, 4);

	buf[0] = 0x1e; buf[1] = 0xdd; buf[2] = 0x2d; buf[3] = 0xe7;
	ret += usb_microdia_control_write(dev, 0xce, buf, 4);

	buf[0] = 0x42;
	ret += usb_microdia_control_write(dev, 0x01, buf, 1);

	buf[0] = 0x64;
	ret += usb_microdia_control_write(dev, 0x17, buf, 1);

	// Sensor Stuff
	for (i = 15; i < 19; i++){
		ret += usb_microdia_control_write(dev, 0x08, om6802_sensor[i], 8);
		msleep(1);
	}

	buf[0] = 0x28;
	ret += usb_microdia_control_write(dev, 0x05, buf, 1);

	buf[0] = 0x54;
	ret += usb_microdia_control_write(dev, 0x06, buf, 1);

	// This changes the mode (VGA = 0 , CIF = 1)
	// buf = 0x40 | 0x05 | (mode << 4) 45 or 55
	buf[0] = 0x45;
	ret += usb_microdia_control_write(dev, 0x18, buf, 1);

	// This is the Q table
	ret += usb_microdia_control_write(dev, 0x0100, qtable_1, 64); // first time
	ret += usb_microdia_control_write(dev, 0x0140, qtable_2, 64); // second time

	// mode: VGA = 0, CIF = 1
	// buf = 0x05 | (mode << 4) 05 or 15
	buf[0] = 0x05;
	ret += usb_microdia_control_write(dev, 0x18, buf, 1);

	buf[0] = 0x71;
	ret += usb_microdia_control_write(dev, 0x02, buf, 1);
	ret += usb_microdia_control_write(dev, 0x02, buf, 1);

	buf[0] = 0x42;    // Sensor on, Video disabled, clk 24MHz
	ret += usb_microdia_control_write(dev, 0x01, buf, 1);

	buf[0] = 0x64;
	ret += usb_microdia_control_write(dev, 0x17, buf, 1);

	// Command to start streaming
	// Sensor on, clk 24MHz, video enabled
	buf[0] = 0x46;    
	ret += usb_microdia_control_write(dev, 0x01, buf, 1);
  
	// When the device reads 46 from reg 0x01 it means that 
	// camera is streaming
	ret += usb_microdia_control_read(dev, 0x01, buf, 1);	// Must read 0x46

	if (ret != 624)
		UDIA_DEBUG("Some read or write were not successful in Start Stream\n");

	return 0;
}


int dev_microdia_stop_stream(struct usb_microdia *dev)
{
	int ret = -ENODEV;
	if(dev && dev->stop_stream)
		ret = dev->stop_stream(dev);

	return ret;
}

/**
 * @brief From stop.htm
 *
 * @param dev
 *
 * @returns 0
 *
 * @author GWater
 *
 * For SN9C201 with MI1310.
 * This function has not been tested yet.
 */
int microdia_6242_stop_stream(struct usb_microdia *dev)
{
	__u8 buf[3];

	__u8 only10c0[3] = {0x0d, 0x00, 0x00};

	buf[0] = 0x01;
	usb_microdia_control_write(dev, 0x1061, buf, 1);

	buf[0] = 0x60;
	usb_microdia_control_write(dev, 0x1007, buf, 1);

	buf[0] = 0x00;
	usb_microdia_control_write(dev, 0x1006, buf, 1);

	sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2,
		only10c0[0], dev->sensor_flags, &only10c0[1]);

	buf[0] = 0x38;
	usb_microdia_control_write(dev, 0x1000, buf, 1);

	buf[0] = 0x6b;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);

	buf[0] = 0x60;
	usb_microdia_control_write(dev, 0x1007, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x1006, buf, 1);


	buf[0] = 0x98; buf[1] = 0xe7; buf[2] = 0x11;
	usb_microdia_control_write(dev, 0x1000, buf, 3);

	/**
	 * After the select alternate setting 0 is actually another transfer:
	 *	buf[0] = 0x00;
	 *	usb_microdia_control_write(dev, 0x1066, buf, 1);
	 */

	return 0;
}

/** 
 * @brief From stopstream.log
 * 
 * @param dev
 * 
 * @returns 0
 *
 * @author GWater
 *
 * For SN9C201 with SOI968.
 * I don't whether the function really stops the stream.
 * Nevertheless the LED on the webcam now stops glowing.
 * The function is very similar to "microdia_624f_stop_stream".
 */
int microdia_624e_stop_stream(struct usb_microdia *dev)
{
	__u8 buf[3];

	buf[0] = 0x01;
	usb_microdia_control_write(dev, 0x1061, buf, 1);

	/* Next two writes added because it works better.
	 * The LED stops glowing and I think the stream does too.
	 * 0x20 = 0b00100000 which is the minimal of what the other stopstreams have in common.
	 */
	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x1007, buf, 1);

	buf[0] = 0x20;
	usb_microdia_control_write(dev, 0x1006, buf, 1);

	buf[0] = 0x3c;
	usb_microdia_control_write(dev, 0x1000, buf, 1);

	buf[0] = 0x6a;
	usb_microdia_control_write(dev, 0x11b8, buf, 1);

	buf[0] = 0x38; buf[1] = 0xe7; buf[2] = 0x11;
	usb_microdia_control_write(dev, 0x1000, buf, 3);

	usb_microdia_control_read(dev, 0x1066, buf, 1);

	return 0;
}

int microdia_624f_stop_stream(struct usb_microdia *dev)
{
	int ret;
	__u16 command;
	__u8 buf[3];

	buf[0] = 0x02;
	command = 0x1066;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	buf[0] = 0x01;
	command = 0x1061;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	buf[0] = 0x38;
	command = 0x1000;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	buf[0] = 0x6a;
	command = 0x11b8;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	buf[0] = 0x20;
	command = 0x1007;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	buf[0] = 0x20;
	command = 0x1006;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	buf[0] = 0x98;
	buf[1] = 0xe7;
	buf[2] = 0x11;
	command = 0x1000;
	ret = usb_microdia_control_write(dev, command, buf, 3);
	if(ret < 0) goto err;

	command = 0x1045;
        ret = usb_microdia_control_read(dev, command, buf, 1);
	if(ret < 0) goto err;

	buf[0] = 0x0f;
	ret = usb_microdia_control_write(dev, command, buf, 1);
	if(ret < 0) goto err;

	return ret;

err:
	UDIA_ERROR("command %x failed (%d)!\n", command, ret);
	return ret;
}

int microdia_6260_stop_stream(struct usb_microdia *dev)
{
	int ret;
	__u8 buf[32];
	
	buf[0] = 0x01;
	ret = usb_microdia_control_write(dev, 0x1061, buf, 1);
	buf[0] = 0x38;
	ret = usb_microdia_control_write(dev, 0x1000, buf, 1);
	buf[0] = 0x6b;
	ret = usb_microdia_control_write(dev, 0x11b8, buf, 1);
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, 0x1007, buf, 1);
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, 0x1006, buf, 1);
	buf[0] = 0x98;
	buf[1] = 0xe7;
	buf[2] = 0x11;
	ret = usb_microdia_control_write(dev, 0x1000, buf, 3);
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, 0x1066, buf, 1);
	ret = usb_microdia_control_read(dev, 0x1045, buf, 1);
	buf[0] = 0x0f;
	ret = usb_microdia_control_write(dev, 0x1045, buf, 1);
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, 0x10c8, buf, 1);
	buf[0] = 0x80;
	ret = usb_microdia_control_write(dev, 0x1020, buf, 1);

	return ret;
}

/**
 * @brief From 7 start-stop-logs (made by Alex Carlos, Boris Borisov, bohzan and Comer352l)
 *
 * @param dev
 *
 * @returns 0 (ok) or -1 (error)
 *
 * @author Comer352l
 *
 * Windows driver version: 5.7.23.000
 * Windows versions: 2000 and XP
 * USB-Logger: SniffUSB 1.8 and 2.0, USBTrace 2.0, SnoopyPro
 * All logs were made using AMCAP with 640x480, RGB24
 */
int microdia_6270_stop_stream(struct usb_microdia *dev)
{
	int ret = 0;
	int retI2C = 0;
	__u8 buf[3];

	if (dev->sensor_slave_address == MT9V111_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x20;
		ret += usb_microdia_control_write(dev, 0x1007, buf, 1);		// URB 3541
		buf[0] = 0x20;
		ret += usb_microdia_control_write(dev, 0x1006, buf, 1);		// URB 3542
		// => These two writes seem to cause the cam to stop sending isochronus USB messages
	}
	buf[0] = 0x01;
	ret += usb_microdia_control_write(dev, 0x1061, buf, 1);		// URB 3544
	if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		buf[0] = 0x60;
		ret += usb_microdia_control_write(dev, 0x1007, buf, 1);		// URB 3541
		buf[0] = 0x60;
		ret += usb_microdia_control_write(dev, 0x1006, buf, 1);		// URB 3542
	}
	if (dev->sensor_slave_address == MT9V011_I2C_SLAVE_ADDRESS)
	{
		// Write to sensor register	0x07: Output Control
		buf[0] = 0x00; buf[1] = 0x00;	// stop sensor readout, normal operation
		retI2C += sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 2, 0x07, dev->sensor_flags, buf);
	}
	buf[0] = 0x38;		// ALEX: 0x3c
	ret += usb_microdia_control_write(dev, 0x1000, buf, 1);		// URB 3545
	buf[0] = 0x6d;		// ALEX: 0xec; FRANK: also 0x6c
	ret += usb_microdia_control_write(dev, 0x11b8, buf, 1);		// URB 3546
	buf[0] = 0x38; buf[1] = 0xe7; buf[2] = 0x11;
	ret += usb_microdia_control_write(dev, 0x1000, buf, 3);		// URB 3547

	// INTERRUPT COMMING BACK (URB 2227)
	// SELECT ALTERNATE SETTINGS (URB 3548)
	// INTERRUPT GOING DOWN (URB 3549)

	buf[0] = 0x00;
	ret += usb_microdia_control_write(dev, 0x1066, buf, 1);		// URB 3550

	if (ret < 9)
	{
		UDIA_INFO("One ore more errors occured while stopping stream !\n");
		return -1;
	}
	else
		return 0;
}

int microdia_627b_stop_stream(struct usb_microdia *dev)
{
	int ret = 0;
	__u16 reg;
	__u8 buf[3];

	reg = 0x1007;
	buf[0] = 0x60;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1006;
	buf[0] = 0x20;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	//reg = 0x1007;
	//buf[0] = 0x60;
	//ret = usb_microdia_control_write(dev, reg, buf, 1);
	//if(ret < 0) goto err;

	//reg = 0x1006;
	//buf[0] = 0x20;
	//ret = usb_microdia_control_write(dev, reg, buf, 1);
	//if(ret < 0) goto err;

	reg = 0x1061;
	buf[0] = 0x01;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	//reg = 0x1007;
	//buf[0] = 0x60;
	//ret = usb_microdia_control_write(dev, reg, buf, 1);
	//if(ret < 0) goto err;

	//reg = 0x1006;
	//buf[0] = 0x20;
	//ret = usb_microdia_control_write(dev, reg, buf, 1);
	//if(ret < 0) goto err;

	reg = 0x1000;
	buf[0] = 0x38;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8;
	buf[0] = 0x6b;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1000;
	buf[0] = 0x38; buf[1] = 0xe7; buf[2] = 0x11;
	ret = usb_microdia_control_write(dev, reg, buf, 3);
	if(ret < 0) goto err;

	reg = 0x1066;
	buf[0] = 0x00;
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	return ret;
err:
	UDIA_ERROR("r/w in Register %x failed (%d)!\n", reg, ret);
	return ret;
}

/** 
 * @brief From UsbSnoop-plugin-parsed.log
 * 
 * @param dev 
 * 
 * @returns 0 (OK) or <0 (Error)
 *
 * @author Vincent, Kuzja
 */
int microdia_6288_stop_stream(struct usb_microdia *dev)
{
	int ret = 0;
	__u16 reg;
	__u8 buf[8];

	reg = 0x1061 ;
	buf[0] = 0x01; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1000 ;
	buf[0] = 0x38; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x11b8 ;
	buf[0] = 0xe9; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1007 ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1006 ;
	buf[0] = 0x20; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	reg = 0x1000 ;
	buf[0] = 0x98; buf[1] = 0xe7; buf[2] = 0x11; 
	ret = usb_microdia_control_write(dev, reg, buf, 3);
	if(ret < 0) goto err;

	reg = 0x1066 ;
	buf[0] = 0x00; 
	ret = usb_microdia_control_write(dev, reg, buf, 1);
	if(ret < 0) goto err;

	return 0;
err:
	UDIA_ERROR("r/w in Register %x failed (%d)!\n", reg, ret);
	return ret;
}

/** 
 * @brief From UsbSnoop-plugin-parsed.log
 * 
 * @param dev 
 * 
 * @returns 0 if all OK
 *
 * This function stops the video stream for microdia 6128
 * Bridge SN9C325 + OM6802 CMOS sensor
 * note: comments are my observations so they could be wrong
 */
int microdia_6128_stop_stream(struct usb_microdia *dev)
{
	int ret = 0;
	__u8 data;

	ret += usb_microdia_control_read(dev, 0x00, &data, 1);
	
	// Stop stream command
	data = 0x00;
	ret += usb_microdia_control_write(dev, 0x01, &data, 1);

	data = 0x00;
	ret += usb_microdia_control_write(dev, 0xf1, &data, 1);

	data = 0x01;
	ret += usb_microdia_control_write(dev, 0xf1, &data, 1);

	if(ret != 4)
		UDIA_DEBUG("Some read or write were not successful in Stop Stream\n");

	return 0;
}



int microdia_624f_set_exposure(struct usb_microdia *dev)
{
	int ret = 0;
	__u8 v1 = (dev->vsettings.exposure >> 4) & 0xff;
	__u8 v2 = dev->vsettings.exposure >> 12;

	ret |= sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, 0x2d,
		dev->sensor_flags, &v1);
	
	ret |= sn9c20x_write_i2c_data(dev, dev->sensor_slave_address, 1, 0x2e,
		dev->sensor_flags, &v2);

	return ret;
}

int microdia_624f_flip_detect(struct usb_microdia *dev)
{
	int ret = 0;
	__u8 val;

	ret = usb_microdia_control_read(dev, 0x1009, &val, 1);
	if (ret < 0) return -EAGAIN;
	if (val & 0x01) {
		dev->vsettings.vflip = 1;
	}
	else {
		dev->vsettings.vflip = 0;
	}

	return ret;
}

int microdia_6260_flip_detect(struct usb_microdia *dev)
{
	const __u8 flip_bit = 0x01;
	int ret = 0;
	__u8 val;
	static __u8 flip_reg = flip_bit;
	__u8 vflip;

	ret = usb_microdia_control_read(dev, 0x1009, &val, 1);
	if (ret < 0)
		return -EAGAIN;
	if (flip_reg != (val & flip_bit)) {
		if (val & flip_bit)
			vflip = 0;
		else
			vflip = 1;
		ret = ov7670_auto_flip(dev, vflip);
		flip_reg = (val & flip_bit);
	}

	return ret;
}
